#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2006 Koninklijke Philips Electronics N.V., 
# All Rights Reserved. 
#
# Filename: etc/Kbuild.mk
#
#
# Rev Date       Author      Comments
#-------------------------------------------------------------------------------
#   1 20060301   batelaan    Initial
#   2 20060302   batelaan    Use EXTRA_FLAGS iso CFLAGS_<objfile> cos EXTRA_FLAGS is not
#                            global to whole kernel build, just for the module itself.
#                            Then we can use .c files from subdirs in a Kbuild file.
#   3 20060303   steel       Add stdio.h support
#   4 20060303   steel       Add string.h support
#   5 20060303   burningh    Add <comp>/cfg to default includes
#   6 20060303   laird       Adding -DEMBEDDED
#   7 20060405   burningh    Adding -DDV_DEBUG
#   8 20060420   burningh    Adding -D_MSVD_STANDARD_TYPES_
#   9 20060428   batelaan    -I switches: removed resources/inc, and added tmgdrTypes/inc
#  10 20060511   batelaan    Add $_TMTOOLCHAIN in paths.
#  11 20060512   iwillis     Added -DTMFL_IC_PNX8335 and -DIC_EMULATION=true
#  12 20060512   iwillis     Replaced -DIC_EMULATION=true with -DTMFL_IC_EMULATION=true
#  13 20070905   burningh    Remove use of phStbCommon
#
# File: Kbuild.mk
# Include makefile for Linux kernel modules Kbuild files.
#
# See: <Kbuild_TEMPLATE.mk>

ifndef _KBUILD_MK_INCLUDED
_KBUILD_MK_INCLUDED=1

# Function: CHECK_VAR_DEFINED
# Checks if variable name given by first argument is defined. 
# Prints error message and exits make if not.
#
# Error message:
# Variable $(1) not set: $(2)
CHECK_VAR_DEFINED=$(eval $(call CHECK_VAR_DEFINED1,$(1),$(2)))
define CHECK_VAR_DEFINED1
  ifndef $(1)
    $$(error Variable $(1) not set: $(2))
  endif
endef

# Variable: TRACE
# If defined and non-empty, then makefile tracing is enabled using <DOTRACE>.
# Expands to an empty string, to can be called anywhere.
# Usage:
# shell> make TRACE=1 ...
TRACE=

# Function: DOTRACE
# Makefile tracing. If <TRACE> is non-empty, the argument will be printed
# to stderr, using the echo command. Messages are automatically indented by
# their makefile include nesting (see <INCLUDE_MAKEFILE>).
#
# Example:
# $(call DOTRACE,'message....')
DOTRACE = $(if $(TRACE),$(shell echo '$(INDENT)'$(1) 1>&2))

SDE_NOT_SETUP_MSG := ensure SDE2 build variables are set (eg use cbuild :mt)
$(call CHECK_VAR_DEFINED,_TMTGTCPUCLASS,$(SDE_NOT_SETUP_MSG))
$(call CHECK_VAR_DEFINED,_TMTGTCPUTYPE,$(SDE_NOT_SETUP_MSG))
$(call CHECK_VAR_DEFINED,_TMTGTENDIAN,$(SDE_NOT_SETUP_MSG))
$(call CHECK_VAR_DEFINED,_TMTGTOS,$(SDE_NOT_SETUP_MSG))
$(call CHECK_VAR_DEFINED,_TMTGTOSCLASS,$(SDE_NOT_SETUP_MSG))

# Obtain list of component locations:
include $(_TMTGTBUILDROOT)/project/loc_list.mk

# Function: SDE_REQUIRES
# Declares dependencies on other component, similar to SDE2 REQUIRES variable.
# Sets up CFLAGS for object files specified.
#
# Usage: $(call SDE_REQUIRES,tmXyz,objfiles,SDE2 component names...)
SDE_REQUIRES=$(eval $(call SDE_REQUIRES1,$(strip $(1)),$(strip $(2)),$(strip $(3))))
define SDE_REQUIRES1
    $(eval EXTRA_CFLAGS := \
      $(if $(CONFIG_PHILIPS_PNX8335),-DTMFL_IC_PNX8335) \
      $(if $(CONFIG_IC_EMULATION),-DTMFL_IC_EMULATION=true) \
      -DEMBEDDED \
      -DDV_DEBUG \
      -D_MSVD_STANDARD_TYPES_ \
      -I$(_$(1)_DIR)/inc \
      -I$(_$(1)_DIR)/cfg \
      -I$(_TMROOT)/inc \
      -I$(_TMROOT)/inc/cfg \
      -I$(_TMROOT)/inc/$(_TMTGTCPUCLASS)_$(_TMTGTOS) \
      -I$(_TMTGTBUILDROOT)/comps/generated/lib/$(_TMTGTCPUCLASS)$(_TMTOOLCHAIN)_$(_TMTGTOS)_$(_TMTGTENDIAN)_$(_TMTGTCPUTYPE) \
      $(foreach r,$(3),-I$(_$(r)_DIR)/inc) \
      -I$(_tmgdrTypes_DIR)/inc \
      -I$(KSRC)/include \
    )
endef
#  $(shell make -C $(_$(1)_DIR) _sde_print_var_5 var=REQUIRES var2=C_SOURCES)
#  $(call DOTRACE, REQUIRES = $(REQUIRES))
#    $(call DOTRACE,'CFLAGS_$(f) = $(CFLAGS_$(f))')

endif # ifndef _KBUILD_MK_INCLUDED
