#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2009 NXP B.V.,
# All Rights Reserved.
#
# Filename: etc/application_first.mk
#
#
# Rev Date       Author      Comments
#-------------------------------------------------------------------------------
#   1 20050824   batelaan    Original
#   2 20050826   burningh    Correcting $(TARGET)gcc issue
#   3 20051013   neill       Allow for additional install targets
#   4 20051104   batelaan    Ensure build output goes below $BUILDROOT/apps/$(PROGRAM)
#   5 20060331   laird       Adapt for Montavista mips2_fp_be
#   6 20060804   batelaan    Fix .c->.o build rule (use $< instead of $^).
#                            Ensure 'all' is the first target.
#                            Use CPUBUILDROOT instead of BUILDROOT.
#   7 20060829   batelaan    Remove dependency on MontaVista
#   8 20061206   steel       change /opt/philips to /opt/nxp
#   9 20061207   neill       use _TMTGTPREFIX
#  10 20070103   batelaan    Add -I switch to find SDE2 generated tmFlags.mk file.
#  11 20070606   barnish     Use _smallram_ diversity to select optimisation level
#  12 20070606   barnish     rework
#  13 20070607   steel       add cpu optimisations
#  14 20070824   batelaan    Doc update.
#  15 20070903   batelaan    If only clean target given don't include loc_list.mk.
#  16 20070912   batelaan    Add -I$(BUILDROOT) to allow include of prjconfig.h
#  17 20071016   steel       add -msoft-float when available
#17.1.1 20071214 laird       Adding Large File Support.
#17.1.2 20080128 laird       Adding $STAGINGDIR/lib to lib search path.
#  18 20071105   batelaan    Add -g if _TMTGTREL is set to debug.
#  19 20080201   batelaan    Merge.
#  20 20080218   batelaan    Add -Wextra
#  21 20080218   batelaan    Enable dynamic linking
#  22 20080220   batelaan    Print input file for CC or GCC
#  23 20080319   laird       Add setting of optimization level and debug level.
#  24 20080707   batelaan    Merge with no changes, for CM purposes.
#  25 20080709   batelaan    Add header file dependencies.
#  26 20080815   batelaan    Include .prjconfig
#  27 20081017   batelaan    Add -MF ... to -MD to solve ccache bug
#  28 20081103   batelaan    Add -MT $@ to -MD to solve ccache bug
#  29 20081105   amccurdy    Add support for S_SOURCES
#  30 20081218   laird       Setup include paths properly.
#  31 20090116   laird       Use sysroot properly.
#  32 20090204   batelaan    Fix c++ support
#  33 20090216   amccurdy    Fix typo.
#  34 20090330   batelaan    Link against STAGINGDIR.
#  35 20090403   ocourson    Remove link against STAGINGDIR.

# File: application_first.mk
# Include makefile for main applications.
# All generated output goes into subdirectory build of the application.
#
# Targets are:-
# - clean: removes the build directory <BUILDDIR>.
# - make: builds <BUILDDIR>/<PROGRAM> from <C_SOURCES> using <CFLAGS>.
# - install: installs <BUILDDIR>/<PROGRAM> in <INSTALL_DIR>.
#
# Conditions before including this makefile:-
# - <PROGRAM> should be defined.
# - <ROOTFS> should be defined.
#
# Includes:
# - ${<BUILDROOT>}/.prjconfig
#
# See: <application_last.mk> <LINUX_EXAMPLE_APP_makefile_TEMPLATE.mk>

# Section: Application Makefile

# Default (first) target:
all:

include $(BUILDROOT)/.prjconfig

ifneq "$(MAKECMDGOALS)" "clean"
  include $(_TMTGTBUILDROOT)/project/loc_list.mk
endif

# Variable: ROOTFS
# Root file system below which the application will be installed.
# Must be defined before including application_first.mk.
#
# See: <INSTALL_DIR>
ifndef ROOTFS
  $(error ROOTFS not defined)
endif

# Variable: CPUBUILDROOT
# Location below which all applications are built.
# Must be defined before including application_first.mk.
#
# See: <BUILDDIR>
ifndef CPUBUILDROOT
  $(error CPUBUILDROOT not defined)
endif

# Define: PROGRAM
# Name of the application to build.
# Must be defined before including application_first.mk.
ifndef PROGRAM
  $(error PROGRAM not defined)
endif

# Define: BUILDDIR
# Directory with the build output for <PROGRAM>. Normally set to
# ${<CPUBUILDROOT>}/apps/${<PROGRAM>}
BUILDDIR := $(CPUBUILDROOT)/apps/$(PROGRAM)

# Define: TARGET
# Cross compilation target. Defaults to ${<CROSS_COMPILE>} without the trailing "-".
TARGET ?= $(patsubst %-,%,$(CROSS_COMPILE))

# Define: CC
# C compiler. Defaults to ${<CROSS_COMPILE>}gcc
CC = $(CROSS_COMPILE)gcc

# Define: CXX
# C++ compiler. Defaults to ${<CROSS_COMPILE>}g++
CXX = $(CROSS_COMPILE)g++

# Define: _TMTGTREL
# Type of build. Defaults to trace. Possible values:-
# - debug
# - trace
# - assert
# - retail
ifndef _TMTGTREL
  $(error _TMTGTREL not defined. Values are: debug trace assert retail)
endif

# Define: TMFL_REL
# C preprocessor flag defined as one of these values (must #include tmFlags.mk):-
# - TMFL_REL_DEBUG
# - TMFL_REL_TRACE
# - TMFL_REL_ASSERT
# - TMFL_REL_RETAIL
TMFL_REL := TMFL_REL_$(shell echo $(_TMTGTREL) | tr '[a-z]' '[A-Z]')

# Define: PREFIXES
# List of directories from which libraries and include paths are derived.
# E.g. /usr/local, <_TMTGTPREFIX>.
# Defaults to <_TMTGTPREFIX> and /usr.
#
# See: <LOCAL_PREFIXES>
PREFIXES = \
	$(LOCAL_PREFIXES) \
	$(_TMTGTPREFIX) \
	/ \
	/usr

# Define: LOCAL_PREFIXES
# Appended to <PREFIXES>

# Define: OPTIMISATION
# The CC flag for the selected optimisation level, added to <COMMON_CFLAGS>.
ifeq ($(CONFIG_TOOLCHAIN_OPTIMIZE_0),y)
OPTIMISATION=-O0
endif
ifeq ($(CONFIG_TOOLCHAIN_OPTIMIZE_1),y)
OPTIMISATION=-O1
endif
ifeq ($(CONFIG_TOOLCHAIN_OPTIMIZE_2),y)
OPTIMISATION=-O2
endif
ifeq ($(CONFIG_TOOLCHAIN_OPTIMIZE_3),y)
OPTIMISATION=-O3
endif
ifeq ($(CONFIG_TOOLCHAIN_OPTIMIZE_S),y)
OPTIMISATION=-Os
endif

# Define: DEBUGGING
# The CC flag for the selected debug level, added to <COMMON_CFLAGS>.
ifeq ($(CONFIG_TOOLCHAIN_DEBUG_1),y)
DEBUGGING=-g1
endif
ifeq ($(CONFIG_TOOLCHAIN_DEBUG_2),y)
DEBUGGING=-g2
endif
ifeq ($(CONFIG_TOOLCHAIN_DEBUG_3),y)
DEBUGGING=-g3
endif

# Define: SYSROOT_OPTION
# The sysroot to use if the compiler was built with --sysroot.
# Is the compiler built with sysroot support and whats its value?
ifneq "$(CROSS_COMPILE)" ""
  SYSROOT_ENABLED:=$(shell $(CC) -v 2>&1 | grep -- --with-sysroot)
  SYSROOT_OPTION=$(if $(SYSROOT_ENABLED),--sysroot=$(STAGINGDIR))
endif

# Define: COMMON_CFLAGS
# Common <CC> and <CXX> flags. Consists of:-
# - -g if _TMTGTREL is set to debug
# - -D option for <TMFL_REL>
# - -D options for Large File Support.
# - -Wall
# - -I switches for include subdirectories of <PREFIXES>
# - <OPTIMISATION> value
# - <DEBUGGING> value
# - MIPS architecture specific flags
# - -MD for dependency file generation
#
# See: <CFLAGS> <CXXFLAGS>
COMMON_CFLAGS = \
	-march=mips32r2 -mtune=4kc \
	-DTMFL_REL=$(TMFL_REL) \
	-D_FILE_OFFSET_BITS=64 -DLARGEFILE_SOURCE -D_LARGEFILE64_SOURCE \
	-Wall \
	-Wextra \
	-MD -MF $(@:.o=.d) -MT $@ \
	-I$(_TMROOT)/inc \
	-I$(BUILDROOT) \
	-I$(_TMTGTBUILDROOT)/comps/generated/lib/$(_TMTGTCPUCLASS)$(_TMTOOLCHAIN)_$(_TMTGTOSCLASS)_$(_TMTGTENDIAN)_$(_TMTGTCPUTYPE) \
	-I$(KSRC)/include \
	$(foreach d, $(PREFIXES), -I$(STAGINGDIR)$(d)/include) \
	$(foreach d, $(PREFIXES), -I$(ROOTFS)$(d)/include) \
	$(OPTIMISATION) \
	$(DEBUGGING) \
	$(SYSROOT_OPTION)

ifneq "$(filter mipsel-linux- mips-linux-,$(CROSS_COMPILE))" ""
    COMMON_CFLAGS+=-msoft-float
endif

# Define: CFLAGS
# <CC> flags. Consists of:-
# - value of <COMMON_CFLAGS>
# - value of <LOCAL_CFLAGS>.
CFLAGS = \
	$(COMMON_CFLAGS) \
	$(LOCAL_CFLAGS)

# Define: LOCAL_CFLAGS
# Application specific flags for compiling (added to <CFLAGS>).

# Define: CXXFLAGS
# <CXX> flags. Consists of:-
# - value of <COMMON_CFLAGS>
# - value of <LOCAL_CXXFLAGS>.
CXXFLAGS = \
	$(COMMON_CFLAGS) \
	$(LOCAL_CXXFLAGS) \

# Define: LOCAL_CXXFLAGS
# Application specific flags for compiling (added to <CXXFLAGS>).




# Define: LDFLAGS
# Flags for linker (uses <CC>). Consists of:-
# - Options to link with libraries stored in directories given by <PREFIXES>.
# - value of <LOCAL_LDFLAGS>.
LDFLAGS = \
  -dynamic \
  -Wl,-rpath-link=$(subst $(space),:,$(strip $(patsubst %,$(ROOTFS)%/lib,$(PREFIXES)))):$(ROOTFS)/lib \
  $(foreach d, $(PREFIXES), -L$(ROOTFS)$(d)/lib) \
  $(foreach d, $(PREFIXES), -L$(STAGINGDIR)$(d)/lib) \
  $(if $(SYSROOT_ENABLED),$(SYSROOT_OPTION),-Wl,-rpath-link=$(subst $(space),:,$(strip $(patsubst %,$(STAGINGDIR)%/lib,$(PREFIXES)))):$(STAGINGDIR)/lib) \
  $(foreach l,$(LIBS),-l$(l)) \
  $(if $V,--trace --verbose) \
  -L$(_TMTGTBUILDROOT)/comps/generated/lib/$(_TMTGTCPUCLASS)$(_TMTOOLCHAIN)_$(_TMTGTOSCLASS)_$(_TMTGTENDIAN)_$(_TMTGTCPUTYPE) \
  $(LOCAL_LDFLAGS)

space = $(empty) $(empty)

# Define: LOCAL_LDFLAGS
# Application specific flags for linking (added to <LDFLAGS>).

# Define: OBJECTS
# Derived from application's makefile's <C_SOURCES> <CXX_SOURCES> <S_SOURCES>.
OBJECTS = $(patsubst %.c,$(BUILDDIR)/%.o,$(C_SOURCES))
OBJECTS += $(foreach f,$(CXX_SOURCES),$(patsubst %,$(BUILDDIR)/%.o,$(basename $f)))
OBJECTS += $(patsubst %.S,$(BUILDDIR)/%.o,$(S_SOURCES))

# Define: C_SOURCES
# List of C source files to be compiled into <PROGRAM>.
# Must be defined by application's makefile.
#
# See: <OBJECTS>

# Define: CXX_SOURCES
# List of C++ source files to be compiled into <PROGRAM>.
# Supported extensions are:
# .cpp .cxx .c++ .cc (in lowercase and uppercase).
# Must be defined by application's makefile.
#
# See: <OBJECTS>

# Define: S_SOURCES
# List of assembler source files (extension: .S) to be compiled into <PROGRAM>.
# Must be defined by application's makefile.
#
# See: <OBJECTS>

# Define: GENERATED_DEPS_FILES
# List of corresponding "xxx.d" file for every object file in <OBJECTS>.
# These files are generated because <COMMON_CFLAGS> contains "-MD".
GENERATED_DEPS_FILES = $(patsubst %.o,%.d,$(OBJECTS))

# Define: INSTALL_DIR
# Installation directory for application.
# Defaults to ${<ROOTFS>}${<_TMTGTPREFIX>}/bin
INSTALL_DIR = $(ROOTFS)$(_TMTGTPREFIX)/bin

INSTALL_EXTRA =

$(BUILDDIR)/%.o:%.c
	$(Q)mkdir -p $(dir $@)
	@echo 'CC $<'
	$(Q)$(CC) $(CFLAGS) -c $< -o $@

define COMPILE_CXX_COMMAND
	$(Q)mkdir -p $(dir $@)
	@echo 'CXX $<'
	$(Q)$(CXX) $(CXXFLAGS) -c $< -o $@
endef
$(BUILDDIR)/%.o:%.cpp
	$(COMPILE_CXX_COMMAND)
$(BUILDDIR)/%.o:%.CPP
	$(COMPILE_CXX_COMMAND)
$(BUILDDIR)/%.o:%.cxx
	$(COMPILE_CXX_COMMAND)
$(BUILDDIR)/%.o:%.CXX
	$(COMPILE_CXX_COMMAND)
$(BUILDDIR)/%.o:%.c++
	$(COMPILE_CXX_COMMAND)
$(BUILDDIR)/%.o:%.C++
	$(COMPILE_CXX_COMMAND)
$(BUILDDIR)/%.o:%.cc
	$(COMPILE_CXX_COMMAND)
$(BUILDDIR)/%.o:%.CC
	$(COMPILE_CXX_COMMAND)

$(BUILDDIR)/%.o:%.S
	$(Q)mkdir -p $(dir $@)
	@echo 'ASM $<'
	$(Q)$(CC) $(CFLAGS) -c $< -o $@

# Variable: V
# Verbose flag. If defined then echo commands. See <Q>.
ifdef V
  Q=
else
  Q=@
endif

# Macro: Q
# Used for controlling the printing of commands.
# If in verbose mode (<V> not empty), then Q is empty, otherwise it is @.
# Use this at the beginning of a Make command line: the command won't be
# printed out if Q expands to @.
