#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2009 NXP B.V.,
# All Rights Reserved.
#
# Filename: platform_control_cpu_linux.mk
#
#
# Rev Date       Author      Comments
#-------------------------------------------------------------------------------
# c30 20080731   watine      USB Key drivers as an option
# c33 20080822   ocourson    Merge with Caen branch / add build option to kernel image
# c36 20081002   ocourson    Add power saving options
#  36 20080925   neill       Add exStbSwKeyboard to LNXEXAPPS
#  37 20081010   batelaan    Correct linux_ENABLE_SERIAL_PNX8XXX_TTYS* vars
#  38 20081014   neill       Add phStbDFBVideoProviderV4L
#37.1.1 20081015 batelaan    Correct linux_ENABLE_SERIAL_PNX8XXX_TTYS* vars again
#  39 20081015   neill       Merge
#  40 20081023   mpage       Add phStbSwSection
#  40 20081022   amccurdy    Add linux_ENABLE_WIRELESS_EXT
#  41 20081022   amccurdy    Fix linux_ENABLE_V4L
#  42 20081027   batelaan    Optimise linux_ENABLE_V4L check
#  43 20081027   mpage       Merge
#  44 20081105   batelaan    Added exStbGpio and exStbSetMode
#  45 20081105   griffiths   Added exStbMetaPlay
#  46 20081113   laird       Merged from 'platform_control_cpu_linux.mk-45:makefile:SOUsb225#1' and 'platform_control_cpu_linux.mk-44.1.1:makefile:SOUsb225#1' by laird.
#  47 20081121   mpage       Add phStbAacDetect
#  48 20081204   neill       Add exStbTeletext
#  49 20081208   batelaan    Merge with Caen version (c)36
#  50 20090106   batelaan    Test $PRJPROPRIETARY (not set in GPL release).
#  50.1.1 20090126 steel     Add Moca Phy support
#  50.1.2 20090126 andyf     Add exStbPolyfuse
#  52 20090127   batelaan    Merge in GYX1D tuner
#  52.1.1 20090127   steel   Add smsc911X support
#  52.1.2 20090203   neill   Add kiss_fft
#  54 20090304   laird       Add support for auto building gstreamer plugins.
#  55 20090305   laird       Merged from 'platform_control_cpu_linux.mk-54.gstreamer:makefile:SOUsb225#1' and 'platform_control_cpu_linux.mk-52.1.2:makefile:SOUsb225#1' by laird.
#  56 20090310   guo         Add Samsung ISDB-T FE drivers (ie tdhl5 dvb4linux API compliant)
#  57 20090312   batelaan    Make some libs only build dependent on BR2_PACKAGE_DIRECTFB and BR2_PACKAGE_GSTREAMER
#  57 20090313   guo         Add Thomson ISDB-T FE drivers (NuTune, dvb4linux API compliant)
#  59 20090313   batelaan    Check $CONFIG_WMDRMPD for phStbWmdrmPd
#  60 20090317   batelaan    Fix phStbDFBVideoProviderASF->phStbWmdrmPd
#  61 20090323   amccurdy    Add linux_ENABLE_xxx support for ext2, ext3, cramfs, jffs2, squashfs and yaffs filesystems
#  61 20090323   cgrosse     Add NXP TDA18218 Silicon Tuner
#  62 20090324   laird       Merge........

# File: platform_control_cpu_linux.mk
# Makefile fragment which builds the control cpu in STB225 for Linux,
# and the only cpu in STB220.
#
# Included from:-
# - <platform_control_cpu.mk> if <CONFIG_OS_LINUX> is set.
#
# Includes:-
# - <platform_cpu_linux.mk>
# - <linux/Makefile> (usually)
# - <rootfs/Makefile>
# - Optionally <docs/Makefile> (may not exist).
# - Optionally <platform_control_cpu_linux_dev.mk> (may not exist).
# - Optionally ${<DI_PRJROOT>}/etc/design_in_control_cpu_linux.mk (may not exist).

include $(PRJROOT)/etc/platform_cpu_linux.mk

# Only register relevant additional SDE2 project directories not mentioned in platform_control_cpu.mk:
$(call SDE2_REGISTER_PROJECTS,\
  $(PRJROOT)/src/demux \
  $(PRJROOT)/src/directfb \
  $(PRJROOT)/src/wmt \
)

# Ensure we include linux make first by loading the linux Makefile
# before other Makefiles.

linux_CONFIG_FILE = $(patsubst "%",%,$(CONFIG_linux_CONFIG_FILE))
buildroot_CONFIG_FILE = $(patsubst "%",%,$(CONFIG_buildroot_CONFIG_FILE))
buildroot_busybox_CONFIG_FILE = $(patsubst "%",%,$(CONFIG_busybox_CONFIG_FILE))
linux_LOADADDRESS = $(CONFIG_MAIN_MIPS_LOAD_ADDR)
linux_VERSION=$(patsubst "%",%,$(CONFIG_linux_VERSION))

ifeq "$(control_KSRC_override)" ""
  # Use own internal kernel tree.
  define linux.make_POSTCMDS
	$(Q)mkdir -p $$TFTPBOOT
	@echo 'Copying $$KSRC/uImage to $$TFTPBOOT/$(TFTPBOOT_UIMAGE_FILE)'
	$(Q)cp -fp $$KSRC/uImage $$TFTPBOOT/$(TFTPBOOT_UIMAGE_FILE)
  endef

  linux_OPTIMIZE_FOR_SIZE := $(if $(CONFIG_TOOLCHAIN_OPTIMIZE_S),true)
  linux_ENABLE_DEBUG := $(if $(CONFIG_RELEASE_BUILD_TYPE_DEBUG),true)
  linux_ENABLE_SATA := $(if $(CONFIG_SATA_PRESENT),true)
  linux_ENABLE_BLK_DEV_LOOP := $(if $(CONFIG_KERNEL_BLK_DEV_LOOP_DRIVER),true)
  linux_ENABLE_WIRELESS_EXT := $(if $(CONFIG_ENABLE_WIRELESS_EXT),true)
  linux_ENABLE_EXT2_FS := $(if $(CONFIG_ENABLE_EXT2_FS),true)
  linux_ENABLE_EXT3_FS := $(if $(CONFIG_ENABLE_EXT3_FS),true)
  linux_ENABLE_CRAMFS_FS := $(if $(CONFIG_ENABLE_CRAMFS_FS),true)
  linux_ENABLE_JFFS2_FS := $(if $(CONFIG_ENABLE_JFFS2_FS),true)
  linux_ENABLE_SQUASHFS_FS := $(if $(CONFIG_ENABLE_SQUASHFS_FS),true)
  linux_ENABLE_YAFFS_FS := $(if $(CONFIG_ENABLE_YAFFS_FS),true)

  linux_ENABLE_SERIAL_PNX8XXX_UART0 := $(if $(CONFIG_UART0_MIPS1_CONSOLE)$(CONFIG_UART0_GENERIC),true)
  linux_ENABLE_SERIAL_PNX8XXX_TTYS0 := $(if $(CONFIG_UART0_MIPS1_CONSOLE)$(CONFIG_UART0_GENERIC),true)
  linux_ENABLE_SERIAL_PNX8XXX_UART1 := $(if $(CONFIG_UART1_MIPS1_CONSOLE)$(CONFIG_UART1_GENERIC),true)
  linux_ENABLE_SERIAL_PNX8XXX_TTYS1 := $(if $(CONFIG_UART1_MIPS1_CONSOLE)$(CONFIG_UART1_GENERIC),true)
  linux_ENABLE_SERIAL_PNX8XXX_CONSOLE := $(if $(CONFIG_UART0_MIPS1_CONSOLE)$(CONFIG_UART1_MIPS1_CONSOLE),true)

  linux_ENABLE_V4L := $(if $(filter 0,$(CONFIG_VIDEO_INPUT_BUFSIZE)),,true)

  linux_ENABLE_DVB_TDA10021 := $(if $(CONFIG_TDA10021),true)
  linux_ENABLE_DVB_TDA10023 := $(if $(CONFIG_TDA10023),true)
  linux_ENABLE_DVB_TDA10046 := $(if $(CONFIG_TDA10046),true)
  linux_ENABLE_DVB_TDA10048 := $(if $(CONFIG_TDA10048),true)
  linux_ENABLE_DVB_TDA10086 := $(if $(CONFIG_TDA10086),true)
  linux_ENABLE_ISDB_TC90507XBG := $(if $(CONFIG_ISDB_TC90507XBG),true)
  linux_ENABLE_ISDB_TDA665XTT := $(if $(CONFIG_ISDB_TDA665XTT),true)
  linux_ENABLE_ISDB_MB86A20S := $(if $(CONFIG_ISDB_MB86A20S),true)
  linux_ENABLE_ISDB_TUA6039F := $(if $(CONFIG_ISDB_TUA6039F),true)
  linux_ENABLE_DVB_TDA826X := $(if $(CONFIG_TDA826X),true)
  linux_ENABLE_DVB_TDA827X := $(if $(CONFIG_TDA8275A),true)
  linux_ENABLE_DVB_TD1316 := $(if $(CONFIG_TD1316)$(CONFIG_DTT8D1T)$(CONFIG_TDQG4)$(CONFIG_DCT4C2),true)
  linux_ENABLE_DVB_TDA18211 := $(if $(CONFIG_TDA18211),true)
  linux_ENABLE_DVB_TDA18218 := $(if $(CONFIG_TDA18218),true)
  linux_ENABLE_DVB_MAX3541 := $(if $(CONFIG_MAX3541),true)
  linux_ENABLE_DVB_GYX1D := $(if $(CONFIG_GYX1D),true)
  linux_ENABLE_IP_MULTICAST := $(if $(CONFIG_MULTICAST),true)
  linux_ENABLE_WATCHDOG := $(if $(CONFIG_WATCHDOG_MODULE),true)
  linux_ENABLE_PRINTK := $(if $(CONFIG_KERNEL_PRINTK_ENABLED),true)
  linux_ENABLE_ETHERNET := $(if $(CONFIG_ETHERNET_PRESENT),true)
  linux_ENABLE_IP3902 := $(if $(CONFIG_ETHERNET_IP3902),true)
  linux_ENABLE_SMSC911X := $(if $(CONFIG_ETHERNET_SMSC911X),true)
  linux_ENABLE_USBMASSKEY := $(if $(CONFIG_USB_MASS_STORAGE_MODULES),true)
  linux_ENABLE_USBALSAKEY := $(if $(CONFIG_ALSA_USB_MODULES),true)
  linux_ENABLE_USBNETKEY := $(if $(CONFIG_NET_USB_MODULES),true)
  linux_ENABLE_USBCORE := $(if $(CONFIG_USB_CORE),true)
  linux_ENABLE_NETPHY_PIO29 := $(if $(CONFIG_NETPHY_RESET_PIO_29),true)
  linux_ENABLE_MOCA_PHY := $(if $(CONFIG_MOCA_PHY),true)
  linux_ENABLE_PM := $(if $(CONFIG_POWER_MGMT_CPU),true)

else # ifeq "$(control_KSRC_override)" ""
  # Use external kernel tree.
  KSRC_override := $(control_KSRC_override)
endif # ifeq "$(control_KSRC_override)" ""

$(call INCLUDE_MAKEFILE, src/open/linux/Makefile)

# include or skip other makefiles dependent on target
ifneq "$(IC_EMULATION)" "1"
  # We are not emulating
  ifeq "$(filter $(ARCH), $(shell uname --hardware-platform --machine --processor))" ""
    # cross compilation - include everything
    $(call INCLUDE_MAKEFILES, \
      $(filter-out \
        src/open/linux/Makefile, \
        $(wildcard src/open/*/Makefile)))

    # call the nandbootloader makefiles
    ifeq "$(CONFIG_NAND_FLASH)" "y"
        $(call INCLUDE_MAKEFILE, src/nandboot/Makefile)
    endif
  else
    # build for host PC - currently don't include anything.
  endif
else
  # emulation build - skip everything
  LNX_USERSPACE_SKIPBUILD=true
endif

$(call INCLUDE_MAKEFILE, src/rootfs/Makefile)

$(call INCLUDE_MAKEFILES,$(wildcard $(PRJROOT)/docs/Makefile))

ifeq "$(BR2_PACKAGE_GSTREAMER)" "y"
  $(call INCLUDE_MAKEFILE, src/open/comps/gstStb22xFramedAudio/Makefile)
  $(call INCLUDE_MAKEFILE, src/open/comps/gstStb22xFramedVideo/Makefile)
endif

# ROKU STUFF
$(call INCLUDE_MAKEFILE, src/rokusafemode/Makefile)

################################################################################
# LNX (LINUX LIBRARIES/APPLICATIONS)
################################################################################

ifneq "$(LNX_USERSPACE_SKIPBUILD)" "true"

#-------------------------------- LNXLIBS --------------------------------

LNXLIBS = \
    $(LNXLIBS_OPEN_SOURCE) \
    $(if $(PRJPROPRIETARY),$(LNXLIBS_PROPRIETARY))

LNXLIBS_OPEN_SOURCE=\
    phKernelIfce \
    phStbDbg \
    $(if $(BR2_PACKAGE_DIRECTFB),phStbDFB) \
    $(if $(BR2_PACKAGE_JPEG), phStbImage) \
    phStbNetDev \
    kiss_fft

LNXLIBS_PROPRIETARY = \
    $(if $(BR2_PACKAGE_DIRECTFB), \
        phStbDFBImageProvider \
        phStbDFBVideoProviderEs \
        $(if $(BR2_PACKAGE_GSTREAMER),phStbDFBVideoProviderGst) \
        phStbDFBVideoProviderTs \
        phStbDFBVideoProviderV4L \
    ) \
    phStbSbmSource \
    phStbSbmSink \
    $(if $(BR2_PACKAGE_ALSA_LIB), tmdlAudio) \
    $(if $(BR2_PACKAGE_ALSA_LIB), tmdlAudioIO) \
    $(if $(findstring _wmv_,$(_TMDIVERSITY)),\
	$(if $(BR2_PACKAGE_DIRECTFB),phStbDFBVideoProviderASF) \
	$(if $(CONFIG_WMDRMPD),phStbWmdrmPd) \
    ) \
    phStbAv \
    phStbDemuxSW \
    phStbIpc \
    phStbDemuxTs \
    phStbDemuxPs \
    phStbDemuxMp4 \
    phStbDemuxAvi \
    phStbSwSection \
    phStbAacDetect

#-------------------------------- LNXAPPS --------------------------------
LNXPLFAPPS=\
    $(LNXPLFAPPS_OPEN_SOURCE) \
    $(if $(PRJPROPRIETARY),$(LNXPLFAPPS_PROPRIETARY))

LNXPLFAPPS_OPEN_SOURCE=\
  tmmanifest

LNXPLFAPPS_PROPRIETARY=\
  tmdbg

ifdef PRJPROPRIETARY
LNXEXAPPS=\
    exStbAES \
    exStbCc \
    $(if $(BR2_PACKAGE_DIRECTFB),exStbDemo) \
    $(if $(BR2_PACKAGE_DIRECTFB),exStbDemux) \
    exStbFb \
    exStbGpio \
    $(if $(filter y-y,$(BR2_PACKAGE_DIRECTFB)-$(CONFIG_HDMI_PRESENT)),exStbHDMI) \
    exStbHwAcc \
    $(if $(CONFIG_JTAG_LOOPBACK_PRESENT),exStbJtag) \
    $(if $(BR2_PACKAGE_DIRECTFB),exStbKey) \
    exStbLED \
    exStbPolyfuse \
    exStbResolution \
    $(if $(BR2_PACKAGE_DIRECTFB),exStbSetMode) \
    exStbSwKeyboard \
    exStbTeletext \
    $(if $(BR2_PACKAGE_DIRECTFB),exStbThumbs) \
    exStbTop \
    $(if $(BR2_PACKAGE_DIRECTFB),exStbVideo) \
    $(if $(BR2_PACKAGE_DIRECTFB),exStbMetaPlay)
endif

endif # ifneq "$(LNX_USERSPACE_SKIPBUILD)" "true"

# Design-In project extensions:
ifdef DI_PRJROOT
  -include $(DI_PRJROOT)/etc/design_in_control_cpu_linux.mk
endif

ifneq "$(PRJRELEASEBUILD)" "true"
  # Development extensions:
  -include $(PRJROOT)/etc/platform_control_cpu_linux_dev.mk
endif
