#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2008 NXP B.V.,
# All Rights Reserved.
#
# Filename: platform_control_cpu_wince.mk
#
#
# Rev Date       Author      Comments
#-------------------------------------------------------------------------------
#   1 20070619   batelaan    Initial
#   2 20070705   batelaan    Correct first implementation
#   3 20070717   batelaan    Use new roster.mk file.
#   4 20070720   batelaan    Deprecate KERNEL_MODE and KMODULE vars.
#   5 20070725   batelaan    Add target copy_sde2_header_files
#   6 20070725   batelaan    Check _TMTGTCOPYINC only when doing the proper build.
#   7 20070809   batelaan    Correct _TMDIVERSITY value
#   8 20071002   batelaan    Integrate MS1-SP3 changes
#   9 20071009   batelaan    Add userlibs capability.
#                            Add non-ported component placeholders
#  10 20071023   batelaan    Further implement userlibs capability.
#  11 20080318   batelaan    Copy more global/standard include files.
#  12 20080814   batelaan    Moved buildrelease.info target(s) to sde2.mk. Renamed.

# File: platform_control_cpu_wince.mk
# Makefile fragment which builds the control cpu in STB225 for WinCE.
#
# Included from:-
# - <platform_control_cpu.mk> if <CONFIG_OS_WINCE> is set.
#
# Includes:-
# - none.

ifneq "$(_TMTGTOSCLASS)" "ce"
  $(error Incorrect environment setup (detected _TMTGTOSCLASS != 'ce'))
endif

include $(PRJROOT)/etc/roster.mk

ifneq "$(filter all,$(MAKECMDGOALS))" ""
  ifeq "$(wildcard $(_TMTGTCOPYINC))" ""
    $(error Environment variable _TMTGTCOPYINC ($(_TMTGTCOPYINC)) is invalid. Expected value is existing directory to copy header files to.)
  endif
endif

#-----------------------------------------------------------------------
# MAIN TARGETS
#-----------------------------------------------------------------------

lib_build: userlibs.lib.make
lib_build: kmodules.lib.make
lib_build: copy_header_files

dll_build: userlibs.dll.make
dll_build: kmodules.dll.make

# Target: nxp.build.success.indicator
# Creates the file with the same name as the target.
# This file is always removed when this makefile is executed.
nxp.build.success.indicator:
	@touch nxp.build.success.indicator
$(shell rm -f $$PRJROOT/nxp.build.success.indicator)

SDE2_BUILD_EXE_OPTS ?= -c

#-----------------------------------------------------------------------
# KERNEL MODULES
#-----------------------------------------------------------------------

KMODULE_COMP_NAME=$(firstword $(KMODULE_COMP_NAME_$1) $1)

.PHONY: kmodules.lib.make
kmodules.lib.make: $(foreach name,$(KMODULE_NAMES),$(call KMODULE_COMP_NAME,$(name)).kmodule.lib.make)
%.kmodule.lib.make: _TMDIVERSITY:=_kmodule$(_TMDIVERSITY)
%.kmodule.lib.make:
	@echo '  KMODULE MAKE $*'
	$(Q)$(MAKECMD) -C $(call SDE2_CI_LOOKUP,$*) kmodule_lib_build

.PHONY: kmodules.dll.make
kmodules.dll.make: $(foreach name,$(KMODULE_NAMES),$(call KMODULE_COMP_NAME,$(name)).kmodule.dll.make)
%.kmodule.dll.make: _TMDIVERSITY:=_kmodule$(_TMDIVERSITY)
%.kmodule.dll.make:
	@echo '  KMODULE MAKE $*'
	$(Q)$(MAKECMD) -C $(call SDE2_CI_LOOKUP,$*) kmodule

#-----------------------------------------------------------------------
# USERMODE USERLIB_NAMES
#-----------------------------------------------------------------------

.PHONY: userlibs.lib.make
userlibs.lib.make:
	$(Q)perl $$_TMROOT/sde/scripts/build_exe.pl \
		-target:_SDE_BIN_REL_LIB \
		$(foreach l,$(USERLIB_NAMES),$(call SDE2_CI_LOOKUP,$l))

.PHONY: userlibs.dll.make
userlibs.dll.make:
	$(Q)perl $$_TMROOT/sde/scripts/build_exe.pl \
		$(foreach l,$(USERLIB_NAMES),$(call SDE2_CI_LOOKUP,$l))

#-----------------------------------------------------------------------
# FILE COPYING
#-----------------------------------------------------------------------

# Define: SDE2_HEADER_FILES_TO_COPY
# Global files that SDE2 generates, and are useful to copy into the <_TMTGTCOPYINC> directory
# using target <copy_sde2_header_files>.
SDE2_HEADER_FILES_TO_COPY = tmFlags.h phFlags.h

# Target: copy_header_files
# Copies various header files into <_TMTGTCOPYINC> directory,
# so that they are available in one easily accessible location
# for compilation under WinCE.
#
# See <copy_sde2_header_files> and <copy_other_header_files>.
copy_header_files: \
    copy_sde2_header_files \
    copy_sde2_global_inc_files \
    $(_TMTGTCOPYINC)/phStbCommon.h \
    $(_TMTGTCOPYINC)/prjconfig.h

# Target: copy_sde2_header_files
# Copies <SDE2_HEADER_FILES_TO_COPY> from <SDE2_COMPS_GENERATED_DIR> to <_TMTGTCOPYINC>.
copy_sde2_header_files: $(addprefix $(_TMTGTCOPYINC)/,$(SDE2_HEADER_FILES_TO_COPY))
$(_TMTGTCOPYINC)/%.h: $(SDE2_COMPS_GENERATED_DIR)/%.h
	cp -f $^ $@


$(foreach f,$(wildcard $(_TMROOT)/inc/*.h), \
  $(eval copy_sde2_global_inc_files: $(_TMTGTCOPYINC)/$(notdir $f)) \
  $(eval $(_TMTGTCOPYINC)/$(notdir $f): $f) \
)

$(_TMTGTCOPYINC)/%.h: $(_TMROOT)/inc/%.h
	cp -f $< $@

$(_TMTGTCOPYINC)/prjconfig.h: $(BUILDROOT)/prjconfig.h
	cp -f $^ $@

$(_TMTGTCOPYINC)/phStbCommon.h: $(_IphStbCommon_DIR)/inc/phStbCommon.h
	cp -f $^ $@

$(_TMTGTCOPYINC)/standard.h: $(_tmgdrTypes_DIR)/inc/standard.h
	cp -f $^ $@

#-----------------------------------------------------------------------
# HELP
#-----------------------------------------------------------------------

help.platform_cpu:: wince.help
.PHONY: wince.help
wince.help:
	@echo 'all              - Builds kernel modules and copies header files'

