#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2009 NXP B.V.,
# All Rights Reserved.
#
# Filename: platform_dsp_cpu.mk
#
# Builds mipsdsp subsystem
#
#
# Rev Date       Author      Comments
#-------------------------------------------------------------------------------
#   1 20060728   batelaan    Initial
#   2 20060810   batelaan    Crosscheck fix
#   3 20060927   batelaan    Add hamburg components directory.
#   4 20061102   batelaan    Use $(PRJROOT)/plf/open/linux/Makefile. Use initramfs.
#                            Add $(PRJROOT)/drv/mipsdsp/AudioCodecs to SDE2_REGISTER_PROJECTS call.
#   5 20061205   batelaan    Define linux_LOADADDRESS and linux.make_POSTCMDS.
#   6 20061218   batelaan    Use _TMTGTPREFIX variable.
#   7 20061218   batelaan    Remove unused code.
#   8 20070112   batelaan    Ensure linux.make depends on userspace lnx target.
#                            initramfs contents depends on lnx target results.
#   9 20070124   batelaan    Only use initramfs if we are cross-compiling.
#  10 20070214   batelaan    Merge drv&plf into src directory.
#  11 20070227   raiyat      Add phdspMainApp
#  12 20070301   awest       Add wmt and wmtEx build directories
#  13 20070619   batelaan    Reworked structure for WinCE.
#  14 20070823   batelaan    Read optional customisation files.
#  15 20071008   raiyat      Add tmdbg
#  16 20071206   batelaan    Add NFSROOT capability next to INITRAMFS
#  17 20080314   laird       Specify linux_VERSION before including src/open/linux/Makefile
#  18 20080326   laird       Specify linux_OPTIMIZE_FOR_SIZE before including src/open/linux/Makefile
#  19 20080327   laird       Change copy location of uImage to firmware.
#  20 20080415   batelaan    Rm DSP_NFSROOT
#  21 20080415   batelaan    Add getfile
#  23 20080425   batelaan    Add linux/buildroot/busybox config file options.
#  24 20080603   batelaan    Rm linux_USE_NFSROOT
#  27.1 080729   ocourson    Add multicast option
#  27.2 080729   ocourson    Add watchdog module selection
#  27.3 080729   watine      Add printk support 
#  27.5 080731   watine      define USB keys macros
#  27.6 080731   watine      USB Key drivers as an option
#  28 20080708   batelaan    Merge; rm unneeded linux_ENABLE* macros.
#  29 20080822   ocourson    Merge; rm unneeded linux_ENABLE* macros.
#  29 20080821   batelaan    Merge; fix build errors for non-trace builds.
#  30 20080911   batelaan    Assign all apps to LNXAPPS, not to LNXAPPSNOSRC since there should always be src.
#  31 20081022   amccurdy    Add linux_ENABLE_WIRELESS_EXT init
#  33 20081209   batelaan    Merge with CFR_STB#27.7
#  34 20090112   batelaan    Check $PRJPROPRIETARY to select what to build.
#  35 20090127   batelaan    Merge to get rid of parallels
#  36 20090323   batelaan    Rename some vars for consistency
#  37 20090327   batelaan    Include toolchains/<CONFIG_MIPS2_TOOLCHAIN>.mk

# File: platform_dsp_cpu.mk
# Makefile for building the dsp processor subsystem.
#
# Includes:-
# - <first.mk>
# - toolchains/<CONFIG_MIPS2_TOOLCHAIN>.mk
# - <platform_cpu.mk>
# - <mipsdsp/rootfs/Makefile>
# - <buildroot/Makefile>
# - <linux/Makefile>
# - src/mipsdsp/open/*/Makefile
# - Optionally <platform_dsp_cpu_dev.mk> (may not exist).
# - Optionally $<DI_PRJROOT>/etc/design_in_dsp_cpu.mk (may not exist).
# - <last.mk>

include $(PRJROOT)/etc/first.mk
include $(call GET_TOOLCHAIN_SETUP_FILE, CONFIG_MIPS2_TOOLCHAIN)
include $(PRJROOT)/etc/platform_cpu.mk
include $(PRJROOT)/etc/platform_cpu_linux.mk

MAKEFILE_TITLE := STB225 DSP CPU build Makefile

# Only register relevant additional SDE2 project directories not mentioned in platform_cpu.mk:
$(call SDE2_REGISTER_PROJECTS,\
  $(PRJROOT)/src/mipsdsp \
  $(PRJROOT)/src/mipsdsp/wmt \
  $(PRJROOT)/src/mipsdsp/wmtEx \
  $(PRJROOT)/src/mipsdsp/AudioCodecs \
)

$(call INCLUDE_MAKEFILE, src/mipsdsp/rootfs/Makefile)

ifeq "$(filter $(ARCH), $(shell uname --hardware-platform --machine --processor))" ""
  # cross compilation

  linux_CONFIG_FILE = $(patsubst "%",%,$(CONFIG_MIPS2_linux_CONFIG_FILE))
  buildroot_CONFIG_FILE = $(patsubst "%",%,$(CONFIG_MIPS2_buildroot_CONFIG_FILE))
  buildroot_busybox_CONFIG_FILE = $(patsubst "%",%,$(CONFIG_MIPS2_busybox_CONFIG_FILE))

  $(call INCLUDE_MAKEFILE, src/open/buildroot/Makefile)

  # Ensure that Linux uses initramfs:
  buildroot_SKIPBUILD := true
  linux_USE_INITRAMFS := true

  # This should be set by doing the following:
  # linux_OPTIMIZE_FOR_SIZE = $(if $(CONFIG_TOOLCHAIN_OPTIMIZE_S),true)
  # but due to audio mips startup issues.....
  linux_OPTIMIZE_FOR_SIZE = true

  #### DO NOT ADD EMPTY linux_ENABLE_XXX VARIABLE DEFINITIONS HERE ####
  ####                      IT IS UNNECESSARY!!!                   #### 

  linux_ENABLE_SERIAL_PNX8XXX_UART0 := $(if $(CONFIG_UART0_MIPS2_CONSOLE),true)
  linux_ENABLE_SERIAL_PNX8XXX_TTYS0 := $(if $(CONFIG_UART0_MIPS2_CONSOLE),true)
  linux_ENABLE_SERIAL_PNX8XXX_UART1 := $(if $(CONFIG_UART1_MIPS2_CONSOLE),true)
  linux_ENABLE_SERIAL_PNX8XXX_TTYS1 := $(if $(CONFIG_UART1_MIPS2_CONSOLE),true)
  linux_ENABLE_SERIAL_PNX8XXX_CONSOLE := $(if $(CONFIG_UART0_MIPS2_CONSOLE),true,$(if $(CONFIG_UART1_MIPS2_CONSOLE),true))
  linux_ENABLE_PRINTK := $(if $(CONFIG_KERNEL_PRINTK_ENABLED),true)

  linux_LOADADDRESS = $(CONFIG_DSP_MIPS_LOAD_ADDR)
  linux_VERSION=$(patsubst "%",%,$(CONFIG_MIPS2_linux_VERSION))
  linux.make_POSTCMDS = \
	$(Q)echo '  COPY    uImage to $$CPUBUILDROOT_control/rootfs/lib/firmware/dsp.fw'; \
	mkdir -p $$CPUBUILDROOT_control/rootfs/lib/firmware; cp -fp $$KSRC/uImage $$CPUBUILDROOT_control/rootfs/lib/firmware/dsp.fw
endif

$(call INCLUDE_MAKEFILE, src/open/linux/Makefile)
$(call INCLUDE_MAKEFILES, $(wildcard src/mipsdsp/open/*/Makefile))

################################################################################
# LNX (LINUX LIBRARIES/APPLICATIONS)
################################################################################

ifneq "$(LNX_USERSPACE_SKIPBUILD)" "true"

#-------------------------------- LNXAPPS --------------------------------

LNXAPPS = \
    $(LNXPLFAPPS_OPEN_SOURCE) \
    $(if $(PRJPROPRIETARY),$(LNXPLFAPPS_PROPRIETARY))

LNXPLFAPPS_OPEN_SOURCE := \

LNXPLFAPPS_PROPRIETARY := \
    phdspMainApp \
    tmdbg \
    getfile

# Ensure linux.make depends on userspace lnx target.
# initramfs contents depends on lnx target results.
$(call OBJ_TARGET_OTHER_DEPS,linux.make, lnx)

endif # ifneq "$(LNX_USERSPACE_SKIPBUILD)" "true"

#-------------------------------------------------------------------------

# Design-In project extensions:
ifdef DI_PRJROOT
  -include $(DI_PRJROOT)/etc/design_in_dsp_cpu.mk
endif

ifneq "$(PRJRELEASEBUILD)" "true"
  # Development extensions:
  -include $(PRJROOT)/etc/platform_dsp_cpu_dev.mk
endif

$(PLATFORM_CPU_END)

include $(PRJROOT)/etc/last.mk
