#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2009 NXP B.V.,
# All Rights Reserved.
#
# Filename: platform_stb225.mk
#
#
# Rev Date       Author      Comments
#-------------------------------------------------------------------------------
#  20 20070301   batelaan    Set _TMLINKTYPE=static, we don't want shared libraries,
#                            if we have only one application.
#  21 20070315   batelaan    Add configuration aliases 'control' and 'dsp'.
#  22 20070423   batelaan    Use sde2.os.linux.env / sde2.os.wince.env variables.
#  23 20071023   batelaan    Introduce control_toolchain and dsp_toolchain vars
#  24 20071023   batelaan    Ensure control/dsp.env are called early enough to
#                            set toolchain.
#  25 20071220   laird       Adding a variable to control the version of directfb we are building. Default 1.0
#  26 20080305   batelaan    Remove linux_VARIANT, now using CONFIG_linux_VERSION.
#  27 20080306   batelaan    Toolchain specification using prjconfig.
#  28 20080314   laird       Specify KSRC in each sub shell now (may have different linux versions).
#  29 20080523   batelaan    Pass V=1 down to sub-make invocations.
#  31 20080717   laird       Remove directfb_VERSION as done by hidden prjconfig option now.
#  32 20080723   batelaan    Check control_KSRC_override
#  33 20080710   watine      Force trace build option for dsp
#  33 20080821   batelaan    Merge Caen changes on _TMTGTREL setting:
#                            If globally building for debug, the dsp will revert to trace mode.
#  35 20081209   batelaan    Always build in trace mode, until other modes have been verified.
#  36 20090112   batelaan    Add dsp config if $(DSP_PREBUILT_FIRMWARE_PATH) does not exist
#                            or $PRJBUILDDSP is defined.
#                            Remove big endian stuff.

# File: platform_stb225.mk
# Makefile fragment for building the stb225 software platform. Included from <platforms.mk>.

#-------------------------------------------------------------------------------
# Initial definition and help
#-------------------------------------------------------------------------------
platforms += stb225

stb225.processors := control dsp
stb225.control.cpus := \
  pnx8335-stb225

stb225.control.pnx8335-stb225.endian := le

stb225.dsp.cpus := \
  pnx8335-stb225

stb225.dsp.pnx8335-stb225.endian := le

stb225/control/pnx8335-stb225-le_ALIAS := control

stb225/dsp/pnx8335-stb225-le_ALIAS := dsp

ifneq "$(CONFIG_OS_WINCE)$(CONFIG_OS_LINUX)" "y"
  $(error No valid CONFIG_OS_... variable defined (WINCE/LINUX))
endif

#-------------------------------------------------------------------------------
# Configuration variables
#-------------------------------------------------------------------------------

# Ensure variables are exported to subprocesses:
$(foreach p,$(stb225.processors),\
   $(eval export CPUBUILDROOT_$p:=))

#-------------------------------------------------------------------------------
# Environment Fragments
#-------------------------------------------------------------------------------
# To work out the inheritance between environments below, run make with the TRACE=1 option.

# Macro: CPUBUILDROOT_SUFFIX_STB225
# Suffix appended to <BUILDROOT> to form <CPUBUILDROOT>.
# - If we are building for pnx8335-stb225 little endian, then it is empty.
# - If we are building for pnx8335-stb225 big endian, then it is "/be".
# - If we are not building for pnx8335-stb225, then it starts with /be for big endian, followed by the cpu indication.
#
# Usage: $(call CPUBUILDROOT_SUFFIX_STB225,$(processor),$(cpu),$(_TMTGTENDIANX))
CPUBUILDROOT_SUFFIX_STB225 = $$(filter /be,/$3)$$(filter-out /pnx8335-stb225,/$2)$$(filter-out /control,/$1)

define platform_stb225.env
  $(call ENVS_ENTER,platform_stb225.env)
  $(call setenvVar,CPUBUILDROOT,$$(CPUBUILDROOT_$$(processor)))
  $(call setenvVar,CPUBUILDROOT_dsp,$$(BUILDROOT)$(call CPUBUILDROOT_SUFFIX_STB225,dsp,$$(cpu),$$(_TMTGTENDIANX)))
  $(call setenvVar,CPUBUILDROOT_control,$$(BUILDROOT)$(call CPUBUILDROOT_SUFFIX_STB225,control,$$(cpu),$$(_TMTGTENDIANX)))
  $(stb22x.env)
  $(call _TMDIVERSITY.addValue,swplf225)
  $(call ENVS_LEAVE,platform_stb225.env)
endef

define cpu/pnx8335.env
  $(call ENVS_ENTER,cpu/pnx8335.env)
  $(cpu/pnx833x.env)
  $(call _TMDIVERSITY.addValue,pnx8335)
  $(call ENVS_LEAVE,cpu/pnx8335.env)
endef

# Variable: control_toolchain
# Toolchain to use for building the control processor.
# This is set to <CONFIG_TOOLCHAIN> without the quotes.
control_toolchain ?= $(patsubst "%",%,$(CONFIG_TOOLCHAIN))
$(call CHECK_TOOLCHAIN, CONFIG_TOOLCHAIN)
include $(call GET_TOOLCHAIN_SETUP_FILE, CONFIG_TOOLCHAIN)

# Variable: control_KSRC_override
# Optional environment or make variable.
# Specifies if the control CPU should use an externally built
# Linux kernel tree. Variable should be the directory of such a kernel tree.
# If defined, then <KSRC> will be set to it for the control CPU.

define stb225/control.env
  $(call ENVS_ENTER,stb225/control.env)
  $(call setenvVar,processor,control)
  $(eval override toolchain:=$(control_toolchain))
  $(call setenvVar,KSRC,$$(firstword $(control_KSRC_override) $$(CPUBUILDROOT)/packages/linux-$$(patsubst "%",%,$$(CONFIG_linux_VERSION))))
  $(if $(CONFIG_OS_LINUX),$(sde2.os.linux.env))
  $(if $(CONFIG_OS_WINCE),$(sde2.os.wince.env))
  $(call ENVS_LEAVE,stb225/control.env)
endef

# Variable: dsp_toolchain
# Toolchain to use for building the dsp processor.
# This is set to <CONFIG_MIPS2_TOOLCHAIN> without the quotes.
dsp_toolchain ?= $(patsubst "%",%,$(CONFIG_MIPS2_TOOLCHAIN))
include $(call GET_TOOLCHAIN_SETUP_FILE, CONFIG_MIPS2_TOOLCHAIN)

define stb225/dsp.env
  $(call ENVS_ENTER,stb225/dsp.env)
  $(call setenvVar,processor,dsp)
  $(call setenvVar,_TMLINKTYPE,static)
  $(call _TMDIVERSITY.addValue,dsp)
  $(eval override toolchain:=$(dsp_toolchain))
  $(call setenvVar,_TMTGTREL,trace)
  $(call setenvVar,KSRC,$$(CPUBUILDROOT)/packages/linux-$$(patsubst "%",%,$$(CONFIG_MIPS2_linux_VERSION)))
  $(sde2.os.linux.env)
  $(call ENVS_LEAVE,stb225/dsp.env)
endef

define stb225/.../pnx8335-stb225.env
  $(call ENVS_ENTER,stb225/.../pnx8335-stb225.env)
  $(call setenvVar,cpu,pnx8335-stb225)
  $(platform_stb225.env)
  $(cpu/pnx8335.env)
  $(call setenvVar,_TMBSL,_stb225)
  $(call ENVS_LEAVE,stb225/.../pnx8335-stb225.env)
endef

#-------------------------------------------------------------------------------
# Actual Environments
#-------------------------------------------------------------------------------
# To work out the inheritance between environments below, run make with the TRACE=1 option.


# stb225 board - little endian

define stb225/control/pnx8335-stb225-le.env
  $(call ENVS_ENTER,stb225/control/pnx8335-stb225-le.env)
  $(call setenvVar,_TMTGTENDIAN,el)
  $(stb225/control.env)
  $(stb225/.../pnx8335-stb225.env)
  $(call ENVS_LEAVE,stb225/control/pnx8335-stb225-le.env)
endef

define stb225/dsp/pnx8335-stb225-le.env
  $(call ENVS_ENTER,stb225/dsp/pnx8335-stb225-le.env)
  $(call setenvVar,_TMTGTENDIAN,el)
  $(stb225/dsp.env)
  $(stb225/.../pnx8335-stb225.env)
  $(call ENVS_LEAVE,stb225/dsp/pnx8335-stb225-le.env)
endef


#-------------------------------------------------------------------------------
# stb225 processor targets
#-------------------------------------------------------------------------------
# Macro: stb225.EXCLUDED_FROM_DEFAULT_BUILDS
# Returns true if the configuration is excluded by default.
#
# Usage: $(call stb225.EXCLUDED_FROM_DEFAULT_BUILDS,configuration)
stb225.EXCLUDED_FROM_DEFAULT_BUILDS=$(if $(filter $(1),$(stb225_configs_default)),,true)

# Macro: DSP_PREBUILT_FIRMWARE_PATH
# Pre-built firmware image for the DSP processor.
# This file may not exist. This depends on whether the project contains
# sufficient sources to re-build it.
#
# Value is: src/rootfs/lib/firmware/dsp.fw
#
# See also: <PRJBUILDDSP>
DSP_PREBUILT_FIRMWARE_PATH := src/rootfs/lib/firmware/dsp.fw

# Variable: PRJBUILDDSP
# If set to true, then allow the dsp configuration to be built.
# Otherwise, the presence of the file <DSP_PREBUILT_FIRMWARE_PATH> is checked,
# and if present the dsp configuration is assumed to be built already.
# Can be set by the environment or command line.
#
# See also: <stb225_configs_default> <DSP_PREBUILT_FIRMWARE_PATH>

# Macro: stb225_configs_default
# List of cpu configurations to be built by default.
# See also target with the same name.
#
# See also: <PRJBUILDDSP>

stb225_configs_default := \
  stb225/control/pnx8335-stb225-le \
  $(if $(wildcard $(DSP_PREBUILT_FIRMWARE_PATH)), \
	$(if $(filter true, $(PRJBUILDDSP)),stb225/dsp/pnx8335-stb225-le) \
	,stb225/dsp/pnx8335-stb225-le) \
  $(stb225_configs_default)

configs += $(stb225_configs_default)

# Target: stb225_configs_default
# Builds default cpu configurations.
# See also target with the same name.
stb225_configs_default: $(stb225_configs_default)
stb225: stb225_configs_default

$(foreach p,$(stb225.processors),\
  $(foreach c,$(stb225.$p.cpus),\
    $(foreach e,$(stb225.$p.$c.endian),\
      $(if $(filter undefined,$(origin stb225/$p/$c-$e.env)),$(warning Missing environment definition: stb225/$p/$c-$e)) \
      $(if $(call stb225.EXCLUDED_FROM_DEFAULT_BUILDS, stb225/$p/$c-$e),\
        $(eval stb225_configs_other += stb225/$p/$c-$e)) \
      $(call setupEnvGoal,stb225/$p/$c-$e,$(stb225/$p/$c-${e}_ALIAS)) \
)))

stb225_configs_other: $(stb225_configs_other)
configs_other += $(stb225_configs_other)

#-------------------------------------------------------------------------------
# stb225 help targets
#-------------------------------------------------------------------------------

help::
	@printf '\nPlatform stb225 consists of the following cpu configuration(s)\n'
	@printf '(target all or stb225_configs_default builds them):\n'
	@$(foreach c,$(stb225_configs_default),printf "    %-38s  %s\n" '$c' $(if $(${c}_ALIAS),'- ALIAS: $(${c}_ALIAS)');)

ifneq "$(strip $(stb225_configs_other))" ""
  help::
	@printf '\nExcluded from "all" builds are the following\n'
	@printf '(target stb225_configs_other builds them):\n'
	@$(foreach c,$(stb225_configs_other),printf "    %-38s  %s\n" '$c' $(if $(${c}_ALIAS),'ALIAS: $(${c}_ALIAS)');)
endif

help::
	@printf '\nBuild result locations:\n'
${foreach p,$(stb225.processors),\
  ${foreach c,$(stb225.$p.cpus),\
    ${foreach e,$(stb225.$p.$c.endian),\
      ${eval help::\; @printf '    %-38s  - %s\n' 'stb225/$p/$c-$e' '$$$$$$$$BUILDROOT$(call CPUBUILDROOT_SUFFIX_STB225,$p,$c,$e)'}\
     }\
   }\
}

#-------------------------------------------------------------------------------
# stb225/<control|dsp>/<cpu> targets
#-------------------------------------------------------------------------------

SUBGOAL_OPTS=$(if $V,V=$V)

stb225/control/%.goal:
	$(MAKECMD) $(SUBGOAL_OPTS) $(call getSubGoals,$@,all) cpu=$(cpu) env=$(patsubst %.goal,%,$@)

stb225/dsp/%.goal:
	$(MAKECMD) $(SUBGOAL_OPTS) $(call getSubGoals,$@,all) cpu=$(cpu) env=$(patsubst %.goal,%,$@)
