#
#                             stb225 Project
#                   ---------------------------------
#
# Copyright (C) 2008 NXP B.V.,
# All Rights Reserved.
#
# Filename: etc/site_setup.mk
#
#
# Rev Date       Author      Comments
#-------------------------------------------------------------------------------
#   1 20080623   batelaan    Initial

# File: site_setup.mk
# Site-specific setup file sourced from <init.mk>.
# It includes file ${<PRJROOT>}/etc/site_setup_${<_TMSITE>}.mk
# if it exists. Otherwise the defaults in this file will be used.
#
# Note: a site-specific file does not have to define all variables
# since this file contains default values for all variables.

-include $(PRJROOT)/etc/site_setup_$(_TMSITE).mk

# Define: SITE_SETUP_SUFFIX
# Suffix used to create file names etc, that depend on the product/version/user
# that is doing the build.
# Used in <TFTPBOOT_UIMAGE_FILE> and <NFSROOT_ROOTFS_LINK>.
# Can be set by the environment.
#
# Default value consists of:
# - "${<STBPLF>}"
# - if <PRODUCTVERSION> not "dev" or empty, then "_${<CONFIG_NAME>}_${<PRODUCTVERSION>}".
SITE_SETUP_SUFFIX ?= $(STBPLF)$(if $(filter-out dev,$(PRODUCTVERSION)),_$(patsubst "%",%,$(CONFIG_NAME))_$(PRODUCTVERSION))

# Define: TFTPBOOT_UIMAGE_FILE
# Name of the uImage file copied to the directory given by <TFTPBOOT> by the
# <linux.make> target using commands defined in <linux.make_POSTCMDS>
# in file <platform_control_cpu_linux.mk>.
# Can be set by the environment.
#
# See: <SITE_SETUP_SUFFIX>
TFTPBOOT_UIMAGE_FILE ?= uImage_$(SITE_SETUP_SUFFIX)

# Define: NFSROOT_ROOTFS_LINK
# Name of the symlink created in the directory given by <NFSROOT> by the
# <rootfs.nfsexport> target.
#
# See: <SITE_SETUP_SUFFIX>
NFSROOT_ROOTFS_LINK ?= rootfs_$(SITE_SETUP_SUFFIX)
