#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2009 NXP B.V.,
# All Rights Reserved.
#
# Filename: toolchains/tuxbuilder_common.mk
#
# Rev Date        Author      Comments
#-------------------------------------------------------------------------------
#   1 20080306    batelaan    Initial
#   2 20080317    amccurdy    Add ccache support
#   4 20090323    steel       update for uclibc

# File: tuxbuilder_common.mk
# Include makefile to define the tuxbuilder MIPS toolchain.
# Defines variable called "toolchain/tuxbuilder-<TUXBUILDER_VERSION>.env" .
#
# It expects make variable <TUXBUILDER_VERSION> to be defined.
#
# Included from:-
# - tuxbuilder-VERSION.mk files.
#
# Includes:-
# - none

export CROSS_COMPILE
export CROSS_COMPILE_BASE
export CROSS_COMPILE_LIBS
export CROSS_COMPILE_PREFIX
export CROSS_COMPILE_ROOTFS
export CROSS_GCC
export GCC_BASE
export GCC_PREFIX

# Define: TUXBUILDER_VERSION
# Version of the tuxbuilder toolchain.
# Must be defined.
#
# See: <TUXBUILDER_BASE>
ifndef TUXBUILDER_VERSION
  $(error Variable TUXBUILDER_VERSION should be defined)
endif

# Define: TUXBUILDER_BASE
# Installation directory of the tuxbuilder toolchain.
# Value: /opt/tuxbuilder-<TUXBUILDER_VERSION>
TUXBUILDER_BASE = /opt/tuxbuilder-$(TUXBUILDER_VERSION)

# Define: TUXBUILDER_TYPE
# C library type tuxbuilder toolchain.
# For example:
# - mipsel-unknown-linux-gnu
# - mipsel-unknown-linux-uclibcgnu
ifndef TUXBUILDER_TYPE
  $(error Variable TUXBUILDER_TYPE should be defined)
endif

ifndef CPUBUILDROOT
  define toolchain/tuxbuilder-$(TUXBUILDER_VERSION).env
    $(call ENVS_ENTER,toolchain/tuxbuilder-$(TUXBUILDER_VERSION).env)
    $(call setenvVar,GCC_BASE,$(TUXBUILDER_BASE)/$(TUXBUILDER_TYPE)/cross)
    $(call setenvVar,CROSS_COMPILE,mipsel-linux-)
    $(call setenvVar,CROSS_COMPILE_BASE,$$(GCC_BASE))
    $(call setenvVar,CROSS_COMPILE_ROOTFS,$(TUXBUILDER_BASE)/$(TUXBUILDER_TYPE)/rootfs)
    $(call setenvVar,CROSS_COMPILE_LIBS,$$(CROSS_COMPILE_ROOTFS)/usr/lib $$(CROSS_COMPILE_ROOTFS)/lib)
    $(call setenvVar,GCC_PREFIX,$(TUXBUILDER_TYPE))
    $(call setenvVar,CROSS_COMPILE_PREFIX,$$(GCC_PREFIX))
    $(call setenvVar,CROSS_GCC,$(TUXBUILDER_BASE)/$(TUXBUILDER_TYPE)/cross/$(TUXBUILDER_TYPE))
    $(call prependenvVar,PATH,$(if $(wildcard $(PRJROOT)/ccache_symlinks),$$(PRJROOT)/ccache_symlinks:,)$$(CROSS_COMPILE_BASE)/bin:)
    $(call ENVS_LEAVE,toolchain/tuxbuilder-$(TUXBUILDER_VERSION).env)
  endef
endif
