/*****************************************************************************/
/* Copyright (c) 2009 NXP B.V.  All rights are reserved.                     */
/*                                                                           */
/* Redistribution and use in source and binary forms, with or without        */
/* modification, are permitted provided that the following conditions        */
/* are met:                                                                  */
/*                                                                           */
/* Redistributions of source code must retain the above copyright            */
/* notice, this list of conditions and the following disclaimer.             */
/*                                                                           */
/* Redistributions in binary form must reproduce the above copyright         */
/* notice, this list of conditions and the following disclaimer in the       */
/* documentation and/or other materials provided with the distribution.      */
/*                                                                           */
/* Neither the name of NXP nor the names of its                              */
/* contributors may be used to endorse or promote products derived from      */
/* this software without specific prior written permission.                  */
/*                                                                           */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       */
/* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         */
/* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     */
/* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      */
/* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,     */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT          */
/* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,     */
/* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY     */
/* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT       */
/* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE         */
/* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH          */
/* DAMAGE.                                                                   */
/*                                                                           */
/*****************************************************************************/
/**
     @file   tmNxCompId.h
     @brief  Brief description of this h file.

     @b  Component:  n/a

     This is a temporary workaround to assist integration of STB810 components
     into the STB225 project. STB810 uses the newer 'tmNxCompId.h' naming
     convention, while existing STB225 include 'tmCompId.h'.

     Set your editor for 4 space indentation.
  *//*

Rev Date        Author      Comments
--------------------------------------------------------------------------------
  1 20060605    G.Lawrance  Original
  2 20080710    hnguyen     Add for TDA9984 compile
  3 20080814    laird       Add CID_MP4READ
  4 20090114    batelaan    Update copyright header
--------------------------------------------------------------------------------
*/
#ifndef TMNXCOMPID_H
#define TMNXCOMPID_H

#ifdef __cplusplus
extern "C"
{
#endif

/* -------------------------------------------------------------------------- */
/* Project include files:                                                     */
/* -------------------------------------------------------------------------- */
#include "tmCompId.h"

// Map tmNxCompId.h onto tmCompId.h
#define CID_STBSTREAMINGSYSTEM CID_COMP_STBSTREAMINGSYSTEM

#define CID_IID_FLAG_BITSHIFT   31
#define CID_ID_BITSHIFT         12
#define IID_ID_BITSHIFT         12

#define CID_FLAG                (0x1U << CID_IID_FLAG_BITSHIFT)
#define IID_FLAG                (0x0U << CID_IID_FLAG_BITSHIFT)

#define CID_ID(number)          ((number) << CID_ID_BITSHIFT)
#define CID_ID_BITMASK          (0x7FFFFU << CID_ID_BITSHIFT)

#define IID_ID(number)          ((number) << IID_ID_BITSHIFT)
#define IID_ID_BITMASK          (0x7FFFFU << IID_ID_BITSHIFT)

#define CID_BSL_HDMIRX          (CID_ID(0x823aU) | CID_FLAG)
#define CID_AACPENC             (CID_ID(0x823bU) | CID_FLAG)
#define CID_DL_HDMIRX           (CID_ID(0x823cU) | CID_FLAG)
#define CID_APP_HDMIRX          (CID_ID(0x823dU) | CID_FLAG)
#define CID_INFRA_HDMI          (CID_ID(0x823eU) | CID_FLAG)
#define CID_DL_HDMICEC          (CID_ID(0x823fU) | CID_FLAG)
#define CID_BSL_HDMITX          (CID_ID(0x8240U) | CID_FLAG)
#define CID_DL_HDMITX           (CID_ID(0x8241U) | CID_FLAG)
#define CID_APP_HDMITX          (CID_ID(0x8242U) | CID_FLAG)

#define CID_MP4READ             (CID_ID(0x808aUL) | CID_FLAG)
#define CID_AVI_READ_DIVX       (CID_ID(0x8019UL) | CID_FLAG)

#ifdef __cplusplus
}
#endif

#endif /* TMNXCOMPID_H */
