#
#                             IPSTB Project
#                   ---------------------------------
#
# Copyright (C) 2009 NXP B.V.,
# All Rights Reserved.
#
# Filename: $_TMROOT/project/project_environment.mk
#
# Included from end of $_TMROOT/sde/environment.mk
#
# Rev Date       Author      Comments
#-------------------------------------------------------------------------------
#   1 20080722   batelaan    Initial
#   2 20081009   batelaan    Scan makefile and diversity.mk file for use of CONFIG_* vars.
#   3 20081009   batelaan    Detect errors in .d file handling
#   4 20081029   batelaan    Check for correct use of frozen options
#   5 20081031   batelaan    Make it also work for apps
#   6 20090115   batelaan    Only do _sde_check_frozen_prjconfig_vars if $_PRJROOT_CI_RELEASE_SPEC) file exists.

_SDE_DEP_MAKEFILES += $(_TMROOT)/project/project_environment.mk

# Include $BUILDROOT/.prjconfig.
# This means that all SDE2 makefiles have access to our CONFIG_ variables.
ifeq "$(wildcard $(BUILDROOT)/.prjconfig)" ""
  $(warning $$BUILDROOT/.prjconfig does not exist!)
  $(error Run prjconfig first)
endif
include $(BUILDROOT)/.prjconfig
_SDE_TEMP_MAKEFILES += $(BUILDROOT)/.prjconfig

#-----------------------------------------------------------------------
# PRJFIXDEP
#-----------------------------------------------------------------------

# Define: PRJFIXDEP
# Path to prjfixdep executable, used by <_SDE_POST_COMPILE_FIXDEP>.
PRJFIXDEP := $(PRJROOT)/.prjconfig.TMP/prjfixdep

# Target: prjfixdep
# Makes <PRJFIXDEP>.
prjfixdep: $(PRJFIXDEP)
.PHONY: prjfixdep
$(PRJFIXDEP): $(PRJROOT)/etc/tools/prjconfig/Makefile $(PRJROOT)/etc/tools/prjconfig/prjfixdep.c
	$(MAKE) -C $(PRJROOT)/etc/tools/prjconfig DESTDIR=$(dir $@) prjfixdep

# Macro: _SDE_POST_COMPILE_FIXDEP
# Runs <> on GCC-generated dependency (.d) file.
_SDE_POST_COMPILE_FIXDEP=\
  $(_SDE_ECHO)$(PRJFIXDEP) $(@:.$(_SDE_O)=.d) $@ 'cmdline...'

# Macro: PRJCONFIG_ADD_DEPS
# Add dependencies to config variables.
#
# Usage: $(call PRJCONFIG_ADD_DEPS,var....)
PRJCONFIG_ADD_DEPS=\
  $(if $2,\
    $(warning Usage: $$(call PRJCONFIG_ADD_DEPS, var1 var2...)) \
    $(error PRJCONFIG_ADD_DEPS macro expects only one argument)) \
  $(eval _SDE_DEP_MAKEFILES+=$(wildcard $(patsubst %,$(BUILDROOT)/.config.d/%.h,$1)))


#-----------------------------------------------------------------------
# Detect any uses of CONFIG_* variables in makefile and diversity.mk files automatically.
# We skip calls to PRJCONFIG_ADD_DEPS that are already in the file.
PRJCONFIG_SCAN_FILE = \
  next if /^\s*\#/; \
  if (/PRJCONFIG_ADD_DEPS\s*,([\w\s]+)/) { map {$$done{$$_}++} split(" ", $$1); next; }  \
  s{\b(CONFIG_\w+)\b}{unless ($$done{$$1}++) {print "\$$(eval _PRJCONFIG_VARS_USED=$$1)\n\$$(eval _SDE_DEP_MAKEFILES+=\$$(wildcard $(BUILDROOT)/.config.d/$$1.h))\n";}}ge;
PRJCONFIG_SCAN_FILE_RESULTS := $(shell perl -w -n -e '$(PRJCONFIG_SCAN_FILE)' $(wildcard makefile Makefile diversity.mk))
$(eval $(PRJCONFIG_SCAN_FILE_RESULTS))

#-----------------------------------------------------------------------
# Check for correct use of frozen configuration options.
# *.dvar files record the dependencies on configuration variables.
_SDE_DVAR_FILES = \
  $(CXX_SOURCES:%.cpp=$(DIR_INTERM)/%.dvar) \
  $(C_SOURCES:%.c=$(DIR_INTERM)/%.dvar) \
  $(CFG_SOURCES:%.c=$(DIR_INTERM)/%.dvar)

ifeq "$(SDE_IN_SDE)" "."
  _PRJROOT_RELATIVE_CI_PATH = $(CURDIR:$(PRJROOT)/%=%)
else
  # We are building a sub-component, assume only the parent component is specified in the ci release spec file:
  _PRJROOT_RELATIVE_CI_PATH = $(_$(notdir $(SDE_IN_SDE))_DIR:$(PRJROOT)/%=%)
endif
#$(warning _PRJROOT_RELATIVE_CI_PATH = $(_PRJROOT_RELATIVE_CI_PATH))

_PRJROOT_CI_RELEASE_SPEC = $(PRJROOT)/etc/ci_release_stb225.spec

# Variable: PRJCONFIG_VARS_SKIP_FROZEN_CHECK
# If set by a component's makefile, then it should be an list of config variables to ignore
# during the check for use of non-frozen variables by a frozen component.

ifneq "$(wildcard $(_PRJROOT_CI_RELEASE_SPEC))" ""
  all: _sde_check_frozen_prjconfig_vars
  _sde_check_frozen_prjconfig_vars: _sde_objects
	$(if $(filter true,$(shell perl -MSDE2::CIReleaseSpec -e 'SDE2::CIReleaseSpec::printIsFrozenCI("prjfilter <$(_PRJROOT_CI_RELEASE_SPEC)|", "$(_PRJROOT_RELATIVE_CI_PATH)");')), \
		$(eval _PRJCONFIG_VARS_USED += $(sort $(shell cat /dev/null $(_SDE_DVAR_FILES)))) \
		$(eval _PRJCONFIG_VARS_NFREEZABLE := $(shell cat $(BUILDROOT)/.config.info.nfreezable)) \
		$(call _PRJCONFIG_VARS_USE_OF_NFREEZABLE,$(filter $(filter-out $(PRJCONFIG_VARS_SKIP_FROZEN_CHECK),$(_PRJCONFIG_VARS_USED)),$(_PRJCONFIG_VARS_NFREEZABLE))))

_PRJCONFIG_VARS_USE_OF_NFREEZABLE=$(if $1,\
  $(shell echo >&2 '**ERROR: Component $(_PRJROOT_RELATIVE_CI_PATH) is frozen') \
  $(shell echo >&2 '	(see $(_PRJROOT_CI_RELEASE_SPEC))') \
  $(shell echo >&2 '	but uses the following un-freezable configuration variable(s):') \
  $(shell echo >&2 '		$1') \
  $(shell echo >&2 '	(Note: this includes using the variable in makefiles, include files or comments.)') \
  $(shell echo >&2 '	Please correct this.') \
  exit 1)

endif
