#-----------------------------------------------------------------------
# COPYRIGHT (c) 2007 NXP Semiconductors 
#
# sde/config_check.mk
#
# Description: Checks the correct settings of CPUTYPE and OSTYPES for all supported configurations.
#              If wrong combinations are found, SDE2 gives an Error and exits
#
# Initial Author: Shivaraj P
# Maintained by : STA/SDM/SDE2 Team
#-----------------------------------------------------------------------
# changes to this file force rebuild
#-----------------------------------------------------------------------
_SDE_DEP_MAKEFILES +=   $(_TMROOT)/sde/config_check.mk

#-----------------------------------------------------------------------
# User configurable CPUTYPE changes (allow users to add new cpu/os types)
#-----------------------------------------------------------------------
-include $(_SDE_PROJECT)/$(DIR_CONFIG)/new_cpu_os_type.mk

#-----------------------------------------------------------------------
# Define supported CPU TYPES for all supported CPU CLASSES
#-----------------------------------------------------------------------
ifneq (,$(filter real,$(_TMTGTCPUCLASS)))
$(warning real CPU CLASS obsolete, Using dsprd CPU CLASS)
ifneq (,$(filter sat,$(_TMTOOLCHAIN)))
$(warning SATURN toolchain name is obsolete, using asdk)
override _TMTOOLCHAIN=asdk
endif
endif

_SDE_CPUTYPES_8051 += 8051
_SDE_CPUTYPES_arm += arm7 arm920t arm940t arm10 strongarm arm926EJS        \
                    arm946 arm922t arm720 arm966es arm968es arm1176JTZS    \
                    arm1020e arm1022e arm1026ejs arm1156t2fs arm1156t2s    \
                    armv4i
_SDE_CPUTYPES_dsprd += rd16023 rd16024 rd16025 rd16024c1 rd16025c1
_SDE_CPUTYPES_dspvd += vd32040
_SDE_CPUTYPES_hp += hp
_SDE_CPUTYPES_mips += 24kc 24kf 24kec 24kef 4KEc mips1 mips2 mips3 mips4   \
                      mips5 mips16 mips32 mips64 \
                      r3940 r4300 r4640 r1910 r4450 r4000 r3900 r3000
_SDE_CPUTYPES_tm += tm1100 tm1300 tm32 tm3260 tm3270 tm3271 tm2270 tm5250
_SDE_CPUTYPES_x86 += i486

_SDE_SUPPORTED_CPUTYPES = $(strip $(_SDE_CPUTYPES_8051) $(_SDE_CPUTYPES_arm) \
                         $(_SDE_CPUTYPES_dsprd) $(_SDE_CPUTYPES_dspvd)       \
                         $(_SDE_CPUTYPES_hp) $(_SDE_CPUTYPES_mips)           \
                         $(_SDE_CPUTYPES_tm) $(_SDE_CPUTYPES_x86))

ifeq (,$(filter $(_TMTGTCPUTYPE),$(_SDE_SUPPORTED_CPUTYPES)))
$(error Unknown _TMTGTCPUTYPE. Supported CPU: $(_SDE_SUPPORTED_CPUTYPES))
endif

#-----------------------------------------------------------------------
# Define supported OS TYPES for all supported OS CLASSES
#-----------------------------------------------------------------------
_SDE_OSTYPES_ce += ce212 ce300 ce600
_SDE_OSTYPES_cexec += cexec
_SDE_OSTYPES_ecos += ecos
_SDE_OSTYPES_integrity += integrity
_SDE_OSTYPES_linux += linux
_SDE_OSTYPES_mtos += mtos
_SDE_OSTYPES_nt += nt4
_SDE_OSTYPES_nucleus += nucleus
_SDE_OSTYPES_nullos += nullos
_SDE_OSTYPES_oscan += oscan
_SDE_OSTYPES_psos += psos200 psos250
_SDE_OSTYPES_ucos += ucos
_SDE_OSTYPES_vxworks += vxworks vxworks540 vxworks550

_SDE_SUPPORTED_OSTYPES = $(strip $(_SDE_OSTYPES_ce) $(_SDE_OSTYPES_cexec)   \
                         $(_SDE_OSTYPES_ecos) $(_SDE_OSTYPES_integrity)     \
                         $(_SDE_OSTYPES_linux) $(_SDE_OSTYPES_mtos)         \
                         $(_SDE_OSTYPES_nt) $(_SDE_OSTYPES_nucleus)         \
                         $(_SDE_OSTYPES_nullos) $(_SDE_OSTYPES_oscan)       \
			 $(_SDE_OSTYPES_psos) $(_SDE_OSTYPES_ucos)          \
                         $(_SDE_OSTYPES_vxworks))

ifeq (,$(filter $(_TMTGTOS),$(_SDE_SUPPORTED_OSTYPES)))
$(error Unknown _TMTGTOS. Supported OS types: $(_SDE_SUPPORTED_OSTYPES))
endif

#-----------------------------------------------------------------------
# Set the CPU CLASS
#-----------------------------------------------------------------------
ifneq (,$(filter $(_TMTGTCPUTYPE),$(_SDE_CPUTYPES_8051)))
_TMTGTCPUCLASS=8051
else
ifneq (,$(filter $(_TMTGTCPUTYPE),$(_SDE_CPUTYPES_arm)))
_TMTGTCPUCLASS=arm
else
ifneq (,$(filter $(_TMTGTCPUTYPE),$(_SDE_CPUTYPES_dsprd)))
_TMTGTCPUCLASS=dsprd
else
ifneq (,$(filter $(_TMTGTCPUTYPE),$(_SDE_CPUTYPES_dspvd)))
_TMTGTCPUCLASS=dspvd
else
ifneq (,$(filter $(_TMTGTCPUTYPE),$(_SDE_CPUTYPES_hp)))
_TMTGTCPUCLASS=hp
else
ifneq (,$(filter $(_TMTGTCPUTYPE),$(_SDE_CPUTYPES_mips)))
_TMTGTCPUCLASS=mips
else
ifneq (,$(filter $(_TMTGTCPUTYPE),$(_SDE_CPUTYPES_x86)))
_TMTGTCPUCLASS=x86
else
ifneq (,$(filter $(_TMTGTCPUTYPE),$(_SDE_CPUTYPES_tm)))
_TMTGTCPUCLASS=tm
else
DIR_CONFIG=default
endif
endif
endif
endif
endif
endif
endif
endif

#-----------------------------------------------------------------------
# Set the OS CLASS
#-----------------------------------------------------------------------
ifneq (,$(filter $(_TMTGTOS),$(_SDE_OSTYPES_ce)))
_TMTGTOSCLASS=ce
else
ifneq (,$(filter $(_TMTGTOS),$(_SDE_OSTYPES_cexec)))
_TMTGTOSCLASS=cexec
else
ifneq (,$(filter $(_TMTGTOS),$(_SDE_OSTYPES_ecos)))
_TMTGTOSCLASS=ecos
else
ifneq (,$(filter $(_TMTGTOS),$(_SDE_OSTYPES_integrity)))
_TMTGTOSCLASS=integrity
else
ifneq (,$(filter $(_TMTGTOS),$(_SDE_OSTYPES_linux)))
_TMTGTOSCLASS=linux
else
ifneq (,$(filter $(_TMTGTOS),$(_SDE_OSTYPES_mtos)))
_TMTGTOSCLASS=mtos
else
ifneq (,$(filter $(_TMTGTOS),$(_SDE_OSTYPES_nt)))
_TMTGTOSCLASS=nt
else
ifneq (,$(filter $(_TMTGTOS),$(_SDE_OSTYPES_nucleus)))
_TMTGTOSCLASS=nucleus
else
ifneq (,$(filter $(_TMTGTOS),$(_SDE_OSTYPES_nullos)))
_TMTGTOSCLASS=nullos
else
ifneq (,$(filter $(_TMTGTOS),$(_SDE_OSTYPES_oscan)))
_TMTGTOSCLASS=oscan
else
ifneq (,$(filter $(_TMTGTOS),$(_SDE_OSTYPES_psos)))
_TMTGTOSCLASS=psos
else
ifneq (,$(filter $(_TMTGTOS),$(_SDE_OSTYPES_ucos)))
_TMTGTOSCLASS=ucos
else
ifneq (,$(filter $(_TMTGTOS),$(_SDE_OSTYPES_vxworks)))
_TMTGTOSCLASS=vxworks
else
override DIR_CONFIG=default
endif
endif
endif
endif
endif
endif
endif
endif
endif
endif
endif
endif
endif

#-----------------------------------------------------------------------
# Check for supported configurations
#-----------------------------------------------------------------------
_SDE_TMP_FILES = $(foreach config,$(wildcard $(DIR_SDE)/*.*),$(notdir $(config)))
_SDE_SUPPORTED_CONFIGS += $(filter-out $(_SDE_TMP_FILES),$(foreach config,$(wildcard $(DIR_SDE)/*_*),$(notdir $(config))))


ifdef _TMTOOLCHAIN
include $(DIR_SDE)/default_toolchain.mk
ifeq ($(_TMTOOLCHAIN),$(_SDE_DEFAULT_TOOL))
DIR_CONFIG := $(_TMTGTCPUCLASS)_$(_TMTGTOSCLASS)
else
DIR_CONFIG := $(_TMTGTCPUCLASS)$(_TMTOOLCHAIN)_$(_TMTGTOSCLASS)
endif
else
DIR_CONFIG := $(_TMTGTCPUCLASS)$(_TMTOOLCHAIN)_$(_TMTGTOSCLASS)
endif

_SDE_DIR_CONFIG := $(_TMTGTCPUCLASS)$(_TMTOOLCHAIN)_$(_TMTGTOSCLASS)

ifeq (,$(filter $(DIR_CONFIG),$(_SDE_SUPPORTED_CONFIGS)))
override DIR_CONFIG:=default
$(error Unsupported configuration. Supported configurations: $(_SDE_SUPPORTED_CONFIGS))
endif

chkconfig:
ifneq ($(_SDE_ERROR),)
	$(_SDE_ECHO)$(ECHO) $(ECHOFLAGS) "$(_SDE_ERROR)"
	$(_SDE_ECHO)exit 1
endif
