_SDE_DIV_DEBUG?=_sde_disabled
# Run make with_SDE_DIV_DEBUG=warning to enable debug output of diversity macros.

#-----------------------------------------------------------------------
# Interface diversity
#-----------------------------------------------------------------------

# Macro: INTERFACE_DIVERSITY
# Declare that the component has the specified interface diversities.
# The current CI inherits any interface diversity of $(REQUIRES).
#
# Usage:-
# $(call INTERFACE_DIVERSITY, tmCompName, divA divB divC...)
INTERFACE_DIVERSITY=$(eval $(call INTERFACE_DIVERSITY1,$(strip $(1)),$(2)))
define INTERFACE_DIVERSITY1
  $$($(_SDE_DIV_DEBUG) >> INTERFACE_DIVERSITY: $1, $2)
  $$(call IMPLEMENTATION_DIVERSITY,$(_SDE_CI),$(2))
  $(if $(filter $(_SDE_CI),$(1)),,_$(_SDE_CI)_DIVERSITIES+=$$(_$(1)_DIVERSITIES))
  $$($(_SDE_DIV_DEBUG) _$(_SDE_CI)_DIVERSITIES is now $$(_$(_SDE_CI)_DIVERSITIES))
  $$($(_SDE_DIV_DEBUG) << INTERFACE_DIVERSITY: $1, $2)
endef

#-----------------------------------------------------------------------
# Implementation diversity
#-----------------------------------------------------------------------

# Macro: IMPLEMENTATION_DIVERSITY
# Declare that the component has the specified implementation diversities.
# The current CI inherits any interface diversity of $(REQUIRES).
#
# Usage:-
# $(call IMPLEMENTATION_DIVERSITY, tmCompName, divA divB divC...)
IMPLEMENTATION_DIVERSITY=$(eval $(call IMPLEMENTATION_DIVERSITY1,$(strip $(1)),$(2)))
define IMPLEMENTATION_DIVERSITY1
  $$($(_SDE_DIV_DEBUG) >> IMPLEMENTATION_DIVERSITY: $1, $2)
  _$(1)_DIVERSITIES:=$$(strip $$(foreach d,$(2),$$(if $$(findstring _$$(d)_,$$(_TMDIVERSITY)),$$(d))))
  $$($(_SDE_DIV_DEBUG) _$(1)_DIVERSITIES is now $$(_$(1)_DIVERSITIES))
  _$(1)_DIVERSITY:=$$(_$(1)_DIVERSITY)$$(foreach d,$(2),$$(findstring _$$(d)_,$$(_TMDIVERSITY)))
  _$(1)_DIVERSITY:=_$$(subst $$(_space_),_,$$(sort $$(subst _, ,$$(_$(1)_DIVERSITY))))
  _$(1)_DIVERSITY:=$$(if $$(filter _,$$(_$(1)_DIVERSITY)),,$$(_$(1)_DIVERSITY))
  $$($(_SDE_DIV_DEBUG) _$(1)_DIVERSITY is now $$(_$(1)_DIVERSITY))
  $$($(_SDE_DIV_DEBUG) << IMPLEMENTATION_DIVERSITY: $1, $2)
endef
ifdef jfkdjfkdjfdjf
  $(foreach d,$(2), $(eval _$(1)_DIVERSITY:=$(_$(1)_DIVERSITY)$(findstring _$(d)_,$(_TMDIVERSITY))))
  $(eval _$(1)_DIVERSITY:=_$(subst $(_space_),_,$(sort $(subst _, ,$(_$(1)_DIVERSITY)))))
  $(if $(filter _,$(_$(1)_DIVERSITY)),$(eval _$(1)_DIVERSITY=))
endif

#-----------------------------------------------------------------------
# Utility macros used by ASSERT_DIVERSITY
#-----------------------------------------------------------------------
# $(call _SDE_CHECK_XXX_OF,$1,$2) returns a non-empty string
# if XXX_OF $2 occurs in $1

# Non-empty if none of $2 occur in $1.
_SDE_CHECK_NUL_OF=$(if $(filter $2,$1),,true)

# Non-empty if exactly one of $2 occur in $1.
_SDE_CHECK_ONE_OF=$(filter 1,$(words $(sort $(filter $2,$1))))

# Non-empty if one or more of $2 occur in $1.
_SDE_CHECK_ANY_OF=$(filter-out 0,$(words $(filter $2,$1)))

# Non-empty if all of $2 occur in $1.
_SDE_CHECK_ALL_OF=$(if $2,$(filter $(words $(sort $2)),$(words $(sort $(filter $2,$1)))),true)

# Same as _SDE_CHECK_ONE_OF
_SDE_CHECK_REQUIRES_ONE_OF=$(call _SDE_CHECK_ONE_OF,$1,$2)

# Same as _SDE_CHECK_ANY_OF
_SDE_CHECK_REQUIRES_ANY_OF=$(call _SDE_CHECK_ANY_OF,$1,$2)

# Same as _SDE_CHECK_ALL_OF
_SDE_CHECK_REQUIRES_ALL_OF=$(call _SDE_CHECK_ALL_OF,$1,$2)

# Opposite of _SDE_CHECK_ANY_OF
_SDE_CHECK_EXCLUDES_ANY_OF=$(if $(call _SDE_CHECK_ANY_OF,$1,$2),,true)

# Opposite of _SDE_CHECK_ALL_OF
_SDE_CHECK_EXCLUDES_ALL_OF=$(if $(call _SDE_CHECK_ALL_OF,$1,$2),,true)

#-----------------------------------------------------------------------
# _TMDIVERSITY check macro ASSERT_DIVERSITY
#-----------------------------------------------------------------------

# Macro: ASSERT_DIVERSITY
# Asserts a requirement on the value of <_TMDIVERSITY>.
# There are 2 and 4 argument versions of this macro:
# - $(call ASSERT_DIVERSITY, {CHECKTYPE}, {diversity}...)
# This is the 2-argument version of ASSERT_DIVERSITY.
# {CHECKTYPE} specifies the assertion type, which is one of:
#     - NUL_OF: asserts the presence of none of the specified diversities.
#     - ONE_OF: asserts the presence of exactly one of the specified diversities.
#     - ANY_OF: asserts the presence of one or more of the specified diversities.
#     - ALL_OF: asserts the presence of all of the specified diversities.
# - $(call ASSERT_DIVERSITY, {IFCHECKTYPE}, {IFdiversity}..., {THENCHECKTYPE}, {THENdiversity}...)
# This is the 4-argument version of ASSERT_DIVERSITY.
# {IFCHECKTYPE} specifies the condition which must be true before
# the {THENCHECKTYPE} is checked. {IFCHECKTYPE} is one of:
#     - ANY_OF: the rule applies if any of the specified IFdiversity's is present.
#     - ALL_OF: the rule applies if all of the specified IFdiversity's are present.
# {THENCHECKTYPE} specifies the condition which must be true
# if {IFCHECKTYPE} is true. {THENCHECKTYPE} is one of:
#     - REQUIRES_ONE_OF: asserts presence of exactly one of the specifies THENdiversity's.
#     - REQUIRES_ANY_OF: asserts the presence of one or more of the specified THENdiversity's.
#     - REQUIRES_ALL_OF: asserts the presence of all of the specified THENdiversity's.
#     - EXCLUDES_ANY_OF: asserts the absence of any of the specified THENdiversity's.
#     - EXCLUDES_ALL_OF: asserts the absence of all of the specified THENdiversity's.
#
# Examples of unconditional assertions:
# $(call ASSERT_DIVERSITY, NUL_OF, a b)
# $(call ASSERT_DIVERSITY, ONE_OF, a b)
# $(call ASSERT_DIVERSITY, ANY_OF, a b)
# $(call ASSERT_DIVERSITY, ALL_OF, a b)

# Examples of conditional assertions:
# $(call ASSERT_DIVERSITY, ANY_OF, a b, REQUIRES_ONE_OF, 1 2)
# $(call ASSERT_DIVERSITY, ANY_OF, a b, REQUIRES_ANY_OF, 1 2)
# $(call ASSERT_DIVERSITY, ANY_OF, a b, REQUIRES_ALL_OF, 1 2)
# $(call ASSERT_DIVERSITY, ANY_OF, a b, EXCLUDES_ANY_OF, 1 2)
# $(call ASSERT_DIVERSITY, ANY_OF, a b, EXCLUDES_ALL_OF, 1 2)
# $(call ASSERT_DIVERSITY, ALL_OF, a b, REQUIRES_ONE_OF, 1 2)
# $(call ASSERT_DIVERSITY, ALL_OF, a b, REQUIRES_ANY_OF, 1 2)
# $(call ASSERT_DIVERSITY, ALL_OF, a b, REQUIRES_ALL_OF, 1 2)
# $(call ASSERT_DIVERSITY, ALL_OF, a b, EXCLUDES_ANY_OF, 1 2)
# $(call ASSERT_DIVERSITY, ALL_OF, a b, EXCLUDES_ALL_OF, 1 2)

ASSERT_DIVERSITY=\
  $(call ASSERT_DIVERSITY_REPORT,\
    $(if $(strip $(call _SDE_CHECK_$(strip $1),$(sort $(subst _, ,$(_TMDIVERSITY))),$2)),\
      $(if $(strip $3 $4),\
        $(call _SDE_CHECK_$(strip $3),$(sort $(subst _, ,$(_TMDIVERSITY))),$4),\
        true),\
      $(if $(strip $3 $4),true)),\
    $1,$2,$3,$4)


# This uses $(error) to fail immediately while parsing the diversity.mk file.
#ASSERT_DIVERSITY_REPORT=$(if $(strip $1),,$(error ASSERT_DIVERSITY check failed: $(strip $2 $3 $4 $5)))
# This extends the diversity target to fail only when invoking the diversity target.
ASSERT_DIVERSITY_REPORT=$(if $(strip $1),,$(eval diversity::;@$(ECHO) "$(lastword $(MAKEFILE_LIST)):";echo "ASSERT_DIVERSITY check failed: $(strip $2 $3 $4 $5)"; exit 1))

diversity::;
