#-----------------------------------------------------------------------
# COPYRIGHT (c) 2007 NXP Semiconductors 
#
# sde/environment.mk
#
# Description: Defines dependency check rules
#              Defines rules for generating objects
#              Defines rules for making option files
#
# Initial Author: Rogier Wester
# Maintained by : STA/SDM/SDE2 Team
#-----------------------------------------------------------------------
# Strip all the whitespaces in environment settings
#-----------------------------------------------------------------------

-include $(_TMROOT)/project/project_environment0.mk

#-----------------------------------------------------------------------
# overwrite any rules below
#-----------------------------------------------------------------------
default_rule : makelist all
$(foreach tmvar,$(filter _TM%,$(.VARIABLES)),$(eval \
  $(tmvar) := $(strip $(subst \,/,$($(tmvar))))))

#-----------------------------------------------------------------------
# Convert cygwin path to windows format
#-----------------------------------------------------------------------
_empty_ :=
_space_ :=      $(_empty_) $(_empty_)
ifeq ($(UNAME),cygwin)
ifneq ($(filter cygdrive,$(subst /,$(_space_),$(CURDIR))),)
override CURDIR:=$(shell cygpath -m $(CURDIR))
endif
endif

_SDE_CI=$(notdir $(DIR_LOCAL))

#ifneq "$(_SDE_CI)" "$(notdir $(CURDIR))"
#  $(warning "The variable DIR_LOCAL ($(DIR_LOCAL)) should end with $(notdir $(CURDIR))")
#endif

#-----------------------------------------------------------------------
# changes to this file force rebuild
#-----------------------------------------------------------------------
_SDE_DEP_MAKEFILES +=   $(_TMROOT)/sde/environment.mk

#-----------------------------------------------------------------------
# Look for Project specific Configuration
#-----------------------------------------------------------------------
DIR_SDE = $(_TMROOT)/sde
_SDE_PROJECT = $(_TMROOT)/project
DIR_PROJECT = $(DIR_SDE)

ifneq ($(_TMPROJECT),$(_TMROOT)/project)
ifneq ($(_TMPROJECT),)
ifneq ($(wildcard $(_TMPROJECT)),)
_SDE_PROJECT = $(_TMPROJECT)
ifneq ($(_TMBSL),)
DIR_PROJECT = $(_TMPROJECT)
endif
endif
endif
endif

_TMDIVERSITYMKLOC ?= tool/hosttarget/mk
ifneq ($(_BLDNOTNHSW),)
_TMDIVERSITYMKLOC = inc/mk
endif

ifndef UNAME
UNAME_TMP=$(strip $(shell uname -s))
ifeq (NT,$(findstring NT,$(UNAME_TMP)))
UNAME=cygwin
else
ifeq (Linux,$(filter Linux,$(UNAME_TMP)))
UNAME=Linux
endif
endif
endif

ifndef HOST_PLATFORM_MK
ifneq (,$(wildcard $(_SDE_PROJECT)/$(UNAME).mk))
HOST_PLATFORM_MK?=$(_SDE_PROJECT)/$(UNAME).mk
else
$(warning $(_SDE_PROJECT)/$(UNAME).mk not found.)
endif
endif

ifndef HOST_PLATFORM_MK
ifdef _TMSITE
$(warning Looking for $(_SDE_PROJECT)/sites/$(_TMSITE)/$(UNAME).mk)
ifneq (,$(wildcard $(_SDE_PROJECT)/sites/$(_TMSITE)/$(UNAME).mk))
HOST_PLATFORM_MK=$(_SDE_PROJECT)/sites/$(_TMSITE)/$(UNAME).mk
else
$(warning $(_SDE_PROJECT)/sites/$(_TMSITE)/$(UNAME).mk not found.)
endif
endif
endif

ifndef HOST_PLATFORM_MK
$(warning  Including $(_TMROOT)/project/default_host.mk.)
HOST_PLATFORM_MK=$(_TMROOT)/project/default_host.mk
endif

include $(DIR_SDE)/diversity_common.mk

-include $(HOST_PLATFORM_MK)

ifeq ($(UNAME),cygwin)
ECHOFLAGS =-en
else
ifeq ($(filter $(UNAME),linux Linux),$(UNAME))
ECHOFLAGS =-en
endif
endif

#-----------------------------------------------------------------------
# Check for correct combination of CPU/OS type and class
#-----------------------------------------------------------------------
include $(DIR_SDE)/config_check.mk

# SDE2 Version 2.3_SP2
_SDE_VERSION=2.3_SP2

#-----------------------------------------------------------------------
# Determine TCS version. TCS is required for all builds of CPU class
# tm, of course, but headers from TCS may also be needed for CPU class
# mips, x86, etc.. If a valid TCS does not exist for mips or x86 builds
# the build in the component that needs it but not here.
#-----------------------------------------------------------------------

# The file TCS_VERSION_* is only in TCS 4.41 or TCS4.5. It's a little
# drastic, but.....
    ifeq ($(DIR_CONFIG),tm_psos)
    ifndef TCS
        $(error TCS not set.)
    endif	
    endif

_SDE_TCSVERSION := $(strip                                                    \
    $(patsubst                                                                \
        TCS_VERSION_%,                                                        \
        %,                                                                    \
        $(notdir                                                              \
            $(wildcard $(TCS)/TCS_VERSION_*)                                  \
        )                                                                     \
    )                                                                         \
)

ifeq            (,$(_SDE_TCSVERSION))
  ifeq ($(DIR_CONFIG),tm_psos)
  $(error TCS in $(TCS) is not a supported version.)
  endif
else   # ifeq   (,$(_SDE_TCSVERSION))
  ifneq           (,$(filter 4.5_PR1 4.5PR2,$(_SDE_TCSVERSION)))
    $(error TCS $(_SDE_TCSVERSION) in $(TCS) is not a supported version.)
  else   # ifneq  (,$(filter 4.5_PR1 4.5PR2,$(_SDE_TCSVERSION)))

    _SDE_TCSVERSION_MAJOR := $(basename $(_SDE_TCSVERSION))
ifeq ($(patsubst .0%,0,$(suffix $(_SDE_TCSVERSION))),0)
	_SDE_TCSVERSION_MINOR := 00
else
ifeq ($(patsubst .1%,1,$(suffix $(_SDE_TCSVERSION))),1)
	_SDE_TCSVERSION_MINOR := 10
else
ifeq ($(patsubst .2%,2,$(suffix $(_SDE_TCSVERSION))),2)
	_SDE_TCSVERSION_MINOR := 20
else
ifeq ($(patsubst .3%,3,$(suffix $(_SDE_TCSVERSION))),3)
	_SDE_TCSVERSION_MINOR := 30
else
ifeq ($(patsubst .4%,4,$(suffix $(_SDE_TCSVERSION))),4)
	_SDE_TCSVERSION_MINOR := 40
else
ifeq ($(patsubst .5%,5,$(suffix $(_SDE_TCSVERSION))),5)
	_SDE_TCSVERSION_MINOR := 50
else
ifeq ($(patsubst .6%,6,$(suffix $(_SDE_TCSVERSION))),6)
	_SDE_TCSVERSION_MINOR := 60
else
ifeq ($(patsubst .7%,7,$(suffix $(_SDE_TCSVERSION))),7)
	_SDE_TCSVERSION_MINOR := 70
else
ifeq ($(patsubst .8%,8,$(suffix $(_SDE_TCSVERSION))),8)
	_SDE_TCSVERSION_MINOR := 80
else
ifeq ($(patsubst .9%,9,$(suffix $(_SDE_TCSVERSION))),9)
	_SDE_TCSVERSION_MINOR := 90
endif
endif
endif
endif
endif
endif
endif
endif
endif
endif

    _SDE_TCSVERSION := $(strip $(_SDE_TCSVERSION_MAJOR)$(_SDE_TCSVERSION_MINOR))
    _SDE_USE_TCS_TMPROCESSOR := 1
    _SDE_RESERVE_TCS_TIMER := 1

# _SDE_TCSVERSION is now:
#   450 for 4.5x
#   460 for 4.6x
#   500 for 5.0x
#   and so on

  endif  # ifneq  (,$(filter 4.5_PR1 4.5PR2,$(_SDE_TCSVERSION)))
endif  # ifeq   (,$(_SDE_TCSVERSION))

#Define _SDE_ARSUFFIX based on pre-defined suffixes
include $(DIR_SDE)/divsuffix.mk


#-----------------------------------------------------------------------
#by default no output
#-----------------------------------------------------------------------
ifneq ($(_TMECHO),)
override _SDE_ECHO=
else
ifneq ($(_ECHOMAKELINES),) # old environment variable
override _SDE_ECHO=
else
override _SDE_ECHO=@
endif
endif


#-----------------------------------------------------------------------
# Calculate the location to store libraries and executables
#-----------------------------------------------------------------------

ifeq ($(SDE_IN_SDE),)
SDE_IN_SDE=.
endif

ifeq ($(_TMTGTBUILDROOT),)
_SDE_TMTGTBUILDROOT=$(_TMROOT)
_SDE_DIR_BUILD       =.
else
_SDE_TMTGTBUILDROOT=$(_TMTGTBUILDROOT)
_SDE_DIR_BUILD       =$(_TMTGTBUILDROOT)/$(SDE_IN_SDE)/$(DIR_LOCAL)
endif

# calculate the directory name starting from the root
_SDE_DIR_BUILD_ROOT = $(_SDE_TMTGTBUILDROOT)/comps/generated


_SDE_LIB_CONFIGURATION  = $(_SDE_DIR_CONFIG)_$(_TMTGTENDIAN)_$(_TMTGTCPUTYPE)
_SDE_DIR_LIB_EXT        = lib/$(_SDE_LIB_CONFIGURATION)
DIR_INTERM_LIB          = $(_SDE_DIR_BUILD)/tmp/$(_SDE_LIB_CONFIGURATION)$(LIB_SUFFIX)$(_SDE_ARSUFFIX)

ifeq ($(SDE_IN_SDE),.)
_SDE_DIR_LIB            = $(_SDE_DIR_BUILD_ROOT)/$(_SDE_DIR_LIB_EXT)
else
_SDE_DIR_LIB            = $(_SDE_TMTGTBUILDROOT)/$(SDE_IN_SDE)/$(_SDE_DIR_LIB_EXT)
endif


ifeq ($(_TMTOOLCHAIN),$(_SDE_DEFAULT_TOOL))
_SDE_LIB_CONFIGURATION_DEF  = $(_TMTGTCPUCLASS)_$(_TMTGTOSCLASS)_$(_TMTGTENDIAN)_$(_TMTGTCPUTYPE)
_SDE_DIR_LIB_EXT_DEF        = lib/$(_SDE_LIB_CONFIGURATION_DEF)

ifeq ($(SDE_IN_SDE),.)
_SDE_DIR_LIB_DEF            = $(_SDE_DIR_BUILD_ROOT)/$(_SDE_DIR_LIB_EXT_DEF)
else
_SDE_DIR_LIB_DEF            = $(_SDE_TMTGTBUILDROOT)/$(SDE_IN_SDE)/$(_SDE_DIR_LIB_EXT_DEF)
endif
endif

ifeq ($(_TMTGTCPUCLASS),tm)
_SDE_TCSHOST=$(_TMTCSHOST)
DIR_INTERM_EXE          = $(_SDE_DIR_BUILD)/tmp/$(_SDE_LIB_CONFIGURATION)$(LIB_SUFFIX)$(_SDE_ARSUFFIX)_$(_SDE_TCSHOST)$(_TMBSL)$(_TMDIVERSITY)
_SDE_DIR_BIN_EXTPATH    = bin/$(_SDE_DIR_CONFIG)_$(_TMLINKTYPE)_$(_TMTGTENDIAN)_$(_TMTGTCPUTYPE)$(LIB_SUFFIX)$(_SDE_ARSUFFIX)_$(_SDE_TCSHOST)$(_TMBSL)$(_TMDIVERSITY)
else
DIR_INTERM_EXE  = $(_SDE_DIR_BUILD)/tmp/$(_SDE_LIB_CONFIGURATION)$(LIB_SUFFIX)$(_SDE_ARSUFFIX)$(_TMBSL)$(_TMDIVERSITY)
_SDE_DIR_BIN_EXTPATH    = bin/$(_SDE_DIR_CONFIG)_$(_TMLINKTYPE)_$(_TMTGTENDIAN)_$(_TMTGTCPUTYPE)$(LIB_SUFFIX)$(_SDE_ARSUFFIX)$(_TMBSL)$(_TMDIVERSITY)
endif

_SDE_DIR_LIB_LOCAL      = $(_SDE_DIR_BUILD)/$(_SDE_DIR_LIB_EXTPATH)
_SDE_DIR_BIN            = $(_SDE_DIR_BUILD)/$(_SDE_DIR_BIN_EXTPATH)


ifeq ($(UNAME),cygwin)
_SDE_HOST_EXE_EXT:=.exe
# else leave it empty
endif

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# sort and remove redundancies in _TMDIVERSITY

ifneq ($(_TMDIVERSITY), )
_empty_ :=
_space_ :=      $(_empty_) $(_empty_)
_TMDIVERSITY := _$(subst $(_space_),_,$(sort $(subst _, ,$(_TMDIVERSITY))))_
endif

_SDE_DIVERSITY = $(_TMDIVERSITY:%_=%)$(REL_COMPS_SUFFIX)

ifeq ($(_SDE_DIVERSITY), )
_SDE_DIVERSITY=.
endif

#-----------------------------------------------------------------------
# Set TCS core and target which determines the chip of our build. 
# This will then be used to set the correct include search paths and 
# libraries in other makefiles. If the target is not a real chip
# the appropriate simulator is listed below and the corresonding
# value for _TMTCSHOST must be set.
#
# If _TMDIVERSITY containing one of the following substrings
# the following core and target is set. 
#
#   Substring              Core          Target               Simulator 
#   ---------              -----         ------               ---------
#
#   _pnx1300_              tm32          pnx1300
#   _pnx1500_              tm3260        pnx1500
#   _pnx1700_              tm5250        pnx1700
#   _pnx8520_              tm2270        pnx8520
#   _pnx8525_              tm32          pnx8525
#   _pnx8535_              tm3270        pnx8535
#   _pnx8550_              tm3260	     pnx8550
#									
#   _tm1_                  tm32          tm1                  tmsim
#   _tm32Minimal_          tm32	         tm32Minimal          tm32sim
#   _tm3260Minimal_        tm3260	     tm3260Minimal        tm3260sim
#   _tm3260c2Minimal_      tm3260        tm3260c2Minimal      tm3260c2sim
#   _tm5250Minimal_        tm5250	     tm5250Minimal        tm5250sim
#   _tm2270Minimal_        tm2270	     tm2270Minimal        tm2270sim
#   _tm3270Minimal_        tm3270	     tm3270Minimal        tm3270sim
#   _tm3270r128Minimal_    tm3270r128	 tm3270r128Minimal    tm3270sim
#
# If no valid _TMDIVERSITY is found we next check _TMTGTCPUTYPE for
# one of the following:
#
#   tm32        use tm32   core, target pnx8525
#   tm3260      use tm3260 core, target pnx8550
#   tm2270      use tm2270 core, target pnx8535
#   tm5250      use tm5250 core, target pnx1700
#
# If no valid _TMTGTCPUTYPE is found we default to _TMTGTCPUTYPE of
# tm3260 and set core and target as above.
#-----------------------------------------------------------------------
ifneq ($(_SDE_TCSVERSION),)

#-----------------------------------------------------------------------
# Pick up floating precision option, if any, from diversity.
#-----------------------------------------------------------------------
_t1 :=		       $(findstring _fp32_,$(_TMDIVERSITY))
_t1 :=		 $(_t1)$(findstring _fp64_,$(_TMDIVERSITY))

ifeq            ($(_t1),_fp64_)

_SDE_TCS_FP =           fp64

else   # ifeq   ($(_t1),_fp64_)

_SDE_TCS_FP =           fp32

endif  # ifeq   ($(_t1),_fp64_)

_t1 :=

_t :=		      $(findstring _pnx1300_,         $(_TMDIVERSITY))
_t :=		 $(_t)$(findstring _pnx1500_,         $(_TMDIVERSITY))
_t :=		 $(_t)$(findstring _pnx1700_,         $(_TMDIVERSITY))
_t :=		 $(_t)$(findstring _pnx8520_,         $(_TMDIVERSITY))
_t :=		 $(_t)$(findstring _pnx8525_,         $(_TMDIVERSITY))
_t :=		 $(_t)$(findstring _pnx8535_,         $(_TMDIVERSITY))
_t :=		 $(_t)$(findstring _pnx8550_,         $(_TMDIVERSITY))
_t :=		 $(_t)$(findstring _tm32Minimal_,     $(_TMDIVERSITY))
_t :=		 $(_t)$(findstring _tm2270Minimal_,   $(_TMDIVERSITY))
_t :=		 $(_t)$(findstring _tm3260Minimal_,   $(_TMDIVERSITY))
_t :=		 $(_t)$(findstring _tm3260c2Minimal_, $(_TMDIVERSITY))
_t :=		 $(_t)$(findstring _tm5250Minimal_,   $(_TMDIVERSITY))

ifeq            ($(_t),_pnx1300_)

_SDE_TCS_CORE =         tm32
_SDE_TCS_TMTARGET =     pnx1300

else   # ifeq   ($(_t),_pnx1300_)

ifeq            ($(_t),_pnx1500_)

_SDE_TCS_CORE =         tm3260
_SDE_TCS_TMTARGET =     pnx1500

else   # ifeq   ($(_t),_pnx1500_)

ifeq            ($(_t),_pnx1700_)

_SDE_TCS_CORE =         tm5250
_SDE_TCS_TMTARGET =     pnx1700

else   # ifeq   ($(_t),_pnx1700_)

ifeq            ($(_t),_pnx8520_)

_SDE_TCS_CORE =         tm2270
_SDE_TCS_TMTARGET =     pnx8520

else   # ifeq   ($(_t),_pnx8520_)

ifeq            ($(_t),_pnx8525_)

_SDE_TCS_CORE =         tm32
_SDE_TCS_TMTARGET =     pnx8525

else   # ifeq   ($(_t),_pnx8525_)

ifeq            ($(_t),_pnx8535_)

_SDE_TCS_CORE =         tm2270
_SDE_TCS_TMTARGET =     pnx8535

else   # ifeq   ($(_t),_pnx8535_)

ifeq            ($(_t),_pnx8550_)

_SDE_TCS_CORE =         tm3260
_SDE_TCS_TMTARGET =     pnx8550

else   # ifeq   ($(_t),_pnx8550_)

ifeq            ($(_t),_tm1_)

_SDE_TCS_CORE =         tm32
_SDE_TCS_TMTARGET =     tm1

else   # ifeq   ($(_t),_tm1_)

ifeq            ($(_t),_tm32Minimal_)

_SDE_TCS_CORE =         tm32
_SDE_TCS_TMTARGET =     tm32Minimal

else   # ifeq   ($(_t),_tm32Minimal_)

ifeq            ($(_t),_tm2270Minimal_)

_SDE_TCS_CORE =         tm2270
_SDE_TCS_TMTARGET =     tm2270Minimal

else   # ifeq   ($(_t),_tm2270Minimal_)

ifeq            ($(_t),_tm3260Minimal_)

_SDE_TCS_CORE =         tm3260
_SDE_TCS_TMTARGET =     tm3260Minimal

else   # ifeq   ($(_t),_tm3260Minimal_)

ifeq            ($(_t),_tm3260c2Minimal_)

_SDE_TCS_CORE =         tm3260
_SDE_TCS_TMTARGET =     tm3260c2Minimal

else   # ifeq   ($(_t),_tm3260c2Minimal_)

ifeq            ($(_t),_tm3270Minimal_)

_SDE_TCS_CORE =         tm3270
_SDE_TCS_TMTARGET =     tm3270Minimal

else   # ifeq   ($(_t),_tm3270Minimal_)

ifeq            ($(_t),_tm3270r128Minimal_)

_SDE_TCS_CORE =         tm3270r128
_SDE_TCS_TMTARGET =     tm3270r128Minimal

else   # ifeq   ($(_t),_tm3270r128Minimal_)

ifeq            ($(_t),_tm5250Minimal_)

_SDE_TCS_CORE =         tm5250
_SDE_TCS_TMTARGET =     tm5250Minimal

else   # ifeq   ($(_t),_tm5250Minimal_)

# Did not find a valid _TMDIVERSITY so look at _TMTGTCPUTYPE

ifeq            ($(_TMTGTCPUTYPE),tm3260)

_SDE_TCS_CORE =         tm3260
_SDE_TCS_TMTARGET =     pnx8550

else   # ifeq   ($(_TMTGTCPUTYPE),tm3260)

ifeq            ($(_TMTGTCPUTYPE),tm2270)

_SDE_TCS_CORE =         tm2270
_SDE_TCS_TMTARGET =     pnx8535

else   # ifeq   ($(_TMTGTCPUTYPE),tm2270)

ifeq            ($(_TMTGTCPUTYPE),tm32)

_SDE_TCS_CORE =         tm32
_SDE_TCS_TMTARGET =     pnx8525

else   # ifeq   ($(_TMTGTCPUTYPE),tm32)

ifeq            ($(_TMTGTCPUTYPE),tm5250)

_SDE_TCS_CORE =         tm5250
_SDE_TCS_TMTARGET =     pnx1700

else   # ifeq   ($(_TMTGTCPUTYPE),tm5250)

ifeq            ($(_TMTGTCPUTYPE),tm3271)

_SDE_TCS_CORE =         tm3271
_SDE_TCS_TMTARGET =     tm3271Minimal

else   # ifeq   ($(_TMTGTCPUTYPE),tm3271)

ifeq            ($(_TMTGTCPUTYPE),tm3270)

_SDE_TCS_CORE =         tm3270
_SDE_TCS_TMTARGET =     tm3270Minimal

else   # ifeq   ($(_TMTGTCPUTYPE),tm3270)

# Did not find a valid _TMTGTCPUTYPE so apply default.

_SDE_TCS_CORE =         tm3260
_SDE_TCS_TMTARGET =     pnx8550

endif  # ifeq   ($(_TMTGTCPUTYPE),tm3270)

endif  # ifeq   ($(_TMTGTCPUTYPE),tm3271)

endif  # ifeq   ($(_TMTGTCPUTYPE),tm5250)

endif  # ifeq   ($(_TMTGTCPUTYPE),tm32)

endif  # ifeq   ($(_TMTGTCPUTYPE),tm2270)

endif  # ifeq   ($(_TMTGTCPUTYPE),tm3260)

endif  # ifeq   ($(_t),_tm5250Minimal_)

endif  # ifeq   ($(_t),_tm3270r128Minimal_)

endif  # ifeq   ($(_t),_tm3270Minimal_)

endif  # ifeq   ($(_t),_tm3260c2Minimal_)

endif  # ifeq   ($(_t),_tm3260Minimal_)

endif  # ifeq   ($(_t),_tm2270Minimal_)

endif  # ifeq   ($(_t),_tm32Minimal_)

endif  # ifeq   ($(_t),_tm1_)

endif  # ifeq   ($(_t),_pnx8550_)

endif  # ifeq   ($(_t),_pnx8535_)

endif  # ifeq   ($(_t),_pnx8525_)

endif  # ifeq   ($(_t),_pnx8520_)

endif  # ifeq   ($(_t),_pnx1700_)

endif  # ifeq   ($(_t),_pnx1500_)

endif  # ifeq   ($(_t),_pnx1300_)

_t :=

endif  # ifneq ($(_SDE_TCSVERSION),)

#-----------------------------------------------------------------------
# Whether building for TriMedia, MIPS, x86, etc. some components need
# headers from the TCS device library. Set its location here. 
#-----------------------------------------------------------------------
_SDE_TCS_LIBDEV        = $(TCS)/libdev
_SDE_LIBDEV_INCLUDES   = -I$(_SDE_TCS_LIBDEV)/include
_SDE_LIBDEV_INCLUDES  += -I$(_SDE_TCS_LIBDEV)/include/$(_SDE_TCS_TMTARGET)

ifneq ($(MAKECMDGOALS),rgd)
-include $(_SDE_DIR_LIB)/tmFlags.mk
-include $(_SDE_DIR_LIB)/phFlags.mk
endif

_SDE_TARGETS_NO_TMFLAGS := clean clean_all help version

# Include tmFlagsMak.mk here, see ticket 69797
include $(DIR_SDE)/tmflags/tmFlagsMak.mk

_SDE_FLAVOR_OPTIONS += -DTMFL_MODE_KERNEL=$(TMFL_MODE_KERNEL)

#-----------------------------------------------------------------------
# Create file list for the required projects 
#-----------------------------------------------------------------------
_SDE_PRJLIST_FILENAME = $(_SDE_PROJECT)/prjlist.txt

_SDE_REQPRJ := $(subst \,/,$(if $(wildcard $(_SDE_PRJLIST_FILENAME)), $(shell $(ECHO) $(shell $(CAT) $(_SDE_PRJLIST_FILENAME)|$(SED) -e 's/\#.*/ /g;')), $(_TMROOT)))

_SDE_COMP_LIST_FILE   := $(foreach COMPS, $(_SDE_REQPRJ), $(wildcard $(COMPS)/comps/*))
_SDE_INTFS_LIST_FILE  := $(foreach INTFS, $(_SDE_REQPRJ), $(wildcard $(INTFS)/intfs/*))
_SDE_TYPES_LIST_FILE  := $(foreach TYPES, $(_SDE_REQPRJ), $(wildcard $(TYPES)/types/*))
_SDE_ALL_COMPS_DIR    := $(strip $(_SDE_INTFS_LIST_FILE) $(_SDE_COMP_LIST_FILE) $(_SDE_TYPES_LIST_FILE))
_SDE_COMPS_FILE       := $(foreach DIR, $(_SDE_ALL_COMPS_DIR), $(notdir $(DIR)) $(DIR))

# The following definitions are needed to update loc_list files if one add a component.
_SDE_COMP_LIST_DIR    := $(foreach COMPS, $(_SDE_REQPRJ), $(wildcard $(COMPS)/comps))
_SDE_INTFS_LIST_DIR   := $(foreach COMPS, $(_SDE_REQPRJ), $(wildcard $(COMPS)/intfs))
_SDE_TYPES_LIST_DIR   := $(foreach COMPS, $(_SDE_REQPRJ), $(wildcard $(COMPS)/types))
_SDE_REQPRJ_DIRS      := $(strip $(_SDE_INTFS_LIST_DIR) $(_SDE_COMP_LIST_DIR) $(_SDE_TYPES_LIST_DIR))
 

LOC_LIST_FILENAME = $(_SDE_TMTGTBUILDROOT)/project/loc_list.txt
LOC_LIST_MKFILENAME = $(_SDE_TMTGTBUILDROOT)/project/loc_list.mk

ifeq ($(filter clean clean_all,$(MAKECMDGOALS)),)

$(LOC_LIST_FILENAME)::                                          \
                $(_SDE_PRJLIST_FILENAME)                        \
                $(_SDE_DEP_MAKEFILES)                           \
                $(_SDE_REQPRJ_DIRS)
	@$(_SDE_ECHO)$(RM) $@;
	@$(MKDIR) $(_SDE_TMTGTBUILDROOT)/project
	@for comp in $(_SDE_ALL_COMPS_DIR) ; \
	do \
	if [ -d $$comp ] ; then \
	$(ECHO) "`basename $$comp` $$comp">> $@; \
	fi ;\
	done

$(LOC_LIST_MKFILENAME)::                                        \
                $(_SDE_PRJLIST_FILENAME)                        \
                $(LOC_LIST_FILENAME)
	@$(MKDIR) $(_SDE_TMTGTBUILDROOT)/project
	@$(ECHO) \# $@ >$@
	@$(ECHO) \# DO NOT EDIT. This file has been generated automatically. >>$@
	@for comp in $(_SDE_ALL_COMPS_DIR) ; \
	do \
	if [ -d $$comp ] ; then \
	$(ECHO) "_`basename $$comp`_DIR := $$comp">> $@; \
	fi ;\
	done

$(_SDE_PRJLIST_FILENAME):
	$(_SDE_ECHO)$(ECHO) $(_TMROOT) > $@

makelist:       $(_SDE_PRJLIST_FILENAME) \
                $(LOC_LIST_MKFILENAME)

-include $(LOC_LIST_MKFILENAME)

endif # ifeq ($(filter clean clean_all,$(MAKECMDGOALS)),)

-include $(_TMROOT)/project/project_environment.mk
