#***********************************************************************
# (C) 2009: NXP Semiconductors
#
# Common part of the generic makefiles for the gnu based configs.
# support: SDE2 Team
#
#***********************************************************************
_SDE_DEP_MAKEFILES +=   $(DIR_SDE)/$(_TMTOOLCHAIN)/common.mk

#-----------------------------------------------------------------------
# set the library path
#-----------------------------------------------------------------------
LD_LIBRARY_PATH:=$(_SDE_DIR_LIB)/$(_SDE_DIVERSITY):$(LD_LIBRARY_PATH)
export LD_LIBRARY_PATH

#------------------------------------------------------------------------------
# extensions
#------------------------------------------------------------------------------
_SDE_LIB_EXTN :=a
_SDE_DLL_EXTN :=so
_SDE_O =o
_SDE_DIRLIB_SUFFIX = -L

#-----------------------------------------------------------------------
# define the debug link options
#-----------------------------------------------------------------------
# Define: _SDE_OPTIMISATION
# The CC flag for the selected optimisation level, added to <_SDE_DEBUG_OPTIONS>.
ifeq ($(CONFIG_TOOLCHAIN_OPTIMIZE_0),y)
_SDE_OPTIMISATION=-O0
endif
ifeq ($(CONFIG_TOOLCHAIN_OPTIMIZE_1),y)
_SDE_OPTIMISATION=-O1
endif
ifeq ($(CONFIG_TOOLCHAIN_OPTIMIZE_2),y)
_SDE_OPTIMISATION=-O2
endif
ifeq ($(CONFIG_TOOLCHAIN_OPTIMIZE_3),y)
_SDE_OPTIMISATION=-O3
endif
ifeq ($(CONFIG_TOOLCHAIN_OPTIMIZE_S),y)
_SDE_OPTIMISATION=-Os
endif

# Define: _SDE_DEBUG
# The CC flag for the selected debug level, added to <_SDE_DEBUG_OPTIONS>.
ifeq ($(CONFIG_TOOLCHAIN_DEBUG_1),y)
_SDE_DEBUG=-g1
endif
ifeq ($(CONFIG_TOOLCHAIN_DEBUG_2),y)
_SDE_DEBUG=-g2
endif
ifeq ($(CONFIG_TOOLCHAIN_DEBUG_3),y)
_SDE_DEBUG=-g3
endif

ifeq "$(_TMTGTREL)" "retail"		#retail mode
_SDE_DEBUG_OPTIONS = -DNDEBUG $(_SDE_OPTIMISATION)
else
ifeq "$(_TMTGTREL)" "assert"		#assert mode
_SDE_DEBUG_OPTIONS = $(_SDE_OPTIMISATION)
else
ifeq "$(_TMTGTREL)" "trace"		#trace mode
_SDE_DEBUG_OPTIONS = $(_SDE_OPTIMISATION) -DTMFL_TRACE
else					#debug mode
_SDE_LD_DEBUG_OPTIONS = $(_SDE_DEBUG)
_SDE_DEBUG_OPTIONS = $(_SDE_DEBUG) -DDEBUG
endif
endif
endif
#-----------------------------------------------------------------------
# Name of the development tools to use
#-----------------------------------------------------------------------

ifneq "$(CROSS_COMPILE)" ""
  _SDE_GCC_PREFIX := $(patsubst %-,%,$(CROSS_COMPILE))
else
  _SDE_GCC_PREFIX := $(GCC_PREFIX)
  CROSS_COMPILE := $(if $(_SDE_GCC_PREFIX),$(_SDE_GCC_PREFIX)-)
endif

CC            = $(CROSS_COMPILE)gcc
GCC           = $(CC)
CXX           = $(CROSS_COMPILE)g++
LB            = $(CROSS_COMPILE)ar
AS            = $(CROSS_COMPILE)gcc
LD            = $(CC)
LD_DLL        = $(CROSS_COMPILE)ld
GCOV          = $(CROSS_COMPILE)gcov

#-----------------------------------------------------------------------
# Set GCC variables if not set
#-----------------------------------------------------------------------


ifdef GCC_VERSION
  _SDE_GCC_VERSION := $(GCC_VERSION)
else
  _SDE_GCC_VERSION ?= $(strip $(shell $(CC) -dumpversion))
endif

ifdef GCC_BASE
  _SDE_GCC_BASE := $(GCC_BASE)
else
  ifneq ($(_TMTGTCPUCLASS),x86)
    $(warning "Warning: GCC_BASE not defined")
  endif
endif

ifdef GCC_INCLUDE
  _SDE_GCC_INCLUDE = $(GCC_INCLUDE)
else
  _SDE_GCC_INCLUDE = $(_SDE_GCC_BASE)/lib/gcc-lib/$(_SDE_GCC_PREFIX)/$(_SDE_GCC_VERSION)/include/linux
endif

#-----------------------------------------------------------------------
# Environment variables for output redirection
#-----------------------------------------------------------------------
ifeq ($(_SDE_BUILD_TARGET),LIB)
DIR_INTERM	= $(DIR_INTERM_LIB)
else
DIR_INTERM	= $(DIR_INTERM_EXE)
endif

#-----------------------------------------------------------------------
# Warning levels (0,1,2,3) determined by _TMTGTWARNINGS
#-----------------------------------------------------------------------
ifeq ($(_TMTGTWARNINGS),0)                  # mode 0, no warnings
_SDE_WARNING_LEVEL +=  -w
else                                        # mode 1, only severe warnings
_SDE_WARNING_LEVEL += \
                      -Wparentheses
ifneq ($(REL_SUFFIX),_g)                     #-Wuninitialized is supported only with -O
_SDE_WARNING_LEVEL += -Wuninitialized
endif
ifeq ($(_TMTGTWARNINGS),2)                  # mode 2, other important warnings
_SDE_WARNING_LEVEL += \
                      -Wimplicit             \
                      -Wreturn-type          \
                      -Wunused
else
ifneq ($(_TMTGTWARNINGS),1)                 # mode 3, default, all warnings
_SDE_WARNING_LEVEL +=  -Wall               \
                      -Wextra              \
                      -Wshadow             \
                      -Wpointer-arith      \
                      -Winline             \
                      -Wundef              \
                      -Wno-unknown-pragmas
ifneq ($(C_SOURCES),)
_SDE_WARNING_LEVEL += -Wstrict-prototypes
endif
endif
endif
endif

#-----------------------------------------------------------------------
# Assembler platform specific options and rules
#-----------------------------------------------------------------------
ifneq ($(_TMNODEPENDENCIES),1)
_SDE_DEPENDENCIES+=                            \
        $(filter %.d,                          \
          $(S_SOURCES:%.S=$(DIR_INTERM)/%.d)   \
          $(S_SOURCES:%.s=$(DIR_INTERM)/%.d)   \
        )
endif

#-----------------------------------------------------------------------
# Environment variables for output redirection
#-----------------------------------------------------------------------
ifeq ($(_SDE_BUILD_TARGET),LIB)
DIR_INTERM  = $(DIR_INTERM_LIB)
else
DIR_INTERM  = $(DIR_INTERM_EXE)
endif

#-----------------------------------------------------------------------
# compute object files from sources
#-----------------------------------------------------------------------
_SDE_OBJECTS += $(CXX_SOURCES:%.cpp=$(DIR_INTERM)/%.$(_SDE_O)) 		\
		$(C_SOURCES:%.c=$(DIR_INTERM)/%.$(_SDE_O)) 		\
                $(filter                                                \
                    %.$(_SDE_O),                                        \
                    $(S_SOURCES:%.s=$(DIR_INTERM)/%.$(_SDE_O))          \
                    $(S_SOURCES:%.S=$(DIR_INTERM)/%.$(_SDE_O))          \
                    $(S_SOURCES:%.asm=$(DIR_INTERM)/%.$(_SDE_O))        \
                )                                                       \
		$(CFG_SOURCES:%.c=$(DIR_INTERM)/%.$(_SDE_O))

#-------------------------------------------------------------------------------
# Compile commands
#-----------------------------------------------------------------------
# Define: _SDE_SYSROOT_OPTION
# The sysroot to use if the compiler was built with --sysroot.
# Is the compiler built with sysroot support and whats its value?
ifneq "$(CROSS_COMPILE)" ""
  _SDE_SYSROOT_ENABLED:=$(shell $(CC) -v 2>&1 | grep -- --with-sysroot)
  _SDE_SYSROOT_OPTION=$(if $(_SDE_SYSROOT_ENABLED),--sysroot=$(STAGINGDIR))
endif

# When compile in linux environment we inherit linux optimization flag.
# This flag needs to be removed because it is overloaded by _SDE_DEBUG_OPTIONS.
__TARGET_CFLAGS = $(subst -O2,,$(TARGET_CFLAGS)) $(_SDE_SYSROOT_OPTION)

_SDE_C_COMPILE_CMD=$(strip $(CC) $(_SDE_COPTS) $(__TARGET_CFLAGS) -c)
_SDE_CXX_COMPILE_CMD=$(strip $(CXX) $(_SDE_CXXOPTS) $(__TARGET_CFLAGS) -c)
_SDE_A_COMPILE_CMD=$(strip $(AS) $(_SDE_AOPTS) $(__TARGET_AFLAGS) -c)

#------------------------------------------------------------------------
# Rules for building object files out of c files, cxx files, and
# assembler files
#------------------------------------------------------------------------
$(DIR_INTERM)/%.$(_SDE_O): %.c $(PRJFIXDEP)
	$(_SDE_ECHO)$(MKDIR) $(dir $@); \
	$(ECHO) compiling $(if $(wildcard $(CURDIR)/$<),$(CURDIR)/$<,$<)
	$(_SDE_ECHO)$(call _SDE_STORE_CMD,_SDE_C_COMPILE_CMD,$@)
	$(_SDE_ECHO)$(_SDE_C_COMPILE_CMD) -o $@ $(if $(wildcard $(CURDIR)/$<), $(CURDIR)/$<, $<)
	$(_SDE_POST_COMPILE_FIXDEP)


$(DIR_INTERM)/%.$(_SDE_O): %.cpp $(PRJFIXDEP)
	$(_SDE_ECHO) $(MKDIR) $(dir $@); \
	$(ECHO) compiling $(if $(wildcard $(CURDIR)/$<), $(CURDIR)/$<, $<)
	$(_SDE_ECHO)$(call _SDE_STORE_CMD,_SDE_CXX_COMPILE_CMD,$@)
	$(_SDE_ECHO)$(_SDE_CXX_COMPILE_CMD) -o $@ $(if $(wildcard $(CURDIR)/$<),$(CURDIR)/$<,$<)
	$(_SDE_POST_COMPILE_FIXDEP)

$(DIR_INTERM)/%.$(_SDE_O): %.s $(PRJFIXDEP)
	$(_SDE_ECHO)$(MKDIR) $(dir $@); \
	$(ECHO) compiling $(if $(wildcard $(CURDIR)/$<), $(CURDIR)/$<, $<)
	$(_SDE_ECHO)$(call _SDE_STORE_CMD,_SDE_A_COMPILE_CMD,$@)
	$(_SDE_ECHO)$(_SDE_A_COMPILE_CMD) -o $@ $(if $(wildcard $(CURDIR)/$<), $(CURDIR)/$<, $<)
	$(_SDE_POST_COMPILE_FIXDEP)

$(DIR_INTERM)/%.$(_SDE_O): %.S $(PRJFIXDEP)
	$(_SDE_ECHO)$(MKDIR) $(dir $@); \
	$(ECHO) compiling $(if $(wildcard $(CURDIR)/$<), $(CURDIR)/$<, $<)
	$(_SDE_ECHO)$(call _SDE_STORE_CMD,_SDE_A_COMPILE_CMD,$@)
	$(_SDE_ECHO)$(_SDE_A_COMPILE_CMD) -o $@ $(if $(wildcard $(CURDIR)/$<), $(CURDIR)/$<, $<)
	$(_SDE_POST_COMPILE_FIXDEP)

$(DIR_INTERM)/%.$(_SDE_O): %.asm $(PRJFIXDEP)
	$(_SDE_ECHO)$(MKDIR) $(dir $@); \
	$(ECHO) compiling $(if $(wildcard $(CURDIR)/$<), $(CURDIR)/$<, $<)
	$(_SDE_ECHO)$(call _SDE_STORE_CMD,_SDE_A_COMPILE_CMD,$@)
	$(_SDE_ECHO)$(_SDE_A_COMPILE_CMD) -o $@ $(if $(wildcard $(CURDIR)/$<), $(CURDIR)/$<, $<)
	$(_SDE_POST_COMPILE_FIXDEP)

#-----------------------------------------------------------------------
# include common stuff after DIR_INTERM and _SDE_OBJECTS is set
#-----------------------------------------------------------------------
include $(DIR_SDE)/common.mk

#-----------------------------------------------------------------------
# for gnu linking
#-----------------------------------------------------------------------
ifeq ($(_TMTGTOSCLASS),linux)
_SDE_EXTRA_LDFLAGS += -lrt -lm -lgcc
endif
#-----------------------------------------------------------------------
# compute the user libraries to be used
#-----------------------------------------------------------------------
_SDE_LIBRARIES     := $(_SDE_LIBS_DIVERSITY:%=-l%)
_SDE_EXT_LIBS      := $(EXTERNAL_LIBS:%=%.$(_SDE_LIB_EXTN))
_SDE_DEP_LIBRARIES := $(_SDE_LIBS_DIVERSITY:%=lib%.$(_SDE_LIB_EXTN)) $(_SDE_EXT_LIBS)
_SDE_DLLS          := $(strip $(_SDE_IMPORT_DLLS:%=-l%))
_SDE_EXT_DLLS      := $(EXTERNAL_DLLS:%=-l%)

ifeq "$(_TMLINKTYPE)" "dynamic"

  # Add rpath and rpath-link options if we're linking dynamically.

  # Define: _SDE_TGTPREFIXES
  # List of directories from which shared libraries are loaded.
  # These paths are all subdirectories of <_TMTGTROOTFS>.
  # E.g. /usr/local, _TMTGTPREFIX.
  #
  # Defaults to <LOCAL_PREFIXES>, <_TMTGTPREFIX>, and /usr.
  #
  # See: <LOCAL_PREFIXES>
  _SDE_TGTPREFIXES = \
    $(LOCAL_PREFIXES) \
    $(_TMTGTPREFIX) \
    /usr

  # Variable: <LOCAL_PREFIXES>
  # A list of directories on <_TMTGTROOTFS> that a user's SDE2 makefile can define.

  # -rpath and -rpath-link options need adding to a shared library link command using ld,
  # and to an executable link command using gcc.
  # This function can cope with both.
  _SDE_DLL_PREFIX_OPTS = \
    $(if $(CROSS_COMPILE),-L$(STAGINGDIR)/lib -L$(STAGINGDIR)/usr/lib $(if $(_SDE_SYSROOT_ENABLED),$(_SDE_SYSROOT_OPTION),$(1)-rpath-link $(1)$(STAGINGDIR)/lib:$(STAGINGDIR)/usr/lib)) \
    $(foreach d, $(_SDE_TGTPREFIXES), -L$(_TMTGTROOTFS)$(d)/lib $(1)-rpath-link $(1)$(_TMTGTROOTFS)$(d)/lib)

  _SDE_GCC_LINKER_OPTION := -Wl,
  # Executable link command, uses gcc:
  _SDE_EXTRA_LDFLAGS += $(call _SDE_DLL_PREFIX_OPTS,$(_SDE_GCC_LINKER_OPTION))
  # Shared library link command, uses ld directly:
  _SDE_DLL_OPTIONS   += $(call _SDE_DLL_PREFIX_OPTS,)
endif

#-----------------------------------------------------------------------
# prepend -I to all include file names; The first 11 include statements are done in the sde/common.mk
#-----------------------------------------------------------------------
# Include path differs for user mode / kernel modules:

ifeq "$(TMFL_MODE_USER)" "1"
_SDE_INCLUDES += -I$(_SDE_GCC_INCLUDE)
else
_SDE_INCLUDES += -I$(KSRC)/include
endif
_SDE_INCLUDES += $(_TMTGTINCLUDES:%=-I%)

ifdef TCS
_SDE_INCLUDES += -I$(TCS)/include/cores/$(_SDE_TCS_CORE)/ops
_SDE_INCLUDES += -I$(TCS)/include/cores/$(_SDE_TCS_CORE)
_SDE_INCLUDES += -I$(TCS)/include/tmlib
_SDE_INCLUDES += -I$(TCS)/tmComm/include
endif

_SDE_INCLUDES_C  = $(_SDE_INCLUDES)

_SDE_INCLUDES_CXX = $(_SDE_INCLUDES)

_SDE_INCLUDES_A  = $(_SDE_INCLUDES)

#-----------------------------------------------------------------------
# GNU Based configurations
#-----------------------------------------------------------------------
# For kernel mode _SDE_KERNEL_OR_USER_MODE_C_OPTIONS and variants are determined in kmodule.mk
ifeq "$(TMFL_MODE_USER)" "1"
  _SDE_KERNEL_OR_USER_MODE_C_OPTIONS=$(_SDE_PROC_DEFINES) -fPIC
  _SDE_KERNEL_OR_USER_MODE_CXX_OPTIONS=$(_SDE_PROC_DEFINES) -fPIC
  _SDE_KERNEL_OR_USER_MODE_A_OPTIONS=$(_SDE_PROC_DEFINES) -fPIC
endif

_SDE_COPTS_FILE1   = $(_SDE_FLAVOR_OPTIONS) $(_SDE_DEBUG_OPTIONS) $(_SDE_WARNING_LEVEL) $(_SDE_CPU_FLAVOR)
_SDE_COPTS_FILE2   = $(_SDE_KERNEL_OR_USER_MODE_C_OPTIONS) -MD -MF $(@:.$(_SDE_O)=.d) -MT $@
_SDE_COPTS_FILE3   = -I. $(_SDE_INCLUDES_C) $(_SDE_ENDIAN_OPTION)
_SDE_COPTS_FILE4   = -c $(_SDE_EXTRA_CFLAGS) $(LOCAL_CFLAGS)

#-----------------------------------------------------------------------
# Options for the C++ compiler
#-----------------------------------------------------------------------
_SDE_CXXOPTS_FILE1 = $(_SDE_FLAVOR_OPTIONS) $(_SDE_DEBUG_OPTIONS) $(_SDE_WARNING_LEVEL) $(_SDE_CPU_FLAVOR)
_SDE_CXXOPTS_FILE2 = $(_SDE_KERNEL_OR_USER_MODE_CXX_OPTIONS) -MD -MF $(@:.$(_SDE_O)=.d) -MT $@
_SDE_CXXOPTS_FILE3 = -I. $(_SDE_INCLUDES_CXX)
_SDE_CXXOPTS_FILE4 = -c $(_SDE_EXTRA_CXXFLAGS) $(LOCAL_CXXFLAGS)

#-----------------------------------------------------------------------
# Options for the assembler
#-----------------------------------------------------------------------
_SDE_AOPTS_FILE1 = $(_SDE_FLAVOR_OPTIONS) $(_SDE_DEBUG_OPTIONS) $(_SDE_WARNING_LEVEL) $(_SDE_CPU_FLAVOR)
_SDE_AOPTS_FILE2 = $(_SDE_KERNEL_OR_USER_MODE_A_OPTIONS) -MD -MF $(@:.$(_SDE_O)=.d) -MT $@
_SDE_AOPTS_FILE3 = -I. $(_SDE_INCLUDES_A)
_SDE_AOPTS_FILE4 = -c $(_SDE_EXTRA_AFLAGS) $(LOCAL_AFLAGS)

#-----------------------------------------------------------------------
_SDE_COPTS         = $(_SDE_ALL_COPTS)
_SDE_CXXOPTS       = $(_SDE_ALL_CXXOPTS)
_SDE_AOPTS         = $(_SDE_ALL_AOPTS)
_SDE_LBOPTS        = $(_SDE_EXTRA_LBFLAGS) -r
_SDE_LDOPTS        = $(_SDE_EXTRA_LDFLAGS) $(_SDE_LD_DEBUG_OPTIONS) $(LOCAL_LDFLAGS)

local_configuration:
ifneq ($(_SDE_WARNING),)
	$(_SDE_ECHO)$(ECHO) $(ECHOFLAGS) $(_SDE_WARNING)
endif

#-----------------------------------------------------------------------
# Support for extensions
#-----------------------------------------------------------------------
-include $(DIR_SDE)/$(_TMTGTOSCLASS)/common_ext.mk

_SDE_INCLUDE_SDE_COMMON_MK_=1
include $(DIR_SDE)/common.mk

