#________________________________________________________________________
# ......................................................................
#              COPYRIGHT (c) NXP Semiconductors  2006               .
#.     All rights are reserved. Reproduction in whole or in part is     .
#.     prohibited without the written consent of the copyright owner.   .
# ......................................................................
# Author: Milan van den Muyzenberg
# Support and adaptaion: Hans den Boer, Peter Kazakov, Shivaraj P
# Includes 32-bit GUID support, 128-bit GUID support (only nt platform is tested)
#____________________________________________________________________SCP__

_SDE_DEP_MAKEFILES += $(DIR_SDE)/idl.mk

# IDL_COMPILATION flag should be set to 1/0 in component and/or application makefile
# for selecting idl compilation

ifeq ($(IDL_COMPILATION),1)
#==============================================================================
# IDL files for which no headers should be generated
# BUT, they generate *Guids.c files for 128 bit guids
#==============================================================================
ifndef _SDE_FILTEROUT_IDL
_SDE_FILTEROUT_IDL := tmIUnknown.idl tmNxTypes.ddl tmCom_Guid.ddl \
                      tmMidlCompat.ddl tmApiTypes.ddl
endif

_SDE_FILTEROUT_PREFIX_IDL := $(patsubst %,\%%,$(_SDE_FILTEROUT_IDL))


#==============================================================================
# IDL filename and path related definitions
#==============================================================================
# e.g. tmIUnknownGuids.c
_SDE_GUIDEXT          := _Guids
_SDE_INTFS_IDL_DIR    := $(_SDE_TMTGTBUILDROOT)/intfs/generated
_SDE_COMPS_IDL_DIR    := $(_SDE_TMTGTBUILDROOT)/comps/generated
_SDE_INTFS_IDL_DIRINC := $(_SDE_INTFS_IDL_DIR)/inc
_SDE_INTFS_IDL_DIRSRC := $(_SDE_INTFS_IDL_DIR)/src
_SDE_COMPS_IDL_DIRSRC := $(_SDE_COMPS_IDL_DIR)/src


#==============================================================================
# Processing of REQUIRES in case of IDL support
#==============================================================================
ifeq ($(findstring tmIUnknown,$(REQUIRES)),tmIUnknown)
_SDE_IMPORT_REQUIRES :=                                                  \
     $(shell                                                             \
        $(ECHO) $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .r $(REQUIRES) \
         | $(GAWK) -f $(DIR_SDE)/awk/recursestatic_requires.awk          \
      )
_SDE_MISSING_RFILES := $(filter ERR_R_%, $(_SDE_IMPORT_REQUIRES))
_SDE_IMPORT_REQUIRES := $(filter-out $(_SDE_MISSING_RFILES), $(_SDE_IMPORT_REQUIRES))
else
_SDE_IMPORT_REQUIRES = $(REQUIRES)
endif

#==============================================================================
# All idl (.idl, .ddl, .cdl) source files
#==============================================================================
# .idl and .ddl files in idl directories of required comps and interfaces
# and in current component idl directory
_SDE_INTFS_IDL_SOURCES := $(sort $(filter-out $(_SDE_FILTEROUT_PREFIX_IDL), \
                             $(foreach entry, $(_SDE_IMPORT_REQUIRES),      \
                               $(wildcard $(_I$(entry)_DIR)/idl/*.idl)      \
                               $(wildcard $(_I$(entry)_DIR)/idl/*.ddl)      \
                               $(wildcard $(_$(entry)_DIR)/idl/*.idl)       \
                               $(wildcard idl/*.idl)                        \
                             )                                              \
                          ))

# cdl files in current component idl directory
_SDE_COMPS_IDL_SOURCES := $(sort $(wildcard idl/*.cdl))

# cdl files in idl directories of components in LIBS
_SDE_COMPS_IDL_SOURCES_IN_LIBS :=                                           \
                          $(sort                                            \
                             $(foreach entry, $(_SDE_LIBS_NOSFX),                      \
                               $(wildcard $(_$(entry)_DIR)/idl/*.cdl)       \
                              )                                             \
                           )

#==============================================================================
# All files generated from the idl (.idl, .ddl, .cdl) source files
#==============================================================================
_SDE_INTFS_IDL_HEADERS := $(sort \
                            $(foreach entry_, $(_SDE_INTFS_IDL_SOURCES), \
                              $(_SDE_INTFS_IDL_DIRINC)/$(basename $(notdir $(entry_))).h \
                             ) \
                           )

_SDE_INTFS_IDL_GUIDS   := $(sort \
                            $(foreach entry_, $(filter-out %.ddl, $(_SDE_INTFS_IDL_SOURCES)), \
                              $(_SDE_INTFS_IDL_DIRSRC)/$(basename $(notdir $(entry_)))$(_SDE_GUIDEXT).c \
                             ) \
                            $(_SDE_INTFS_IDL_DIRSRC)/tmIUnknown$(_SDE_GUIDEXT).c \
                           )

_SDE_COMPS_IDL_GUIDS   := $(sort \
                            $(foreach entry_, $(_SDE_COMPS_IDL_SOURCES), \
                              $(_SDE_COMPS_IDL_DIRSRC)/$(basename $(notdir $(entry_)))$(_SDE_GUIDEXT).c \
                             ) \
                           )

_SDE_COMPS_IDL_GUIDS_IN_LIBS := $(sort \
                            $(foreach entry_, $(_SDE_COMPS_IDL_SOURCES_IN_LIBS), \
                              $(_SDE_COMPS_IDL_DIRSRC)/$(basename $(notdir $(entry_)))$(_SDE_GUIDEXT).c \
                             ) \
                           )

#==============================================================================
# Vpaths for all idl types
#==============================================================================
_SDE_IDLVPATH               += $(foreach entry, $(_SDE_IMPORT_REQUIRES),    \
                               $(wildcard $(_I$(entry)_DIR)/idl)            \
                               $(wildcard $(_$(entry)_DIR)/idl))            \
                               idl
                          
vpath %.idl $(_SDE_IDLVPATH)
vpath %.ddl $(_SDE_IDLVPATH)
vpath %.cdl $(_SDE_IDLVPATH)
vpath %.c $(_SDE_INTFS_IDL_DIRSRC) $(_SDE_COMPS_IDL_DIRSRC)

_SDE_IDLFLAGS     += -nologo -noverbose -root$(_TMROOT) $(patsubst %,-I%,$(_SDE_IDLVPATH))


#==============================================================================
# Library extension for the diverse platforms
#==============================================================================
ifneq (,$(filter nt ce,$(_TMTGTOSCLASS)))
    ifndef _SDE_IDL_OUT_LIB
        _SDE_IDL_OUT_LIB :=lib
    endif
else
    ifndef _SDE_IDL_OUT_LIB
        _SDE_IDL_OUT_LIB :=a
    endif
endif
	

#==============================================================================
# Rules to make header and guid source files from IDL files
#==============================================================================

#  Make header file from interface description idl file
$(_SDE_INTFS_IDL_DIRINC)/%.h: %.idl
	$(_SDE_ECHO)$(ECHO) Generating $(notdir $@) from $(notdir $<);                    \
	$(MKDIR) $(_SDE_INTFS_IDL_DIRINC);                                   \
	$(IDLTOOL) $(_SDE_IDLFLAGS) -dih$(_SDE_INTFS_IDL_DIRINC) $(basename $(notdir $@)).idl
ifeq ($(_TMTGTCOPYGUIDS),1)
	$(_SDE_ECHO)$(ECHO) copying the idl header file to $(patsubst %/idl/, %/inc/, $(dir $<)); \
        $(MKDIR) $(patsubst %/idl/, %/inc/, $(dir $<)); \
        $(CP) -r $(_SDE_INTFS_IDL_DIRINC)/$(notdir $@) $(patsubst %/idl/, %/inc/, $(dir $<));
endif

# Make header file from typedef/const description ddl file (*.ddl)
$(_SDE_INTFS_IDL_DIRINC)/%.h: %.ddl
	$(_SDE_ECHO)$(ECHO) Generating $(notdir $@) from $(notdir $<);                    \
	$(MKDIR) $(_SDE_INTFS_IDL_DIRINC);                                   \
	$(IDLTOOL) $(_SDE_IDLFLAGS) -dih$(_SDE_INTFS_IDL_DIRINC) $(basename $(notdir $@)).ddl
ifeq ($(_TMTGTCOPYGUIDS),1)
	$(_SDE_ECHO)$(ECHO) copying the idl header file to $(patsubst %/idl/, %/inc/, $(dir $<)); \
        $(MKDIR) $(patsubst %/ddl/, %/inc/, $(dir $<)); \
        $(CP) -r $(_SDE_INTFS_IDL_DIRINC)/$(notdir $@) $(patsubst %/idl/, %/inc/, $(dir $<));
endif

# Make guid file from interface description file (*.idl)
$(_SDE_INTFS_IDL_DIRSRC)/%$(_SDE_GUIDEXT).c: %.idl
ifneq ($(_TMTGTGUIDS),32)
	$(_SDE_ECHO)$(ECHO) Generating $(basename $(notdir $@)).c from $(notdir $<); \
	$(MKDIR) $(_SDE_INTFS_IDL_DIRSRC);                        \
	$(IDLTOOL) $(_SDE_IDLFLAGS) -dgc$(_SDE_INTFS_IDL_DIRSRC) $<
endif


# Make guid file from coclass description file (*.cdl)
$(_SDE_COMPS_IDL_DIRSRC)/%$(_SDE_GUIDEXT).c: %.cdl
	$(_SDE_ECHO)$(ECHO) Generating $(notdir $@) from $(notdir $<);         \
	$(MKDIR) $(_SDE_COMPS_IDL_DIRSRC);                                     \
	$(IDLTOOL) $(_SDE_IDLFLAGS) -dgc$(_SDE_COMPS_IDL_DIRSRC) $(notdir $<)	


#==============================================================================
# Here the distinction is made between libs (that do not need to link in the
# guids) and executables/dlls (that do need to link in the guids). The 
# distinction is made based on the inclusion of tmIUnknown in REQUIRES.
#==============================================================================

# directory where the object files are stored
_SDE_INTFS_OBJ_GUIDS_DIR := \
    $(_SDE_TMTGTBUILDROOT)/tmp/idl/obj$(REL_SUFFIX)_$(_SDE_LIB_CONFIGURATION)Guids

_SDE_COMPS_OBJ_GUIDS_DIR := \
    $(_SDE_TMTGTBUILDROOT)/tmp/cdl/obj$(REL_SUFFIX)_$(_SDE_LIB_CONFIGURATION)Guids


# only take following steps if 128 bit guids and tmIUnknown is in the REQUIRES
ifneq ($(_TMTGTGUIDS),32)

ifeq (tmIUnknown,$(findstring tmIUnknown,$(REQUIRES)))
    _SDE_DEP_IDL_LIBRARIES := \
        $(DIR_INTERM)/tmp/guids$(REL_SUFFIX).$(_SDE_IDL_OUT_LIB)

    ifneq ($(_SDE_BUILD_TARGET),LIB) # target is an executable
        _SDE_EXE_DLL_TARGETS:= $(_SDE_DEP_IDL_LIBRARIES)        \
                               $(_SDE_INTFS_IDL_GUIDS)          \
                               $(_SDE_COMPS_IDL_GUIDS_IN_LIBS)
        EXTERNAL_LIBS += $(DIR_INTERM)/tmp/guids$(REL_SUFFIX)
    else
        ifneq ($(EXPORTS),) # target is a dynamic link library
            _SDE_EXE_DLL_TARGETS:= $(_SDE_DEP_IDL_LIBRARIES)      \
                                   $(_SDE_INTFS_IDL_GUIDS)        \
                                   $(_SDE_COMPS_IDL_GUIDS_IN_LIBS)
            EXTERNAL_LIBS += $(DIR_INTERM)/tmp/guids$(REL_SUFFIX)
        endif # EXPORTS
    endif # ! LIB
endif # tmIUnknown: this prevents unwanted generation of guids libraries.

#==============================================================================
# All interface and component guid object files derived from idl source files
#==============================================================================
_SDE_INTFS_GUID_OBJECTS := $(patsubst %.c,$(_SDE_INTFS_OBJ_GUIDS_DIR)/%.$(_SDE_O),  \
                             $(notdir $(sort $(_SDE_INTFS_IDL_GUIDS)))        \
                            )

_SDE_COMPS_GUID_OBJECTS := $(patsubst %.c,$(_SDE_COMPS_OBJ_GUIDS_DIR)/%.$(_SDE_O),  \
                             $(notdir $(sort $(_SDE_COMPS_IDL_GUIDS)))        \
                            )                                                 \
                           $(patsubst %.c,$(_SDE_COMPS_OBJ_GUIDS_DIR)/%.$(_SDE_O),  \
                             $(notdir $(sort $(_SDE_COMPS_IDL_GUIDS_IN_LIBS)))\
                            )
         
#==============================================================================
# Additional compiler and linker defines
#==============================================================================
ifneq (,$(filter nt ce,$(_TMTGTOSCLASS)))
    ifndef _SDE_IDL_OUT_LB
        _SDE_IDL_OUT_LB := /out:
    endif
    ifndef _SDE_IDL_OUT_OBJ
        _SDE_IDL_OUT_OBJ := -Fo
    endif
else
    ifndef _SDE_IDL_OUT_OBJ
        _SDE_IDL_OUT_OBJ := -o$(_empty_)
    endif
endif

endif # ifneq ($(_TMTGTGUIDS),32)

ifeq ($(_TMTGTGUIDS),32)
LOCAL_CFLAGS   +=   -DTMFL_GUID_IS_32_BITS
LOCAL_CXXFLAGS +=   -DTMFL_GUID_IS_32_BITS
endif

#==============================================================================
# Rules to make guid object files from the guid source files and a rule to
# make a library from these object files.
#==============================================================================
# Compile any .c file in the generated intfs/src directory
$(_SDE_INTFS_GUID_OBJECTS): $(_SDE_INTFS_IDL_GUIDS)
	@$(ECHO) Compiling $(notdir $@) from $(basename $(notdir $@))$(_SDE_GUID_EXT).c ...
	@if [ ! -d $(dir $@) ]; then mkdir -p $(dir $@); fi
	@if [ ! -d $(DIR_INTERM) ]; then mkdir -p $(DIR_INTERM); fi
	$(_SDE_ECHO)$(CC) $(_SDE_COPTS) $(TARGET_CFLAGS) -c $(_SDE_IDL_OUT_OBJ)$@ $(_SDE_INTFS_IDL_DIRSRC)/$(patsubst %.$(_SDE_O),%.c,$(notdir $@))

# Compile guid .c file in the generated comps/src directory
$(_SDE_COMPS_GUID_OBJECTS): $(_SDE_COMPS_IDL_GUIDS) $(_SDE_COMPS_IDL_GUIDS_IN_LIBS)
	@$(ECHO) Compiling $(notdir $@) from $(basename $(notdir $@))$(_SDE_GUID_EXT).c ...
	@if [ ! -d $(dir $@) ]; then mkdir -p $(dir $@); fi
	@if [ ! -d $(DIR_INTERM) ]; then mkdir -p $(DIR_INTERM); fi
	$(_SDE_ECHO)$(CC) $(_SDE_COPTS) $(TARGET_CFLAGS) -c $(_SDE_IDL_OUT_OBJ)$@ $(_SDE_COMPS_IDL_DIRSRC)/$(patsubst %.$(_SDE_O),%.c,$(notdir $@))

# Rule to make library from the guid objects
$(_SDE_DEP_IDL_LIBRARIES): $(_SDE_INTFS_GUID_OBJECTS) $(_SDE_COMPS_GUID_OBJECTS)
	@echo Creating $(notdir $@) ...
	@if [ ! -d $(dir $@) ]; then mkdir -p $(dir $@); fi
	$(_SDE_ECHO)$(LB) $(_SDE_LBOPTS) $(_SDE_IDL_OUT_LB)$@ $(_SDE_INTFS_GUID_OBJECTS) $(_SDE_COMPS_GUID_OBJECTS)
	@echo
ifeq ($(_TMTGTCOPYGUIDS),1)
	$(_SDE_ECHO)$(ECHO) copying the guid library file to $(_SDE_LOCAL_LIB_DIR); \
        $(MKDIR) $(_SDE_LOCAL_LIB_DIR); \
        $(CP) -r  $@ $(_SDE_LOCAL_LIB_DIR);
endif


#==============================================================================
# Generate interface headers and componente guids; for non lib targets like 
# exe's and dll's, generate interface guids and link all guids in a library
#==============================================================================
idl:    $(_SDE_INTFS_IDL_HEADERS) $(_SDE_COMPS_IDL_GUIDS) \
        $(_SDE_EXE_DLL_TARGETS)

.SECONDARY:
FORCE:
endif # ifeq ($(IDL_COMPILATION),1)
