#.      COPYRIGHT (c) 2009 NXP Semiconductors
# All Rights Reserved.
#
# Filename: kmodule_common.mk
#
# Common part of kernel driver building

# changes to this file force rebuild
_SDE_DEP_MAKEFILES +=   $(DIR_SDE)/kmodule_common.mk

#-----------------------------------------------------------------------
# Problems with parallelism with kernel modules.
ifneq "$(filter modules modules_install kmodule_include,$(MAKECMDGOALS))" ""
  .NOTPARALLEL:
endif
#-----------------------------------------------------------------------

# Define: KMODULE_BUILD_DIR
# Directory where kernel module specific build products are stored.
KMODULE_BUILD_DIR := $(DIR_INTERM_LIB)/kmodule

# Macro to get library name for a specified component, taking its diversity into account.
# Usage: $(call KMODULE_GET_LIB_NAME,phCompName)
KMODULE_GET_LIB_NAME=lib$(strip $(if $(findstring $(origin _$(1)_SUFFIX), undefined),        \
                      $(1)$(call _SDE_GET_LIB_DIV,$(1))$(REL_COMPS_SUFFIX),               \
                      $(patsubst %_r,%,$(1)$(call _SDE_GET_LIB_DIV,$(1))$(_$(1)_SUFFIX)))).$(_SDE_LIB_EXTN)

# Macro to get a library path, taking its diversity and whether it has been released
# in a component's lib directory (binary release component).
# It would be an error if neither existed.
KMODULE_GET_LIB_PATH = $(firstword $(shell ls 2>/dev/null $(_SDE_DIR_LIB)/$2 $(_$(1)_DIR)/$(_SDE_DIR_LIB_EXT)/$2) KMODULE_GET_LIB_PATH__no_match_error)

# Define: KMODULE_INCLUDED_LIB_PATHS
# Paths of libraries of all <KMODULE_INCLUDED_COMPS>.
KMODULE_INCLUDED_LIB_PATHS = $(foreach c,$(KMODULE_INCLUDED_COMPS),$(_SDE_DIR_LIB)/$(call KMODULE_GET_LIB_NAME,$c))






# Defining KMODULE_INCLUDED_COMPS (even if empty) means the component is
# the toplevel SDE2 component for a kernel module/driver.
ifeq "$(origin KMODULE_INCLUDED_COMPS)" "undefined"

#-----------------------------------------------------------------------
# WE ARE AN INCLUDED COMPONENT IN A KERNEL MODULE
#-----------------------------------------------------------------------




# Define: KMODULE_BUILD_TARGETS
# List of make targets to build for this component, when it is built as a kernel module.
# If $(<BINARY_RELEASE_AVAILABILITY>} is true, then it equals install, otherwise all.
ifeq "$(BINARY_RELEASE_AVAILABILITY)" "true"
  KMODULE_BUILD_TARGETS:=install
  kmodule_include_banner:
	@$(ECHO) 'Binary release available for component $(_SDE_CI)'
else
  KMODULE_BUILD_TARGETS:=all
  kmodule_include_banner:
	@$(ECHO) 'Building component $(_SDE_CI)'
endif

kmodule_include: $(KMODULE_BUILD_TARGETS)
$(KMODULE_BUILD_TARGETS): kmodule_include_banner




else # ifeq "$(origin KMODULE_INCLUDED_COMPS)" "undefined"



#-----------------------------------------------------------------------
# WE ARE A TOPLEVEL COMPONENT REPRESENTING A KERNEL MODULE
#-----------------------------------------------------------------------




# Define: KMODULE_NAME
# Name of kernel module, which defaults to the value of <TARGET>.
KMODULE_NAME ?= $(TARGET)

# Define: KMODULE_INCLUDED_COMPS
# List of other components to link into this kernel module (empty if none).
# Must be defined in the makefile of the toplevel component
# for a kernel module,
# to indicate that the component represents a kernel module.
ifneq "$(filter $(_SDE_CI),$(KMODULE_INCLUDED_COMPS))" ""
  $(error KMODULE_INCLUDED_COMPS should not contain toplevel component ($(_SDE_CI)))
endif
ifneq "$(filter $(KMODULE_NAME),$(KMODULE_INCLUDED_COMPS))" ""
  $(error KMODULE_INCLUDED_COMPS should not contain kernel module name ($(KMODULE_NAME)))
endif

# Define: KMODULE_COMPS
# List of components to link into this kernel module.
# This includes <KMODULE_INCLUDED_COMPS> and the current component (<TARGET>),
KMODULE_COMPS = $(TARGET) $(KMODULE_INCLUDED_COMPS)

#-----------------------------------------------------------------------
# INFORMATION
#-----------------------------------------------------------------------

# Target: print_kmodule_comps
# Print <KMODULE_COMPS> value, one item per line, to file descriptor 3.
print_kmodule_comps:
	@for c in $(KMODULE_COMPS); do echo "$$c" 1>&3; done

#-----------------------------------------------------------------------
# BUILD
#-----------------------------------------------------------------------

install: kmodule

kmodule: kmodule_banner
kmodule_banner:
	@echo '  KMODULE $(KMODULE_NAME)'

ifeq "$(BINARY_RELEASE_AVAILABILITY)" "true"
  kmodule: binary_release_available_message
endif

#-----------------------------------------------------------------------
# INCLUDED COMPONENTS
#-----------------------------------------------------------------------

kmodule_included_comps_banner:
	@$(ECHO) ''; $(ECHO) '$(_SDE_CI): Building included components: $(strip $(KMODULE_INCLUDED_COMPS))'
kmodule_included_comps: \
  kmodule_included_comps_banner \
  $(patsubst %,%.kmodule_include,$(KMODULE_INCLUDED_COMPS))
ifeq "$(_TMECHO)" "1"
kmodule_included_comps:
	@$(ECHO) '$(_SDE_CI): Finished building included components'
endif

# Function: KMODULE_BINARY_RELEASE_NOT_AVAILABLE
# Returns a shell command string to use for testing if a
# binary release for a component is available.
# Argument 1 is the directory containing the component.
KMODULE_BINARY_RELEASE_NOT_AVAILABLE=\
  $(MAKE) --silent --no-print-directory -C $(1) check_binary_release | grep -F 'check_binary_release=false'

%.kmodule_include:
	$(if $(_$*_DIR),,$(error Component $* (listed in KMODULE_INCLUDED_COMPS) not found))
ifeq "$(_TMECHO)" "1"
	@$(ECHO) '>>Recursing component $* (included in kmodule $(KMODULE_NAME))'
endif
	$(_SDE_ECHO)$(MAKE) -C $(_$*_DIR) KMODULE_NAME=$(KMODULE_NAME) kmodule_include
	@$(ECHO) ''

#-----------------------------------------------------------------------
# CLEAN
#-----------------------------------------------------------------------

# Add dependents to this, elsewhere.
kmodule_clean:;

#-----------------------------------------------------------------------
# Include the _xxx_DIVERSITY settings for all the components we reference.
#-----------------------------------------------------------------------

_t1 := \
$(foreach foo, $(KMODULE_INCLUDED_COMPS), \
  $(if $(_$(foo)_REPLACE), \
       $(wildcard $(_$(_$(foo)_REPLACE)_DIR)/diversity.mk), \
       $(wildcard $(_$(foo)_DIR)/diversity.mk)))

ifneq ($(strip $(_t1)),)
-include $(_t1)
endif  # ifneq  ($(_t1),)




endif #ifeq "$(origin KMODULE_INCLUDED_COMPS)" "undefined"

#-----------------------------------------------------------------------
# Obsolete variables check
ifdef KMODULE
  $(warning KMODULE variable should not be used. Use _kmodule_ in _TMDIVERSITY instead.)
endif
ifdef KERNEL_MODE
  $(warning KERNEL_MODE variable should not be used. Use _kmode_ in _TMDIVERSITY instead.)
endif

# Backwards compatibility (deprecated):
kmodule_build kmodule_install:
	@echo '**WARNING: target $@ is deprecated. Do not use it.'
	@echo 'A toplevel SDE2 component for a kernel module/driver should define'
	@echo 'variable KMODULE_INCLUDED_COMPS to trigger a kmodule build&install'
