#-----------------------------------------------------------------------
# COPYRIGHT (c) 2007 NXP Semiconductors  
# File:        libs.mk
# Author:      Peter Kazakov
# Description: Add proper suffixes to the library names.
# Details:     We have the following steps
#              1. We scan recursively for all libraries (and DLLs). This step is 
#                 configuration specific, there is different behaviour for tm_psos, nullos and Microsoft (nt & ce).
#              2. If there exists circular dependencies, we report an warning.
#              3. We include all present diversity.mk
#              4. We compute the proper suffixes (flavours) for the libraries. This includes:
#                 4.1. Library diversity defined in _<CompName>_DIVERSITY
#                 4.2. Library rel.mode, if the default must be overriden, defined in _<CompName>_SUFFIX
#                 4.3. Replace library (in runtime), defined in _<CompName>_REPLACE
#                 4.4. If the libray is replaced with an empty one, we remove _* suffix
#              5. We compute all present paths for the binary release
#-----------------------------------------------------------------------

_SDE_DEP_MAKEFILES +=   $(DIR_SDE)/libs.mk

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# We don't want to bother computing _SDE_IMPORT_DLLS and
# _SDE_IMPORT_LIBS unless we need to.
#
# We avoid it if we're doing an _sde_print_vars on anything but
# _SDE_IMPORT_DLLS or _SDE_IMPORT_LIBS, and the usual suspects like rgd,
# clean, clean_all, makelist, etc.
#
# If MAKECMDGOALS is empty, we're doing all.

ifeq            (,$(filter _SDE_IMPORT_DLLS _SDE_IMPORT_LIBS _SDE_REQUIRED_PATH_DLLS _SDE_LIB_PATHS _SDE_LIBS_NOSFX _SDE_REQUIRED_PATH_LIBS VPATH _SDE_DLLS _SDE_LIBRARIES _SDE_DEP_LIBRARIES _SDE_LIBS_DIVERSITY _SDE_RECURSE_STATIC_SFX_LIBS _SDE_RECURSE_STATIC_LIBS _t1,$(var)))
_t :=   _sde_print_var
else   # ifeq   (,$(filter _SDE_IMPORT_DLLS _SDE_IMPORT_LIBS _SDE_REQUIRED_PATH_DLLS _SDE_LIB_PATHS _SDE_LIBS_NOSFX _SDE_REQUIRED_PATH_LIBS VPATH _SDE_DLLS _SDE_LIBRARIES _SDE_DEP_LIBRARIES _SDE_LIBS_DIVERSITY _SDE_RECURSE_STATIC_SFX_LIBS _SDE_RECURSE_STATIC_LIBS _t1,$(var)))
_t :=
endif  # ifeq   (,$(filter _SDE_IMPORT_DLLS _SDE_IMPORT_LIBS _SDE_REQUIRED_PATH_DLLS _SDE_LIB_PATHS _SDE_LIBS_NOSFX _SDE_REQUIRED_PATH_LIBS VPATH _SDE_DLLS _SDE_LIBRARIES _SDE_DEP_LIBRARIES _SDE_LIBS_DIVERSITY _SDE_RECURSE_STATIC_SFX_LIBS _SDE_RECURSE_STATIC_LIBS _t1,$(var)))

ifeq            (,$(MAKECMDGOALS))
_u :=   all
else   # ifeq   (,$(MAKECMDGOALS))
_u :=   $(MAKECMDGOALS)
endif  # ifeq   (,$(MAKECMDGOALS))


# Macro to get diversity for a specified component.
# Usage: $(call _SDE_GET_LIB_DIV,phCompName)
_SDE_GET_LIB_DIV=$(_$(1)_DIVERSITY)$(if $(findstring $(KMODE_SUFFIX),$(_$(1)_DIVERSITY)),,$(KMODE_SUFFIX))

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# We distinguish three types of "linking behaviour".
# The first is Microsoft type, which has a DLLs. It is used for nt and ce.
# The second is unix ('nullos') type, no DLLs. It is used for nullos, vxworks and mips_psos
# The third type is tm_psos, it is a mixture. We use DLLs, but we do not link them in the final executable (image).
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
_SDE_LINKSTYLE = nullos
ifeq ($(_TMTGTOSCLASS),nt)
_SDE_LINKSTYLE = ms
else
ifeq ($(_TMTGTOSCLASS),ce)
_SDE_LINKSTYLE = ms
else
ifneq ($(findstring $(DIR_CONFIG),x86gnu_nullos mipsgnu_linux x86gnu_linux armgnu_linux),)
_SDE_LINKSTYLE = linux
else
ifneq ($(filter $(_TMTGTOSCLASS),psos nullos),)
ifeq ($(_TMTGTCPUCLASS),tm)
_SDE_LINKSTYLE = tm
endif
endif
endif
endif
endif

ifneq ($(_SDE_LINKSTYLE),nullos)
_SDE_LIB_PATHS     += $(_SDE_DIRLIB_SUFFIX)$(_SDE_DIR_LIB)/$(_SDE_DIVERSITY)
endif
_SDE_LIB_PATHS     += $(_SDE_DIRLIB_SUFFIX)$(_SDE_DIR_LIB)

_SDE_ONLYSTATICLIB :=
ifeq            ($(_SDE_BUILD_TARGET),LIB)
ifeq            ($(EXPORTS),)
_SDE_ONLYSTATICLIB := 1
endif
endif

ifneq           (,$(filter-out rgd clean clean_all makelist $(_t), $(_u)))
ifdef           LIBS
ifneq           ($(_SDE_ONLYSTATICLIB),1)

#-----------------------------------------------------------------------
# At this point:
#
# LIBS
#   list of components this component references, unadorned
#
# Define:
#
# _SDE_IMPORT_LIBS
#   List of libraries to be statically linked.  Each does not include diversity and REL_SUFFIX
#
# _SDE_IMPORT_DLLS
#   List of libraries that are to be referenced as DLLs.  No adornment.

ifeq ($(_SDE_LINKSTYLE),ms)               # nt and ce case
ifeq ($(_SDE_BUILD_TARGET),LIB)
_SDE_IMPORT_DLLS :=                                                           \
    $(shell                                                                   \
        $(ECHO) $(_SDE_DIR_LIB)/$(_SDE_DIVERSITY)/$(_SDE_DLL_PREFIX) .lib $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .l /tmp/$(_SDE_LIB_CONFIGURATION_DEF)/$(_SDE_DIVERSITY)/ $(LOC_LIST_FILENAME) $(LIBS) \
        | $(GAWK) -f $(DIR_SDE)/awk/recursedlls.awk                       \
    )

_SDE_IMPORT_LIBS :=                                                           \
   $(shell                                                                   \
        $(ECHO) $(_SDE_DIR_LIB)/$(_SDE_DIVERSITY)/$(_SDE_DLL_PREFIX) .lib $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .l /tmp/$(_SDE_LIB_CONFIGURATION_DEF)/$(_SDE_DIVERSITY)/ $(LOC_LIST_FILENAME) $(LIBS) \
        | $(GAWK) -f $(DIR_SDE)/awk/recurselibs.awk                       \
    )
else
ifeq ($(_TMLINKTYPE),static)
_SDE_IMPORT_DLLS :=
_SDE_IMPORT_LIBS :=                                                           \
    $(shell                                                                   \
        $(ECHO) $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .l $(LIBS) \
        | $(GAWK) -f $(DIR_SDE)/awk/recursestatic.awk                     \
    )
else
_SDE_IMPORT_DLLS :=                                                           \
    $(shell                                                                   \
        $(ECHO) $(_SDE_DIR_LIB)/$(_SDE_DIVERSITY)/ .lib $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .l /tmp/$(_SDE_LIB_CONFIGURATION_DEF)/$(_SDE_DIVERSITY)/ $(LOC_LIST_FILENAME) $(LIBS) \
        | $(GAWK) -f $(DIR_SDE)/awk/recursedlls.awk                       \
    )

_SDE_IMPORT_LIBS :=                                                           \
   $(shell                                                                   \
        $(ECHO) $(_SDE_DIR_LIB)/$(_SDE_DIVERSITY)/ .lib $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .l /tmp/$(_SDE_LIB_CONFIGURATION_DEF)/$(_SDE_DIVERSITY)/ $(LOC_LIST_FILENAME) $(LIBS) \
        | $(GAWK) -f $(DIR_SDE)/awk/recurselibs.awk                       \
    )
endif 
endif
else
ifeq ($(_SDE_LINKSTYLE),linux)               # x86gnu_nullos
ifeq ($(_TMLINKTYPE),static)
_SDE_IMPORT_DLLS :=
_SDE_IMPORT_LIBS :=                                                           \
    $(shell                                                                   \
        $(ECHO) $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .l $(LIBS) \
        | $(GAWK) -f $(DIR_SDE)/awk/recursestatic.awk                     \
    )
else

_SDE_IMPORT_DLLS :=                                                           \
    $(shell                                                                   \
        $(ECHO) $(_SDE_DIR_LIB)/$(_SDE_DIVERSITY)/lib .so $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .l /tmp/$(_SDE_LIB_CONFIGURATION_DEF)/$(_SDE_DIVERSITY)/  $(LOC_LIST_FILENAME) $(LIBS) \
        | $(GAWK) -f $(DIR_SDE)/awk/recursedlls.awk                       \
    )

_SDE_IMPORT_LIBS :=                                                           \
   $(shell                                                                   \
        $(ECHO) $(_SDE_DIR_LIB)/$(_SDE_DIVERSITY)/lib .so $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .l /tmp/$(_SDE_LIB_CONFIGURATION_DEF)/$(_SDE_DIVERSITY)/  $(LOC_LIST_FILENAME) $(LIBS) \
        | $(GAWK) -f $(DIR_SDE)/awk/recurselibs.awk                       \
    )

endif #ifeq ($(_TMLINKTYPE),static)
else

ifeq ($(_SDE_LINKSTYLE),nullos)           # nullos

ifeq            ($(_SDE_BUILD_TARGET),LIB)
_SDE_IMPORT_LIBS := $(LIBS)
else   # ifeq   ($(_SDE_BUILD_TARGET),LIB)

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
# We're building an executable.  All executables are built fully static. 

_SDE_IMPORT_LIBS :=                                                           \
    $(shell                                                                   \
        $(ECHO) $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .l $(LIBS) \
        | $(GAWK) -f $(DIR_SDE)/awk/recursestatic.awk                     \
    )
endif  # ifeq   ($(_SDE_BUILD_TARGET),LIB)

else                                      # tm_psos

ifeq            ($(_SDE_BUILD_TARGET),LIB)
ifndef BUILD_DLLS
_SDE_IMPORT_DLLS :=                                                           \
    $(shell                                                                   \
        $(ECHO) $(_SDE_DIR_LIB)/$(_SDE_DIVERSITY)/lib .dll $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .l /tmp/$(_SDE_LIB_CONFIGURATION_DEF)/$(_SDE_DIVERSITY)/  $(LOC_LIST_FILENAME) $(LIBS) \
        | $(GAWK) -f $(DIR_SDE)/awk/recursedlls.awk                       \
    )
else
_SDE_IMPORT_DLLS :=                                                           \
    $(shell                                                                   \
        $(ECHO) $(_SDE_DIR_LIB)/$(_SDE_DIVERSITY)/lib .dll $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .l /tmp/$(_SDE_LIB_CONFIGURATION_DEF)/$(_SDE_DIVERSITY)/  $(LOC_LIST_FILENAME) \
	$(filter-out NONE,$(BUILD_DLLS)) SDE2LIST $(LIBS)| $(GAWK) -f $(DIR_SDE)/awk/recursedlls_new.awk                       \
    )
endif

ifndef BUILD_LIBS
_SDE_IMPORT_LIBS :=                                                           \
    $(shell                                                                   \
        $(ECHO) $(_SDE_DIR_LIB)/$(_SDE_DIVERSITY)/lib .dll $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .l /tmp/$(_SDE_LIB_CONFIGURATION_DEF)/$(_SDE_DIVERSITY)/  $(LOC_LIST_FILENAME) $(LIBS) \
        | $(GAWK) -f $(DIR_SDE)/awk/recurselibs.awk                       \
    )
else
_SDE_IMPORT_LIBS :=                                                           \
    $(shell                                                                   \
        $(ECHO) $(_SDE_DIR_LIB)/$(_SDE_DIVERSITY)/lib .dll $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .l /tmp/$(_SDE_LIB_CONFIGURATION_DEF)/$(_SDE_DIVERSITY)/  $(LOC_LIST_FILENAME) \
	$(filter-out NONE,$(BUILD_LIBS)) SDE2LIST $(LIBS)| $(GAWK) -f $(DIR_SDE)/awk/recurselibs_new.awk                       \
    )
endif

else   # ifeq   ($(_SDE_BUILD_TARGET),LIB)


#-----------------------------------------------------------------------
# We're building an executable.  In TriMedia, the executables are built
# either for static or dynamic linking.
#-----------------------------------------------------------------------

ifeq ($(_TMLINKTYPE),static)

_SDE_IMPORT_DLLS :=

_SDE_IMPORT_LIBS :=                                                           \
    $(shell                                                                   \
        $(ECHO) $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .l $(LIBS) \
        | $(GAWK) -f $(DIR_SDE)/awk/recursestatic.awk                     \
    )
else

ifndef DEFERRED_DLLS
_SDE_IMPORT_DLLS :=                                                           \
    $(shell                                                                   \
        $(ECHO) $(_SDE_DIR_LIB)/$(_SDE_DIVERSITY)/lib .dll $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .l /tmp/$(_SDE_LIB_CONFIGURATION_DEF)/$(_SDE_DIVERSITY)/  $(LOC_LIST_FILENAME) $(LIBS) \
        | $(GAWK) -f $(DIR_SDE)/awk/recursedlls.awk                       \
    )
else
ifeq ($(strip $(DEFERRED_DLLS)),none)
_SDE_IMPORT_DLLS :=                                                           \
    $(shell                                                                   \
        $(ECHO) $(_SDE_DIR_LIB)/$(_SDE_DIVERSITY)/lib .dll $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .l /tmp/$(_SDE_LIB_CONFIGURATION_DEF)/$(_SDE_DIVERSITY)/  $(LOC_LIST_FILENAME) $(LIBS) \
        | $(GAWK) -f $(DIR_SDE)/awk/recursedlls.awk                       \
    )
endif
endif
ifndef DEFERRED_DLLS
_SDE_IMPORT_LIBS :=                                                           \
    $(shell                                                                   \
        $(ECHO) $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .l $(LIBS) \
        | $(GAWK) -f $(DIR_SDE)/awk/recursestatic.awk                     \
    )
else
ifeq ($(strip $(DEFERRED_DLLS)),none)
_SDE_IMPORT_LIBS :=                                                           \
    $(shell                                                                   \
        $(ECHO) $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .l $(LIBS) \
        | $(GAWK) -f $(DIR_SDE)/awk/recursestatic.awk                     \
    )
endif
endif
endif  # ifeq ($(_TMLINKTYPE),static)

endif  # ifeq   ($(_SDE_BUILD_TARGET),LIB)
endif
endif
endif

#-----------------------------------------------------------------------
# Check whether circular dependencies are found
ifeq ($(findstring ERR_CIRC_COMPS_, $(_SDE_IMPORT_LIBS) $(_SDE_IMPORT_DLLS)),ERR_CIRC_COMPS_)
_SDE_CIR_DEP_LIBS := $(subst ERR_CIRC_COMPS_,,$(filter ERR_CIRC_COMPS_%, \
                     $(_SDE_IMPORT_LIBS) $(_SDE_IMPORT_DLLS)))
_SDE_IMPORT_LIBS := $(filter-out ERR_CIRC_COMPS_%, $(_SDE_IMPORT_LIBS))
_SDE_IMPORT_DLLS := $(filter-out ERR_CIRC_COMPS_%, $(_SDE_IMPORT_DLLS))
endif

#-----------------------------------------------------------------------
# _SDE_RECURSE_STATIC_LIBS is needed for 
# 1. makefile to include all diversity.mk files
# 2. Perl script to know right order of the libraries 

_SDE_RECURSE_STATIC_LIBS :=

ifneq ($(_SDE_BUILD_TARGET),LIB)
# For executables, only for Microsoft and Linux it differs from _SDE_IMPORT_LIBS
_SDE_RECURSE_STATIC_LIBS :=                                                           \
    $(shell                                                                   \
        $(ECHO) $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .l $(LIBS) \
        | $(GAWK) -f $(DIR_SDE)/awk/recursestatic.awk                     \
    )
else
ifneq ($(EXPORTS),)
# For libraries we are interested to calculate this only if we have DLLs
_SDE_RECURSE_STATIC_LIBS :=                                                           \
    $(shell                                                                   \
        $(ECHO) $(_SDE_TMTGTBUILDROOT)/ /tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY)/ .l $(LIBS) \
        | $(GAWK) -f $(DIR_SDE)/awk/recursestatic.awk                     \
    )
endif
endif

#-----------------------------------------------------------------------
# Check whether circular dependencies are found
ifeq ($(findstring ERR_CIRC_COMPS_, $(_SDE_RECURSE_STATIC_LIBS)),ERR_CIRC_COMPS_)
_SDE_CIR_DEP_LIBS += $(subst ERR_CIRC_COMPS_,,$(filter ERR_CIRC_COMPS_%, \
                     $(_SDE_RECURSE_STATIC_LIBS)))
_SDE_RECURSE_STATIC_LIBS := $(filter-out ERR_CIRC_COMPS_%, $(_SDE_RECURSE_STATIC_LIBS))
endif
ifneq ($(_SDE_CIR_DEP_LIBS),)
_SDE_WARNING    += "Warning: Circular dependencies detected! Check LIBS section of $(sort $(_SDE_CIR_DEP_LIBS)) components\\n"
endif

#-----------------------------------------------------------------------
# Check whether *.l is missing
_comma_ := ,
ifeq ($(findstring ERR_L_, $(_SDE_IMPORT_LIBS) $(_SDE_IMPORT_DLLS) $(_SDE_RECURSE_STATIC_LIBS)),ERR_L_)
_SDE_MISSING_LFILES := $(filter ERR_L_%, $(_SDE_IMPORT_LIBS) $(_SDE_IMPORT_DLLS) $(_SDE_RECURSE_STATIC_LIBS))
_SDE_IMPORT_LIBS := $(filter-out $(_SDE_MISSING_LFILES), $(_SDE_IMPORT_LIBS))
_SDE_IMPORT_DLLS := $(filter-out $(_SDE_MISSING_LFILES), $(_SDE_IMPORT_DLLS))
_SDE_RECURSE_STATIC_LIBS := $(filter-out $(_SDE_MISSING_LFILES), $(_SDE_RECURSE_STATIC_LIBS))
_SDE_WARNING    += "Warning: $(subst $(_space_),$(_comma_),$(sort $(patsubst ERR_L_%,%.l,$(_SDE_MISSING_LFILES)))) file(s) in $(_SDE_TMTGTBUILDROOT)/comps/<CompName>/tmp/$(_SDE_LIB_CONFIGURATION)/$(_SDE_DIVERSITY) is/are missing. Please read Chapter 3.7.4 of the User Manual!!!\\n"
endif
    


#-----------------------------------------------------------------------
# Include the _xxx_DIVERSITY settings for all the components we
# reference
# _SDE_RECURSE_STATIC_LIBS is here without suffix. If there are with suffix, we accept that one overrides defaults
#-----------------------------------------------------------------------

_t1 :=   $(foreach foo,                                                    \
                  $(_SDE_RECURSE_STATIC_LIBS),                             \
                  $(if $(_$(foo)_REPLACE),                                 \
                      $(wildcard $(_$(_$(foo)_REPLACE)_DIR)/diversity.mk), \
                      $(wildcard $(_$(foo)_DIR)/diversity.mk)              \
                  )                                                        \
         )

ifneq           ($(strip $(_t1)),)
-include $(_t1)
endif  # ifneq  ($(_t1),)

#-----------------------------------------------------------------------
# We now have _<tmComponentName>_DIVERSITY defined with the appropriate suffix
# for each component listed in LIBS.  The definition does _not_ include
# the REL_SUFFIX.
#
# We have also _<tmComponentName>_SUFFIX in diversity.mk with the following values:
# empty or undefined    - compile always in default mode
# _g                    - always debug
# _r                    - always retail
# _a                    - always assert
# _<tmComponentName>_SUFFIX is placed after REL_SUFFIX and we have the following conversion for each library:
# _g_g                  -> _g
# _g_a                  -> _a
# _g_r                  -> empty
# _a_g                  -> _g
# _a_a                  -> _a
# _a_r                  -> empty
# _g                    -> unchanged
# _a                    -> unchanged
# _r                    -> empty
# Additionally, we can replace one component with another in a link time
# This can be done if you define _<ComponentName>_REPLACE, example:
# _tmComp1_REPLACE = tmComp2
# Warning: This replaces only the component name, not its flavours !!!
#-----------------------------------------------------------------------

_SDE_IMPORT_LIBS_REPLACED :=                                                   \
               $(foreach f, $(_SDE_IMPORT_LIBS),                               \
                  $(if $(_$(f)_REPLACE),                                       \
                     $(_$(f)_REPLACE),                                         \
                     $(f)))

_SDE_LIBS_DIVERSITY :=                                                         \
               $(foreach f,                                                    \
                  $(_SDE_IMPORT_LIBS_REPLACED),                                \
                  $(if $(findstring $(origin _$(f)_SUFFIX), undefined),        \
                      $(f)$(call _SDE_GET_LIB_DIV,$(f))$(REL_COMPS_SUFFIX),    \
                      $(patsubst %_r,%,$(f)$(call _SDE_GET_LIB_DIV,$(f))$(_$(f)_SUFFIX))))

_SDE_IMPORT_DLLS :=                                                            \
                  $(filter-out _%,                                             \
                    $(foreach                                                  \
                      f,                                                       \
                      $(_SDE_IMPORT_DLLS),                                     \
                      $(if $(_$(f)_REPLACE),                                   \
                        $(_$(f)_REPLACE),                                      \
                        $(f)                                                   \
                      )                                                        \
                    )                                                          \
                  )

#-----------------------------------------------------------------------
# First change, and then look at the paths

# strip DLLS and look into them
_SDE_REQUIRED_PATH_DLLS += $(foreach nosfxdll, $(_SDE_IMPORT_DLLS), \
        $(if $(wildcard $(_$(nosfxdll)_DIR)/$(_SDE_DIR_LIB_EXT)/$(_SDE_DIVERSITY)/), \
                $(_$(nosfxdll)_DIR)/$(_SDE_DIR_LIB_EXT)/$(_SDE_DIVERSITY),          \
        ))

_SDE_REQUIRED_PATH_DLLS += $(foreach nosfxdll, $(_SDE_IMPORT_DLLS), \
        $(if $(wildcard $(_$(nosfxdll)_DIR)/$(_SDE_DIR_LIB_EXT_DEF)/$(_SDE_DIVERSITY)/), \
                $(_$(nosfxdll)_DIR)/$(_SDE_DIR_LIB_EXT_DEF)/$(_SDE_DIVERSITY),          \
        ))


_SDE_LIB_PATHS     += $(_SDE_REQUIRED_PATH_DLLS:%=$(_SDE_DIRLIB_SUFFIX)%)

# strip LIBS and look into them; 
_SDE_LIBS_NOSFX    := $(foreach z, $(_SDE_LIBS_DIVERSITY),         \
                           $(word 1,$(subst _, ,$(z)))             \
                       )

_SDE_REQUIRED_PATH_LIBS += $(foreach nosfxlib, $(_SDE_LIBS_NOSFX), \
        $(if $(wildcard $(_$(nosfxlib)_DIR)/$(_SDE_DIR_LIB_EXT)/), \
             $(_$(nosfxlib)_DIR)/$(_SDE_DIR_LIB_EXT), \
        ))

_SDE_REQUIRED_PATH_LIBS += $(foreach nosfxlib, $(_SDE_LIBS_NOSFX), \
        $(if $(wildcard $(_$(nosfxlib)_DIR)/$(_SDE_DIR_LIB_EXT_DEF)/), \
             $(_$(nosfxlib)_DIR)/$(_SDE_DIR_LIB_EXT_DEF), \
        ))

_SDE_LIB_PATHS     += $(_SDE_REQUIRED_PATH_LIBS:%=$(_SDE_DIRLIB_SUFFIX)%)


# --------------------------------------------------------------------------
# This part is needed for the build_exe.pl perl script

_SDE_RECURSE_STATIC_LIBS_REPLACED :=                                           \
               $(foreach f, $(_SDE_RECURSE_STATIC_LIBS),                       \
                  $(if $(_$(f)_REPLACE),                                       \
                     $(_$(f)_REPLACE),                                         \
                     $(f)))

_SDE_RECURSE_STATIC_SFX_LIBS :=                                                \
               $(foreach f,                                                    \
                  $(_SDE_RECURSE_STATIC_LIBS_REPLACED),                        \
                  $(if $(findstring $(origin _$(f)_SUFFIX), undefined),        \
                     $(f)$(call _SDE_GET_LIB_DIV,$(f))$(REL_COMPS_SUFFIX),     \
                     $(patsubst %_r,%,$(f)$(_$(f)_DIVERSITY)$(_$(f)_SUFFIX))))


endif  # ifneq  ($(_SDE_ONLYSTATICLIB),1)
endif  # ifdef  LIBS
endif  # ifneq  (,$(filter-out rgd clean clean_all makelist $(_t), $(_u)))
