#-----------------------------------------------------------------------
# COPYRIGHT (c) 2007 NXP Semiconductors 
#
# sde/common.mk
#
# Description: Defines dependency check rules
#              Defines rules for generating objects
#              Defines rules for making option files
#
# Initial Author: Lee Boekelheide
# Maintained by : SDE2 Team
#
# NOTES:        This makefile builds $(_TMCFGPATH)/tmFlags.mk from the
#               environmental or command line variables passed in.
#
#               If the file already exists, its settings are checked
#               against the incoming variables, and if they differ, the
#               file is deleted so it'll get rebuilt.
#
#               This is intended to run under GNU make 3.77 or later.
#               It does not support Microsoft nmake or MKS make.  You
#               can try them if you like, but don't cry to me if there's
#               trouble.
#
#               Required input variables:
#                   _TMBIN    
#                   _TMCOMROOT
#                   _TMRELROOT
#                   _TMCFGPATH
#                   _TMTGTCPUTYPE
#                   _TMTGTENDIAN
#                   _TMTGTOS
#
#               Optional input variables:
#                   _ECHOMAKELINES
#
#------------------------------------------------------------------------------
#
# Input Variables
#
#   Variables that are directory names should not include a trailing /.
#
#   Required for all builds:
#
#     _TMBIN            Directory holding Unix-flavor tools like sh, cp,
#                       echo, rm and so on.
#
#     _TMCOMROOT        Common root is the root of the DVP build source
#                       tree.  It's where you fetched $/dvp1/common from
#                       source control.  The build process treats this
#                       tree as read only.
#
#     _TMRELROOT        Release root is the root of the DVP release
#                       tree.  It's where any output from the build
#                       process goes.
#
#     _TMTGTOS          target o/s                 $(_TMTGTOS)
#                         WinCE                    ce212 ce300
#                           NT                         nt4
#                        TriMedia                    psos200
#                          pSOS                      psos250
#                        boot mgr                      btm
#                        VXWorks                 vxworks vxworks540
#                          mtos                      RD24120
#                          cexec                      cexec
#			   nucleus		     nucleus
#                          UCos	                     ucos
#                          INTEGRITY                 integrity
#                          linux                     linux
#                          oscan                     oscan
#
#     _TMTGTENDIAN      target endianness      $(_TMTGTENDIAN)
#                             little                  el
#                              big                    eb
#
#     _TMTGTCPUTYPE     target cpu type       $(_TMTGTCPUTYPE)
#                            WinCE        r3940, r4300, r4640, arm720 or i486
#                          TriMedia       tm32, tm3260
#                            pSOS         r3940, r1910, r4450, 4kec, r4000, mips32, r3900 
#                           VXWorks       arm7, arm920t, arm940t, arm10, strongarm, arm926EJS, arm946, r3940, r4300 
#                            mtos         RD24120
#	                     cexec        arm7, arm920t, arm940t, arm10, strongarm, arm926EJS, arm946
#			     nucleus      arm7, arm920t, arm940t, arm10, strongarm, arm926EJS, arm946
#                            ucos         arm7, arm920t, arm940t, arm10, strongarm, arm926EJS, arm946
#                            integrity    mips, 
#
#			
#   Optional input variables:
#
#     _ECHOMAKELINES or _TMECHO Echo all if it is set
#
#     _TMCFGPATH        $(_TMRELROOT)/cfg/$(_TMTGTPATH)
#
#------------------------------------------------------------------------------
_TMCFGPATH = $(_SDE_DIR_LIB)
_TMINCPATH = $(_SDE_DIR_LIB)

_SDE_SINGLE_QUOTE_MAKE_VERSIONS=3.81
ifeq ($(UNAME),cygwin)
ifneq (,$(filter $(MAKE_VERSION),$(_SDE_SINGLE_QUOTE_MAKE_VERSIONS)))
_SDE_QUOTE='
endif
endif

_SDE_QUOTE?="
#------------------------------------------------------------------------------
#
# There are 2 files of interest here.
#
# $(_TMCFGPATH)/tmFlags.mk is used to define the other TMFL_xxx build
# flags, in makefile form.
#
# $(_TMCFGPATH)/tmFlags.h is used to define the other TMFL_xxx build
# flags, in .h header file form.
#
# Neither tmFlags.mk nor tmFlags.h should be modified by the user.
# They are both built here.
#
# We check the validity of the settings in tmFlags.mk and delete it --
# and tmFlags.h -- if there's something wrong.
#
# tmFlags.mk depends on tmFlags.h so that if the user foolishly
# modifies tmFlags.h, both tmFlags.mk and tmFlags.h get rebuilt.
#
#------------------------------------------------------------------------------
GLOBAL_DEPENDENTS +=    $(DIR_SDE)/tmflags/tmFlagsMak.mk
#------------------------------------------------------------------------------
#
# If necessary, build a new $(_TMCFGPATH)/tmFlags.mk.
#
# TMFL_BUILD_VERSION is the only user-modifiable build
# flags.  All others are derived from the environmental or command line
# settings passed in to the main shared common makefile.
#
_DVP_COMMON_TOOLS_TMFLAGS_MAK_MAK_ := $(_TMROOT)/sde/tmflags/tmFlagsMak.mk
# Hard wire the constants so we can use them here
include $(DIR_SDE)/tmflags/tmFlags_values.mk

# Strip leading and trailing spaces

TMFL_BUILD_VERSION :=   $(strip $(TMFL_BUILD_VERSION))
TMFL_CPU :=             $(strip $(TMFL_CPU))
TMFL_ENDIAN :=          $(strip $(TMFL_ENDIAN))
TMFL_OS :=              $(strip $(TMFL_OS))

#------------------------------------------------------------------------------
#
# If either there is no $(_TMCFGPATH)/tmFlags.mk yet, or if its
# settings disagree with the settings passed in to this makefile, we
# make it.
#
# We will define:
#
#    TMFL_BUILD_VERSION     <MajorVersion.MinorVersion.BuildNumber>
#                           e. g. 1.02.0497
#    TMFL_CPU               TMFL_CPU_xxx
#    TMFL_ENDIAN            TMFL_ENDIAN_BIG or TMFL_ENDIAN_LITTLE
#    TMFL_OS                TM_OS_xxx
#
#
# Constants used in the above definitions:
#
#    TMFL_CPU_TYPE_MASK         0xffff0000
#    TMFL_CPU_TYPE_X86          0x00010000
#    TMFL_CPU_TYPE_MIPS         0x00020000
#    TMFL_CPU_TYPE_TM           0x00030000
#    TMFL_CPU_TYPE_HP           0x00040000
#    TMFL_CPU_TYPE_ARM          0x00050000
#    TMFL_CPU_TYPE_DSPRD        0x00060000
#    TMFL_CPU_TYPE_8051         0x00070000
#    TMFL_CPU_TYPE_DSPVD        0x00080000
#
#    TMFL_CPU_MODEL_MASK        0x0000ffff
#    TMFL_CPU_MODEL_I486        0x00000001
#    TMFL_CPU_MODEL_R3940       0x00000002
#    TMFL_CPU_MODEL_R4300       0x00000003
#    TMFL_CPU_MODEL_TM1100      0x00000004
#    TMFL_CPU_MODEL_TM1300      0x00000005
#    TMFL_CPU_MODEL_TM32        0x00000006
#    TMFL_CPU_MODEL_HP          0x00000007
#    TMFL_CPU_MODEL_R4640       0x00000008
#    TMFL_CPU_MODEL_ARM7        0x00000009
#    TMFL_CPU_MODEL_ARM920T     0x0000000a
#    TMFL_CPU_MODEL_ARM940T     0x0000000b
#    TMFL_CPU_MODEL_ARM10       0x0000000c
#    TMFL_CPU_MODEL_STRONGARM   0x0000000d
#    TMFL_CPU_MODEL_RD24120   0x0000000e
#    TMFL_CPU_MODEL_ARM926EJS   0x0000000f
#    TMFL_CPU_MODEL_ARM946      0x00000010
#    TMFL_CPU_MODEL_R1910       0x00000011
#    TMFL_CPU_MODEL_R4450       0x00000012
#    TMFL_CPU_MODEL_TM3260      0x00000013
#    TMFL_CPU_MODEL_8051	0x00000014
#    TMFL_CPU_MODEL_ARM922T	0x00000015
#    TMFL_CPU_4KEC              0x00000016
#    TMFL_CPU_R4000             0x00000017
#    TMFL_CPU_MIPS32            0x00000018
#    TMFL_CPU_ARM720            0x00000019
#    TMFL_CPU_R3900             0x0000001a
#    TMFL_CPU_RD16023           0x0000001b
#    TMFL_CPU_RD16024           0x0000001c
#    TMFL_CPU_MODEL_ARM1176JTZS 0x0000001d
#    TMFL_CPU_MODEL_ARM966      0x0000001e
#    TMFL_CPU_MODEL_ARM968      0x0000001f
#
#    TMFL_ENDIAN_BIG            1
#    TMFL_ENDIAN_LITTLE         0
#
#    TMFL_OS_MASK               0xff000000
#    TMFL_OS_VERSION_MASK       0x00ffffff
#    TMFL_OS_BTM                0x00000000
#    TMFL_OS_CE                 0x01000000
#    TMFL_OS_CE212              0x01020102
#    TMFL_OS_CE300              0x01030000
#    TMFL_OS_NT                 0x02000000
#    TMFL_OS_NT4                0x02040000
#    TMFL_OS_PSOS               0x03000000
#    TMFL_OS_PSOS250            0x03020500
#    TMFL_OS_PSOS200            0x03020000
#    TMFL_OS_NULLOS             0x04000000
#    TMFL_OS_ECOS               0x05000000
#    TMFL_OS_VXWORKS            0x06000000
#    TMFL_OS_MTOS               0x07000000
#    TMFL_OS_CEXEC              0x08000000
#    TMFL_OS_NUCLEUS            0x09000000
#    TMFL_OS_UCOS               0x0A000000
#    TMFL_OS_INTEGRITY          0x0B000000
#    TMFL_OS_LINUX              0x0C000000
#    TMFL_OS_OSCAN              0x0D000000
#
#    TMFL_SCOPE_SP              0
#    TMFL_SCOPE_MP              1
#
# They will be defined as decimal numbers with the above hex values.
#
#
# Combinations defined for convenience:
#
#    TMFL_CPU_I486      (TMFL_CPU_TYPE_X86 | TMFL_CPU_MODEL_I486)
#    TMFL_CPU_R3940     (TMFL_CPU_TYPE_MIPS | TMFL_CPU_MODEL_R3940)
#    TMFL_CPU_R4300     (TMFL_CPU_TYPE_MIPS | TMFL_CPU_MODEL_R4300)
#    TMFL_CPU_TM1100    (TMFL_CPU_TYPE_TM | TMFL_CPU_MODEL_TM1100)
#    TMFL_CPU_TM1300    (TMFL_CPU_TYPE_TM | TMFL_CPU_MODEL_TM1300)
#    TMFL_CPU_TM32      (TMFL_CPU_TYPE_TM | TMFL_CPU_MODEL_TM32)
#    TMFL_CPU_RD24120 (TMFL_CPU_TYPE_DSPRD | TMFL_CPU_MODEL_RD24120)
#    The same for arm processors and r4640, r1910, r4450, tm3260
#
# Booleans defined for convenience:
#
#    TMFL_CPU_IS_X86    ((TMFL_CPU & TMFL_CPU_TYPE_MASK) == TMFL_CPU_TYPE_X86)
#    TMFL_CPU_IS_MIPS   ((TMFL_CPU & TMFL_CPU_TYPE_MASK) == TMFL_CPU_TYPE_MIPS)
#    TMFL_CPU_IS_TM     ((TMFL_CPU & TMFL_CPU_TYPE_MASK) == TMFL_CPU_TYPE_TM)
#
#    TMFL_OS_IS_BTM     ((TMFL_OS & TMFL_OS_MASK) == TMFL_OS_BTM)
#    TMFL_OS_IS_CE      ((TMFL_OS & TMFL_OS_MASK) == TMFL_OS_CE)
#    TMFL_OS_IS_NT      ((TMFL_OS & TMFL_OS_MASK) == TMFL_OS_NT)
#    TMFL_OS_IS_PSOS    ((TMFL_OS & TMFL_OS_MASK) == TMFL_OS_PSOS)
#    TMFL_OS_IS_ECOS    ((TMFL_OS & TMFL_OS_MASK) == TMFL_OS_ECOS)  
#    TMFL_OS_IS_VXWORKS ((TMFL_OS & TMFL_OS_MASK) == TMFL_OS_VXWORKS)  
#    TMFL_OS_IS_MTOS    ((TMFL_OS & TMFL_OS_MASK) == TMFL_OS_MTOS)  
#    TMFL_OS_IS_CEXEC   ((TMFL_OS & TMFL_OS_MASK) == TMFL_OS_CEXEC)
#    TMFL_OS_IS_NUCLEUS ((TMFL_OS & TMFL_OS_MASK) == TMFL_OS_NUCLEUS)  
#    TMFL_OS_IS_UCOS    ((TMFL_OS & TMFL_OS_MASK) == TMFL_OS_UCOS)  
#    TMFL_OS_IS_INTEGRITY ((TMFL_OS & TMFL_OS_MASK) == TMFL_OS_INTEGRITY)  
#    TMFL_OS_IS_LINUX     ((TMFL_OS & TMFL_OS_MASK) == TMFL_OS_LINUX)  
#    TMFL_OS_IS_OSCAN     ((TMFL_OS & TMFL_OS_MASK) == TMFL_OS_OSCAN)  
#
#
# Since tmFlags.h depends on tmFlags.mk, the .h will get rebuilt when
# the .mk gets rebuilt.

RM-F = $(RM)

_TMFLAGS_MAK_ :=        $(_TMCFGPATH)/tmFlags.mk
_TMFLAGS_H_ :=          $(_TMINCPATH)/tmFlags.h
_PHFLAGS_MAK_ :=        $(_TMCFGPATH)/phFlags.mk
_PHFLAGS_H_ :=          $(_TMINCPATH)/phFlags.h


# If there's a tmFlags.mk; we need to check if its settings agree with
# what we expect.

TMFL_CPU :=
TMFL_ENDIAN :=
TMFL_OS :=

ifneq "$(filter-out $(_SDE_TARGETS_NO_TMFLAGS),$(MAKECMDGOALS))" ""
  _INCLUDE_FLAGS_MAK_FILES:=1
endif
ifeq "$(MAKECMDGOALS)" ""
  _INCLUDE_FLAGS_MAK_FILES:=1
endif

ifdef _INCLUDE_FLAGS_MAK_FILES
-include $(_TMFLAGS_MAK_)
-include $(_PHFLAGS_MAK_)
endif

# Strip leading and trailing spaces

TMFL_BUILD_VERSION :=   $(strip $(TMFL_BUILD_VERSION))
TMFL_CPU :=             $(strip $(TMFL_CPU))
TMFL_ENDIAN :=          $(strip $(TMFL_ENDIAN))
TMFL_OS :=              $(strip $(TMFL_OS))

# Hard wire the constants in case tmFlags.mk corrupted them
include $(DIR_SDE)/tmflags/tmFlags_values.mk

# Validate the TMFL_xxx
#
# If we find something wrong, we clear _ALL_OK_.

_ALL_OK_ :=     ok

# Check cpu
_UPPER_CPU := $(shell $(ECHO) $(_TMTGTCPUTYPE)|$(GAWK) -f $(_TMROOT)/sde/awk/upper.awk)
ifneq ($(filter $(_TMTGTCPUTYPE),$(_SDE_SUPPORTED_CPUTYPES)),)
ifneq ($(TMFL_CPU),$(TMFL_CPU_$(_UPPER_CPU)))
_ALL_OK_ := $(_TMTGTCPUTYPE)
endif
endif

# Check endianness

ifeq            ($(_TMTGTENDIAN), eb)
ifneq           ($(TMFL_ENDIAN),$(TMFL_ENDIAN_BIG))
_ALL_OK_ :=eb
endif
endif
ifeq            ($(_TMTGTENDIAN), el)
ifneq           ($(TMFL_ENDIAN),$(TMFL_ENDIAN_LITTLE))
_ALL_OK_ :=el
endif
endif

# Check o/s
_UPPER_OS := $(shell $(ECHO) $(_TMTGTOS)|$(GAWK) -f $(_TMROOT)/sde/awk/upper.awk)
ifneq ($(filter $(_TMTGTOS),$(_SDE_SUPPORTED_OSTYPES)),)
ifneq ($(TMFL_OS),$(TMFL_OS_$(_UPPER_OS)))
_ALL_OK_ := $(_TMTGTOS)
endif
endif

# Check compilation
ifneq           ($(_ALL_OK_), ok)

# Either there was no tmFlags.mk or the settings in tmFlags.mk didn't
# agree with those we were called with.  Set the variables ourselves and
# force a rebuild.
TMFL_CPU :=             $(TMFL_CPU_$(_UPPER_CPU))
TMFL_CPU_IS_8051  :=      0
TMFL_CPU_IS_ARM   :=      0
TMFL_CPU_IS_HP    :=      0
TMFL_CPU_IS_MIPS  :=      0
TMFL_CPU_IS_DSPRD :=      0
TMFL_CPU_IS_DSPVD :=      0
TMFL_CPU_IS_TM    :=      0
TMFL_CPU_IS_X86   :=      0

ifneq            ($(filter $(_TMTGTCPUTYPE),$(_SDE_CPUTYPES_8051)),)
TMFL_CPU_IS_8051:=      1
else
ifneq            ($(filter $(_TMTGTCPUTYPE),$(_SDE_CPUTYPES_arm)),)
TMFL_CPU_IS_ARM :=      1
else
ifneq            ($(filter $(_TMTGTCPUTYPE),$(_SDE_CPUTYPES_hp)),)
TMFL_CPU_IS_HP  :=      1
else
ifneq            ($(filter $(_TMTGTCPUTYPE),$(_SDE_CPUTYPES_mips)),)
TMFL_CPU_IS_MIPS :=     1
else
ifneq            ($(filter $(_TMTGTCPUTYPE),$(_SDE_CPUTYPES_dsprd)),)
TMFL_CPU_IS_DSPRD :=     1
else
ifneq            ($(filter $(_TMTGTCPUTYPE),$(_SDE_CPUTYPES_dspvd)),)
TMFL_CPU_IS_DSPVD :=     1
else
ifneq            ($(filter $(_TMTGTCPUTYPE),$(_SDE_CPUTYPES_tm)),)
TMFL_CPU_IS_TM  :=      1
else
ifneq            ($(filter $(_TMTGTCPUTYPE),$(_SDE_CPUTYPES_x86)),)
TMFL_CPU_IS_X86  :=     1
else
_ERROR_ += _TMTGTCPUTYPE does not have a legal value.\nLegal values are: \n\t$(_SDE_SUPPORTED_CPUTYPES)\n
endif
endif
endif
endif
endif
endif
endif
endif
endif # ifneq           ($(_ALL_OK_), ok)

ifeq            ($(_TMTGTENDIAN), eb)
TMFL_ENDIAN :=  $(TMFL_ENDIAN_BIG)
else
ifeq            ($(_TMTGTENDIAN), el)
TMFL_ENDIAN :=  $(TMFL_ENDIAN_LITTLE)
else
_ERROR_ += _TMTGTENDIAN does not have a legal value.\nLegal values are: \n\teb, el, be, le.\n
endif
endif

TMFL_TOOLCHAIN_NCSC := 0
TMFL_TOOLCHAIN_OSCI := 0
TMFL_TOOLCHAIN_DDK :=  0
ifeq ($(_TMTOOLCHAIN), ncsc)
TMFL_TOOLCHAIN_NCSC :=  1
else
ifeq ($(_TMTOOLCHAIN), osci)
TMFL_TOOLCHAIN_OSCI :=  1
else
ifeq ($(_TMTOOLCHAIN), ddk)
TMFL_TOOLCHAIN_DDK :=  1
endif
endif
endif

TMFL_MODE_KERNEL := 1
ifeq ($(DIR_CONFIG),x86_nt)
  TMFL_MODE_KERNEL := 0
else
  ifneq "$(filter $(_TMTGTOSCLASS),ce linux)" ""
    ifeq ($(findstring _kmode_,$(_TMDIVERSITY))$(findstring _kmodule_,$(_TMDIVERSITY)),)
      TMFL_MODE_KERNEL := 0
    endif
  endif
endif
TMFL_MODE_USER = $(if $(filter 1,$(TMFL_MODE_KERNEL)),0,1)

ifneq           ($(_ALL_OK_), ok)
TMFL_OS 		:=   $(TMFL_OS_$(_UPPER_OS))
TMFL_OS_IS_BTM 		:=   0
TMFL_OS_IS_CE 		:=   0
TMFL_OS_IS_CEXEC  	:=   0
TMFL_OS_IS_ECOS   	:=   0
TMFL_OS_IS_INTEGRITY  	:=   0
TMFL_OS_IS_LINUX  	:=   0
TMFL_OS_IS_MTOS   	:=   0
TMFL_OS_IS_NT 		:=   0
TMFL_OS_IS_NUCLEUS	:=   0
TMFL_OS_IS_NULLOS 	:=   0
TMFL_OS_IS_OSCAN  	:=   0
TMFL_OS_IS_PSOS 	:=   0
TMFL_OS_IS_UCOS   	:=   0
TMFL_OS_IS_VXWORKS	:=   0

ifneq            ($(filter $(_TMTGTOS),$(_SDE_OSTYPES_ce)),)
TMFL_OS_IS_CE :=        1
else
ifneq            ($(filter $(_TMTGTOS),$(_SDE_OSTYPES_cexec)),)
TMFL_OS_IS_CEXEC  :=    1
else
ifneq            ($(filter $(_TMTGTOS),$(_SDE_OSTYPES_ecos)),)
TMFL_OS_IS_ECOS   :=    1
else
ifneq            ($(filter $(_TMTGTOS),$(_SDE_OSTYPES_integrity)),)
TMFL_OS_IS_INTEGRITY   :=    1
else
ifneq            ($(filter $(_TMTGTOS),$(_SDE_OSTYPES_linux)),)
TMFL_OS_IS_LINUX   :=    1
else
ifneq            ($(filter $(_TMTGTOS),$(_SDE_OSTYPES_mtos)),)
TMFL_OS_IS_MTOS   :=    1
else
ifneq            ($(filter $(_TMTGTOS),$(_SDE_OSTYPES_nt)),)
TMFL_OS_IS_NT :=        1
else
ifneq            ($(filter $(_TMTGTOS),$(_SDE_OSTYPES_nucleus)),)
TMFL_OS_IS_NUCLEUS:=    1
else
ifneq            ($(filter $(_TMTGTOS),$(_SDE_OSTYPES_nullos)),)
TMFL_OS_IS_NULLOS :=    1
else
ifneq            ($(filter $(_TMTGTOS),$(_SDE_OSTYPES_oscan)),)
TMFL_OS_IS_OSCAN   :=    1
else
ifneq            ($(filter $(_TMTGTOS),$(_SDE_OSTYPES_psos)),)
TMFL_OS_IS_PSOS :=      1
else
ifneq            ($(filter $(_TMTGTOS),$(_SDE_OSTYPES_ucos)),)
TMFL_OS_IS_UCOS   :=    1
else
ifneq            ($(filter $(_TMTGTOS),$(_SDE_OSTYPES_vxworks)),)
TMFL_OS_IS_VXWORKS:=    1
else
_ERROR_ += \t_TMTGTOS does not have a legal value.Legal values are: \n\t$(_SDE_SUPPORTED_OSTYPES).\n
endif
endif
endif
endif
endif
endif
endif
endif
endif
endif
endif
endif
endif

# Force a rebuild of tmFlags.mk and tmFlags.h.
$(_TMFLAGS_MAK_):  chkconfig FORCE

$(_TMFLAGS_H_):    chkconfig FORCE

$(_PHFLAGS_MAK_):  chkconfig FORCE

$(_PHFLAGS_H_):    chkconfig FORCE

endif  # ifneq  ($(_ALL_OK_), ok)

#------------------------------------------------------------------------------
# Here we rebuild tmFlags.h
# 
# Note that the rebuild won't take place until the whole makefile tree
# has been read in, so it's important that all the TMFL_xxx build flags
# be correctly defined by now even at read time.

_TMFLAGS_H_DEPEND_ :=   $(GLOBAL_DEPENDENTS)

ifdef _INCLUDE_FLAGS_MAK_FILES

$(_TMFLAGS_H_):         $(_TMFLAGS_H_DEPEND_)
	@$(RM-F) $@
	-@$(MKDIR) $(_TMCFGPATH) $(_TMINCPATH)
	@$(ECHO) generating $(_TMFLAGS_H_);                             \
	$(CAT)  $(DIR_SDE)/tmflags/tmFlags_template.mk | $(SED)         \
	-e $(_SDE_QUOTE)s/TMFL_BUILD_VERSION_VALUE/$(strip $(TMFL_BUILD_VERSION))/g$(_SDE_QUOTE) \
	-e $(_SDE_QUOTE)s/TMFL_CPU_VALUE/$(strip $(TMFL_CPU))/g$(_SDE_QUOTE)                    \
	-e $(_SDE_QUOTE)s/TMFL_ENDIAN_VALUE/$(strip $(TMFL_ENDIAN))/g$(_SDE_QUOTE)              \
	-e $(_SDE_QUOTE)s/TMFL_OS_VALUE/$(strip $(TMFL_OS))/g$(_SDE_QUOTE)                      \
	-e $(_SDE_QUOTE)s/TMFL_CPU_IS_8051_VALUE/$(strip $(TMFL_CPU_IS_8051))/g$(_SDE_QUOTE)    \
	-e $(_SDE_QUOTE)s/TMFL_CPU_IS_ARM_VALUE/$(strip $(TMFL_CPU_IS_ARM))/g$(_SDE_QUOTE)      \
	-e $(_SDE_QUOTE)s/TMFL_CPU_IS_HP_VALUE/$(strip $(TMFL_CPU_IS_HP))/g$(_SDE_QUOTE)        \
	-e $(_SDE_QUOTE)s/TMFL_CPU_IS_MIPS_VALUE/$(strip $(TMFL_CPU_IS_MIPS))/g$(_SDE_QUOTE)    \
	-e $(_SDE_QUOTE)s/TMFL_CPU_IS_DSPRD_VALUE/$(strip $(TMFL_CPU_IS_DSPRD))/g$(_SDE_QUOTE)    \
	-e $(_SDE_QUOTE)s/TMFL_CPU_IS_DSPVD_VALUE/$(strip $(TMFL_CPU_IS_DSPVD))/g$(_SDE_QUOTE)    \
	-e $(_SDE_QUOTE)s/TMFL_CPU_IS_TM_VALUE/$(strip $(TMFL_CPU_IS_TM))/g$(_SDE_QUOTE)        \
	-e $(_SDE_QUOTE)s/TMFL_CPU_IS_X86_VALUE/$(strip $(TMFL_CPU_IS_X86))/g$(_SDE_QUOTE)      \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_BTM_VALUE/$(strip $(TMFL_OS_IS_BTM))/g$(_SDE_QUOTE)        \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_CE_VALUE/$(strip $(TMFL_OS_IS_CE))/g$(_SDE_QUOTE)          \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_CEXEC_VALUE/$(strip $(TMFL_OS_IS_CEXEC))/g$(_SDE_QUOTE)    \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_ECOS_VALUE/$(strip $(TMFL_OS_IS_ECOS))/g$(_SDE_QUOTE)      \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_INTEGRITY_VALUE/$(strip $(TMFL_OS_IS_INTEGRITY))/g$(_SDE_QUOTE) \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_LINUX_VALUE/$(strip $(TMFL_OS_IS_LINUX))/g$(_SDE_QUOTE)    \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_MTOS_VALUE/$(strip $(TMFL_OS_IS_MTOS))/g$(_SDE_QUOTE)      \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_NT_VALUE/$(strip $(TMFL_OS_IS_NT))/g$(_SDE_QUOTE)          \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_NUCLEUS_VALUE/$(strip $(TMFL_OS_IS_NUCLEUS))/g$(_SDE_QUOTE) \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_NULLOS_VALUE/$(strip $(TMFL_OS_IS_NULLOS))/g$(_SDE_QUOTE)  \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_OSCAN_VALUE/$(strip $(TMFL_OS_IS_OSCAN))/g$(_SDE_QUOTE)    \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_PSOS_VALUE/$(strip $(TMFL_OS_IS_PSOS))/g$(_SDE_QUOTE)      \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_UCOS_VALUE/$(strip $(TMFL_OS_IS_UCOS))/g$(_SDE_QUOTE)      \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_VXWORKS_VALUE/$(strip $(TMFL_OS_IS_VXWORKS))/g$(_SDE_QUOTE) \
	-e $(_SDE_QUOTE)s/TMFL_TOOLCHAIN_NCSC_VALUE/$(strip $(TMFL_TOOLCHAIN_NCSC))/g$(_SDE_QUOTE) \
	-e $(_SDE_QUOTE)s/TMFL_TOOLCHAIN_OSCI_VALUE/$(strip $(TMFL_TOOLCHAIN_OSCI))/g$(_SDE_QUOTE) \
	-e $(_SDE_QUOTE)s/TMFL_TOOLCHAIN_DDK_VALUE/$(strip $(TMFL_TOOLCHAIN_DDK))/g$(_SDE_QUOTE) \
	-e $(_SDE_QUOTE)s!_DVP_COMMON_TOOLS_TMFLAGS_MAK_MAK_!$(strip $(_DVP_COMMON_TOOLS_TMFLAGS_MAK_MAK_))!g$(_SDE_QUOTE) \
	-e $(_SDE_QUOTE)/^[ \t]*\#/s/$$/ \*\//$(_SDE_QUOTE)                                     \
	-e $(_SDE_QUOTE)/^[ \t]*\#/s/^[ \t]*\#/\/\* /$(_SDE_QUOTE)                              \
	-e $(_SDE_QUOTE)/^[ \t]*TMFL/s/^/\#define/g$(_SDE_QUOTE)                                \
	-e $(_SDE_QUOTE)/startifstmt/s/.*/\#ifndef _TMFLAGS_H_SEEN_/g$(_SDE_QUOTE)              \
	-e $(_SDE_QUOTE)/definitionstmt/s/.*/\#define _TMFLAGS_H_SEEN_/g$(_SDE_QUOTE)           \
	-e $(_SDE_QUOTE)s/tmFlags.mk/tmFlags.h/g$(_SDE_QUOTE)                                   \
	-e $(_SDE_QUOTE)s/\:\=//g$(_SDE_QUOTE)  > $@
	@($(ECHO) $(_SDE_QUOTE)/* XXFL_MODE_KERNEL placeholder */$(_SDE_QUOTE); \
	  $(ECHO) $(_SDE_QUOTE)#if TMFL_MODE_KERNEL$(_SDE_QUOTE); \
	  $(ECHO) $(_SDE_QUOTE)#define TMFL_MODE_USER 0$(_SDE_QUOTE); \
	  $(ECHO) $(_SDE_QUOTE)#else$(_SDE_QUOTE); \
	  $(ECHO) $(_SDE_QUOTE)#define TMFL_MODE_USER 1$(_SDE_QUOTE); \
	  $(ECHO) $(_SDE_QUOTE)#endif$(_SDE_QUOTE)) >> $@
	@$(CAT) $(DIR_SDE)/tmflags/tmFlags_values.mk | $(SED)            \
	-e $(_SDE_QUOTE)/^[ \t]*\#/s/$$/\*\//$(_SDE_QUOTE)                                      \
	-e $(_SDE_QUOTE)/^[ \t]*\#/s/^[ \t]*\#/\/\* /$(_SDE_QUOTE)                              \
	-e $(_SDE_QUOTE)/^[ \t]*TMFL/s/^/\#define/g$(_SDE_QUOTE)                                \
	-e $(_SDE_QUOTE)/endofconstants/s/.*/\#endif \/\* \#ifndef _TMFLAGS_H_SEEN \*\//g$(_SDE_QUOTE) \
	-e $(_SDE_QUOTE)s/\:\=//g$(_SDE_QUOTE)	 >> $@

$(_PHFLAGS_H_):  $(_TMFLAGS_H_) $(_TMFLAGS_H_DEPEND_)
	@$(RM-F) $@
	-@$(MKDIR) $(_TMCFGPATH) $(_TMINCPATH)
	@$(ECHO) generating $(_PHFLAGS_H_) ;\
	$(SED) -e $(_SDE_QUOTE)s/TMFL/PHFL/g$(_SDE_QUOTE) -e $(_SDE_QUOTE)s/tmFlags/phFlags/g$(_SDE_QUOTE) \
	-e $(_SDE_QUOTE)s/.*XXFL_MODE_KERNEL placeholder.*/#define  PHFL_MODE_KERNEL TMFL_MODE_KERNEL/$(_SDE_QUOTE) < $(_TMFLAGS_H_) > $@

endif  # ifdef _INCLUDE_FLAGS_MAK_FILES

GLOBAL_DEPENDENTS +=    $(_TMFLAGS_H_) $(_PHFLAGS_H_)

#------------------------------------------------------------------------------
# Here we rebuild tmFlags.mk
# 
# Note that the rebuild won't take place until the whole makefile tree
# has been read in, so it's important that all the TMFL_xxx build flags
# be correctly defined by now even at read time.

_TMFLAGS_MAK_DEPEND_ := $(GLOBAL_DEPENDENTS)

ifdef _INCLUDE_FLAGS_MAK_FILES

$(_TMFLAGS_MAK_):       $(_TMFLAGS_MAK_DEPEND_)
	@$(RM-F) $@
	-@$(MKDIR) $(_TMCFGPATH) $(_TMINCPATH)
	@$(ECHO) Making $(_TMFLAGS_MAK_);			\
	$(CAT)  $(DIR_SDE)/tmflags/tmFlags_template.mk |		\
	$(SED) -e $(_SDE_QUOTE)s/TMFL_BUILD_VERSION_VALUE/$(strip $(TMFL_BUILD_VERSION))/g$(_SDE_QUOTE) \
	-e $(_SDE_QUOTE)s/TMFL_CPU_VALUE/$(strip $(TMFL_CPU))/g$(_SDE_QUOTE)                    \
	-e $(_SDE_QUOTE)s/TMFL_ENDIAN_VALUE/$(strip $(TMFL_ENDIAN))/g$(_SDE_QUOTE)              \
	-e $(_SDE_QUOTE)s/TMFL_OS_VALUE/$(strip $(TMFL_OS))/g$(_SDE_QUOTE)                      \
	-e $(_SDE_QUOTE)s/TMFL_CPU_IS_8051_VALUE/$(strip $(TMFL_CPU_IS_8051))/g$(_SDE_QUOTE)    \
	-e $(_SDE_QUOTE)s/TMFL_CPU_IS_ARM_VALUE/$(strip $(TMFL_CPU_IS_ARM))/g$(_SDE_QUOTE)      \
	-e $(_SDE_QUOTE)s/TMFL_CPU_IS_HP_VALUE/$(strip $(TMFL_CPU_IS_HP))/g$(_SDE_QUOTE)        \
	-e $(_SDE_QUOTE)s/TMFL_CPU_IS_MIPS_VALUE/$(strip $(TMFL_CPU_IS_MIPS))/g$(_SDE_QUOTE)    \
	-e $(_SDE_QUOTE)s/TMFL_CPU_IS_DSPRD_VALUE/$(strip $(TMFL_CPU_IS_DSPRD))/g$(_SDE_QUOTE)    \
	-e $(_SDE_QUOTE)s/TMFL_CPU_IS_DSPVD_VALUE/$(strip $(TMFL_CPU_IS_DSPVD))/g$(_SDE_QUOTE)    \
	-e $(_SDE_QUOTE)s/TMFL_CPU_IS_TM_VALUE/$(strip $(TMFL_CPU_IS_TM))/g$(_SDE_QUOTE)        \
	-e $(_SDE_QUOTE)s/TMFL_CPU_IS_X86_VALUE/$(strip $(TMFL_CPU_IS_X86))/g$(_SDE_QUOTE)      \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_BTM_VALUE/$(strip $(TMFL_OS_IS_BTM))/g$(_SDE_QUOTE)        \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_CE_VALUE/$(strip $(TMFL_OS_IS_CE))/g$(_SDE_QUOTE)          \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_CEXEC_VALUE/$(strip $(TMFL_OS_IS_CEXEC))/g$(_SDE_QUOTE)    \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_ECOS_VALUE/$(strip $(TMFL_OS_IS_ECOS))/g$(_SDE_QUOTE)      \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_INTEGRITY_VALUE/$(strip $(TMFL_OS_IS_INTEGRITY))/g$(_SDE_QUOTE) \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_LINUX_VALUE/$(strip $(TMFL_OS_IS_LINUX))/g$(_SDE_QUOTE)    \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_MTOS_VALUE/$(strip $(TMFL_OS_IS_MTOS))/g$(_SDE_QUOTE)      \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_NT_VALUE/$(strip $(TMFL_OS_IS_NT))/g$(_SDE_QUOTE)          \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_NUCLEUS_VALUE/$(strip $(TMFL_OS_IS_NUCLEUS))/g$(_SDE_QUOTE) \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_NULLOS_VALUE/$(strip $(TMFL_OS_IS_NULLOS))/g$(_SDE_QUOTE)	\
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_OSCAN_VALUE/$(strip $(TMFL_OS_IS_OSCAN))/g$(_SDE_QUOTE)    \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_PSOS_VALUE/$(strip $(TMFL_OS_IS_PSOS))/g$(_SDE_QUOTE)      \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_UCOS_VALUE/$(strip $(TMFL_OS_IS_UCOS))/g$(_SDE_QUOTE)      \
	-e $(_SDE_QUOTE)s/TMFL_OS_IS_VXWORKS_VALUE/$(strip $(TMFL_OS_IS_VXWORKS))/g$(_SDE_QUOTE) \
	-e $(_SDE_QUOTE)s/TMFL_TOOLCHAIN_NCSC_VALUE/$(strip $(TMFL_TOOLCHAIN_NCSC))/g$(_SDE_QUOTE) \
	-e $(_SDE_QUOTE)s/TMFL_TOOLCHAIN_OSCI_VALUE/$(strip $(TMFL_TOOLCHAIN_OSCI))/g$(_SDE_QUOTE) \
	-e $(_SDE_QUOTE)s/TMFL_TOOLCHAIN_DDK_VALUE/$(strip $(TMFL_TOOLCHAIN_DDK))/g$(_SDE_QUOTE) \
	-e $(_SDE_QUOTE)s!_DVP_COMMON_TOOLS_TMFLAGS_MAK_MAK_!$(strip $(_DVP_COMMON_TOOLS_TMFLAGS_MAK_MAK_))!g$(_SDE_QUOTE) > $@ ;\
	$(CAT)  $(DIR_SDE)/tmflags/tmFlags_values.mk >> $@

$(_PHFLAGS_MAK_): $(_TMFLAGS_MAK_) $(_TMFLAGS_MAK_DEPEND_)
	@$(RM-F) $@
	-@$(MKDIR) $(_TMCFGPATH) $(_TMINCPATH)
	@$(ECHO) Making $(_PHFLAGS_MAK_)	;\
	$(CAT)  $(_TMFLAGS_MAK_)|$(SED) -e $(_SDE_QUOTE)s/TMFL/PHFL/g$(_SDE_QUOTE) -e $(_SDE_QUOTE)s/tmFlags/phFlags/g$(_SDE_QUOTE) > $@ ;\

endif # ifdef _INCLUDE_FLAGS_MAK_FILES

GLOBAL_DEPENDENTS +=    $(_TMFLAGS_MAK_) $(_PHFLAGS_MAK_)


#------------------------------------------------------------------------------
# If clean, delete the 3 files.

.PHONY: _CLEAN_TMFLAGS_

clean_tmflags:  _CLEAN_TMFLAGS_

_CLEAN_TMFLAGS_: FORCE
	$(_SDE_ECHO)$(RM-F) $(_TMFLAGS_H_) $(_TMFLAGS_MAK_) $(_PHFLAGS_H_) $(_PHFLAGS_MAK_)


#==============================================================================
# Report any errors so far

ifneq           ($(_ERROR_), )
_ERROR_ := *** !!! Error in $(_DVP_COMMON_TOOLS_TMFLAGS_MAK_MAK_):\n$(_ERROR_)

%:      FORCE
	$(_SDE_ECHO)$(ECHO) $(ECHOFLAGS) "$(_ERROR_)"
	$(_SDE_ECHO) exit 1
	quit
endif  # ifneq  ($(_ERROR_), )

FORCE:

