#!/bin/sh

# Try to mount USB storage devices with type vfat - usually a USB memory stick,
# if that fails then try mounting with type ext3 or ext2 - usually a hard disk.

# Location of all auto-mounted devices
mount_base="/media"

# Location of auto-mount information
mountinfo_base="${mount_base}/automount_info"
mountinfo_media="${mountinfo_base}/media"
mountinfo_device="${mountinfo_base}/device"
mountinfo_file="${mountinfo_device}/$MDEV"

# Empty for read/write, "-r" for read only
# Fixme: read only should probably be the default.
RDWR=

# Log stdout and stderr
exec 1>/tmp/automount.log
exec 2>&1
set -x

# Work out next available auto mount location
getmountpoint()
{
	if [ ! -e "${1}/disk" ]
	then
		echo "disk"
		return
	fi

	n=1
	while [ $n -lt 256 ]
	do
		if [ ! -e "${1}/disk-${n}" ]
		then
			echo "disk-${n}"
			return
		fi
		let n=n+1
	done
}

if [ "$ACTION" = remove ]
then
	for fs in vfat ext3 ext2
	do
		if [ -n "$(mount -t $fs | grep $MDEV)" ]
		then
			# Execute filesystem specific unmount script if present
			# Fixme: shouldn't "${mount_base}/${mountpoint}" be passed to the unmount script ?
			[ -x /sbin/umount.${fs} ] && /sbin/umount.${fs}

			mountpoint="$(cat $mountinfo_file)"
			umount -f "${mount_base}/${mountpoint}"
			rm -rf "${mount_base}/${mountpoint}" $mountinfo_file "${mountinfo_media}/${mountpoint}"

			# Unmount succeeded
			echo "==> Unmounted $MDEV (${mount_base}/${mountpoint})" >/dev/console
			exit 0
		fi
	done
else
	mkdir -p $mountinfo_device $mountinfo_media

	# Get the next available mount point
	mountpoint=$(getmountpoint $mountinfo_media)

	mkdir -p "${mount_base}/${mountpoint}"

	for fs in vfat ext3 ext2
	do
		if mount -t $fs ${RDWR} -noatime $MDEV "${mount_base}/${mountpoint}"
		then
			# Execute filesystem specific mount script if present
			[ -x /sbin/mount.${fs} ] && /sbin/mount.${fs} "${mount_base}/${mountpoint}"

			echo $mountpoint > "$mountinfo_file"
			echo $MDEV > "${mountinfo_media}/${mountpoint}"

			# Mount succeeded
			echo "==> Mounted $MDEV on ${mount_base}/${mountpoint}" >/dev/console
			exit 0
		fi
	done

	# Mount attempts failed... remove mount directory
	rm -rf "${mount_base}/${mountpoint}"
fi

