#!/bin/sh
# -*- backup-inhibited: t -*-

[% IF !CONFIG_DVB_MODULES -%]
  [% SKIPFILE %]
[% END -%]


num_hw_frontend_demux=[%CONFIG_NUM_HW_FRONTEND_DEMUXES%]
num_hw_memory_demux=$(([%CONFIG_NUM_HW_DEMUXES%]-[%CONFIG_NUM_HW_FRONTEND_DEMUXES%]))

case ${num_hw_frontend_demux} in
    "0")
       hw_demux_frontend_mask=0x0
       ;;
    "1")
       hw_demux_frontend_mask=0x1
       ;;
    "2")
       hw_demux_frontend_mask=0x3
       ;;
    *)
       echo "Invalid HW frontend demux number"
       exit 1
       ;;
esac

case ${num_hw_memory_demux} in
    "0")
       hw_demux_memory_mask=0x0
       ;;
    "1")
       hw_demux_memory_mask=0x4
       ;;
    "2")
       hw_demux_memory_mask=0xc
       ;;
    *)
       echo "Invalid HW demux number"
       exit 1
       ;;
esac

hw_demux_mask=$((hw_demux_frontend_mask|hw_demux_memory_mask))

# PVR demuxes number
[% IF !CONFIG_PVR_RECORD -%]
    pvr_demux_mask=0x0
[% ELSE %]
    pvr_demux_mask=${hw_demux_frontend_mask}
[% END -%]

# HW demuxes number
    opts_phStbDemux833x="enabled=${hw_demux_mask} pvr=${pvr_demux_mask}"

# TS inputs internal routing
    opts_phStbDemux833x="$opts_phStbDemux833x TS0_routing=[%CONFIG_TS0_ROUTING%] TS1_routing=[%CONFIG_TS1_ROUTING%] TS2_routing=[%CONFIG_TS2_ROUTING%]"


case "$1" in
    start)
    # Stop the loading messages appearing
    #cat /proc/sys/kernel/printk > /tmp/default_log_level
    kernelRelease=`uname -r`
    d=/lib/modules/$kernelRelease/kernel/drivers/media/dvb
    f=/lib/modules/$kernelRelease/kernel/drivers/media/dvb/frontends
    x=/lib/modules/$kernelRelease/extra

    # Load DVB modules if available. This will load support for DVB, tuners and HW demux.
[% IF CONFIG_DVB_MODULES -%]
    vecho -n "Loading DVB modules..."
    opts_tda10048="init_on_startup=1"
    opts_phStbDemux="demod0=[% CONFIG_TS0_demod %] tuner0=[% CONFIG_TS0_tuner %] demod1=[% CONFIG_TS1_demod %] tuner1=[% CONFIG_TS1_tuner %]"

    for m in tda1004x tda10048 tda10086 tc90507xbg mb86a20s; do
       if [ -f $f/$m.ko ]; then
          eval insmod $f/$m.ko \$opts_$m
       fi
    done

    if [ -f $d/dvb-core/dvb-core.ko ]; then
       eval insmod $d/dvb-core/dvb-core.ko
    fi
    
    if [ -f $d/nxp/phStbDemux.ko ]; then
       eval insmod $d/nxp/phStbDemux.ko \$opts_phStbDemux
    fi

[% IF CONFIG_TDHL5_T -%]
    if [ -f $n/tdhl5_sam.ko ]; then
       mknod /dev/tdhl5 c 210 1
       insmod $n/tdhl5_sam.ko
    fi
[% END -%]
    if [ -f $x/phStbDemux833x.ko ]; then
       eval insmod $x/phStbDemux833x.ko \$opts_phStbDemux833x
    fi
[% IF CONFIG_COMMON_INTERFACE_PRESENT -%]
    # Common Interface GPIOs assignments
    opts_phStbCommonInterface="gpio_reset_pin=[% CONFIG_COMMON_INTERFACE_RESET_GPIO_PIN %] gpio_detect_pin=[% CONFIG_COMMON_INTERFACE_DETECT_GPIO_PIN %] gpio_bypass_pin=[% CONFIG_COMMON_INTERFACE_BYPASS_GPIO_PIN %] gpio_memory_pin=[% CONFIG_COMMON_INTERFACE_MEMORY_GPIO_PIN %]"

    if [ -f $x/phStbCommonInterface.ko ]; then
        eval insmod $x/phStbCommonInterface.ko \$opts_phStbCommonInterface
    fi
[% END -%]
    vecho "done"
[% END -%]

    #cat /tmp/default_log_level > /proc/sys/kernel/printk
    #rm -f /tmp/default_log_level
    ;;
    stop)
    vecho "Cannot unload DVB modules!"
    ;;
    *)
    echo "Usage: $0 {start|stop}"
    exit 1
    ;;
esac
