#ifndef PHSTBSTREAMINGSYSTEM_RESOURCE_H    /* Multi-include protection.   <FILENAME>_H */
#define PHSTBSTREAMINGSYSTEM_RESOURCE_H

/*****************************************************************************/
/* Copyright (c) 2009 NXP B.V.  All rights are reserved.                     */
/*                                                                           */
/* Redistribution and use in source and binary forms, with or without        */
/* modification, are permitted provided that the following conditions        */
/* are met:                                                                  */
/*                                                                           */
/* Redistributions of source code must retain the above copyright            */
/* notice, this list of conditions and the following disclaimer.             */
/*                                                                           */
/* Redistributions in binary form must reproduce the above copyright         */
/* notice, this list of conditions and the following disclaimer in the       */
/* documentation and/or other materials provided with the distribution.      */
/*                                                                           */
/* Neither the name of NXP nor the names of its                              */
/* contributors may be used to endorse or promote products derived from      */
/* this software without specific prior written permission.                  */
/*                                                                           */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       */
/* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         */
/* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     */
/* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      */
/* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,     */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT          */
/* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,     */
/* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY     */
/* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT       */
/* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE         */
/* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH          */
/* DAMAGE.                                                                   */
/*                                                                           */
/*****************************************************************************/
/**
    @file   phStbStreamingSystem_Resource.h
    @brief  Interface for a streaming resource

@b  Component:  phStbStreamingSystem_Resource.

    This header file is the interface of the Resource class from the
    Streaming System detailed design. It contains only the externally
    used functions.
*//*

Rev Date        Author      Comments
--------------------------------------------------------------------------------
  1 20050531    J Grant     Skeleton
  2 20050617    burningh    Adding implementation
  3 20050621    burningh    Adding User Data to StreamingComponent interface
  4 20060707    burningh    Correcting constness in prototype
  5 20060124    batelaan    Add extern "C" for use with C++
  6 20080625    burningh    Update license text
  7 20090120    batelaan    Update file copyright/license text
--------------------------------------------------------------------------------
    For consistency and standardisation retain the Section Separators.
*/

/*******************
* INCLUDE FILES    *
********************/
#include <tmNxCompId.h>

#include "phStbStreamingSystem_InstanceTypes.h"
#include "phStbStreamingSystem_StreamingComponent.h"


#ifdef __cplusplus
extern "C"
{
#endif

/**************************************************************************
* EXPORTED MACROS         PH[<LNX|TM|NONE>]_IPSTBSTREAMINGSYSTEM_RESOURCE_<_WORD>+        *
***************************************************************************/

/*********************************************************************
* EXPORTED TYPEDEFS       [p]phStbStreamingSystem_Resource_<Word>+_t  *
**********************************************************************/

/**************************************************************************
* EXPORTED DATA      g[k|p|kp|pk|kpk]phStbStreamingSystem_Resource_<Word>+ *
***************************************************************************/

/****************************************************************************
* EXPORTED FUNCTION PROTOTYPES     phStbStreamingSystem_Resource[_<Word>+]   *
*****************************************************************************/

/* Class Operations */

/******************************************************************************/
/** Create new resource instance
    Constraints:
        None

    @param pClient          IN  Client interface to corresponding streaming
                                component

    @param pName            IN  Unique textual name of resource
    @param sourcePinCount   IN  Number of source pins for the resource
    @param ppSourcePins     IN  Array of pointers to all source pins for the
                                resource
    @param sinkPinCount     IN  Number of sink pins for the resource
    @param ppSinkPins       IN  Array of pointers to all sink pins for the
                                resource
    @param pUserData        IN  The user data that is passed on each call to
                                the StreamingComponent interface
    @param ppNewResource    OUT Newly created object

    @retval TM_OK           Success
    @retval AnotherValue    Error occured.
*/
tmErrorCode_t phStbStreamingSystem_Resource_Create
    (phStbStreamingSystem_StreamingComponent_t  pClient,
     const char*                                pName,
     int                                        sourcePinCount,
     phStbStreamingSystem_SourcePin_t**         ppSourcePins,
     int                                        sinkPinCount,
     phStbStreamingSystem_SinkPin_t**           ppSinkPins,
     void*                                      pUserData,
     phStbStreamingSystem_Resource_t**          ppNewResource);


/* Instance Operations */

/** Destroy resource instance
    Constraints:
        None

    @param pResource    IN  This resource

    @retval TM_OK           Success
    @retval AnotherValue    Error occured.
*/
tmErrorCode_t phStbStreamingSystem_Resource_Destroy
    (phStbStreamingSystem_Resource_t* pResource);


/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/

#ifdef __cplusplus
}
#endif

#endif /* PHSTBSTREAMINGSYSTEM_RESOURCE_H      Do not add any thing below this line */
