#ifndef PHSTBSTREAMINGSYSTEM_TSSASOURCEPIN_H    /* Multi-include protection.   <FILENAME>_H */
#define PHSTBSTREAMINGSYSTEM_TSSASOURCEPIN_H

/*****************************************************************************/
/* Copyright (c) 2009 NXP B.V.  All rights are reserved.                     */
/*                                                                           */
/* Redistribution and use in source and binary forms, with or without        */
/* modification, are permitted provided that the following conditions        */
/* are met:                                                                  */
/*                                                                           */
/* Redistributions of source code must retain the above copyright            */
/* notice, this list of conditions and the following disclaimer.             */
/*                                                                           */
/* Redistributions in binary form must reproduce the above copyright         */
/* notice, this list of conditions and the following disclaimer in the       */
/* documentation and/or other materials provided with the distribution.      */
/*                                                                           */
/* Neither the name of NXP nor the names of its                              */
/* contributors may be used to endorse or promote products derived from      */
/* this software without specific prior written permission.                  */
/*                                                                           */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       */
/* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         */
/* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     */
/* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      */
/* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,     */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT          */
/* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,     */
/* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY     */
/* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT       */
/* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE         */
/* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH          */
/* DAMAGE.                                                                   */
/*                                                                           */
/*****************************************************************************/
/**
    @file   phStbStreamingSystem_TSSASourcePin.h
    @brief  Interface for a TSSA Source Pin.

@b  Component:  phStbStreamingSystem.

    This header file is the interface of the TSSASourcePin class from the
    Streaming System detailed design.
*//*

Rev Date        Author      Comments
--------------------------------------------------------------------------------
  1 20050610    J Grant     Skeleton
  2 20060124    batelaan    Add extern "C" for use with C++
  3 20071217    burningh    Improve documentation of top-level APIs
  4 20080626    burningh    Update license text
  5 20090120    batelaan    Update file copyright/license text
--------------------------------------------------------------------------------
    For consistency and standardisation retain the Section Separators.
*/

/*******************
* INCLUDE FILES    *
********************/

#include <tmAvFormats.h>
#include <tsaDefault.h>

#include "phStbStreamingSystem_InstanceTypes.h"


#ifdef __cplusplus
extern "C"
{
#endif

/**************************************************************************
* EXPORTED MACROS         PH[<LNX|TM|NONE>]_IPSTBSTREAMINGSYSTEM_TSSASOURCEPIN_<_WORD>+        *
***************************************************************************/

/*********************************************************************
* EXPORTED TYPEDEFS       [p]ph[<lnx|tm|NONE>]StbStreamingSystem_TSSASourcePin_<Word>+_t  *
**********************************************************************/

/**************************************************************************
* EXPORTED DATA      g[k|p|kp|pk|kpk]ph[<lnx|tm|NONE>]StbStreamingSystem_TSSASourcePin_<Word>+ *
***************************************************************************/

/****************************************************************************
* EXPORTED FUNCTION PROTOTYPES     ph[<lnx|tm|NONE>]StbStreamingSystem_TSSASourcePin[_<Word>+]   *
*****************************************************************************/

/* Class Operations */

/******************************************************************************/
/** Create new TSSASourcePin instance
    Constraints:
        Specified TSSA pin must not be controlled outside of this new object
        All TSSA information (pDefaultCapabilities, outputPinID, pFormat) must
            be valid for the corresponding TSSA component.


    @param pDefaultCapabilities IN  Default capabilities of corresponding TSSA
                                    component
    @param outputPinID          IN  Index of TSSA output pin that will
                                    correspond to the new TSSASourcePin
                                    instance
    @param pFormat              IN  Format of TSSA packets that are output from
                                    the corresponding TSSA pin

    @param ppSourcePin          OUT Newly created object



    @retval TM_OK           Success
    @retval AnotherValue    Error occured.
*/
tmErrorCode_t phStbStreamingSystem_TSSASourcePin_Create
    (/* ALInstance */   /* TBD: this is in the design but not needed */
     ptsaDefaultCapabilities_t  pDefaultCapabilities,
     int                        outputPinID,
     ptmAvFormat_t              pFormat,
     phStbStreamingSystem_SourcePin_t** ppSourcePin);



/* Instance Operations */

/* None */


/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/

#ifdef __cplusplus
}
#endif

#endif /* PHSTBSTREAMINGSYSTEM_TSSASOURCEPIN_H      Do not add any thing below this line */
