#ifndef PHSTBVIDEORENDERER_ERRORS_H    /* Multi-include protection. */
#define PHSTBVIDEORENDERER_ERRORS_H    /* Place here to get around a Qmore bug !   */

/*****************************************************************************/
/* Copyright (c) 2009 NXP B.V.  All rights are reserved.                     */
/*                                                                           */
/* Redistribution and use in source and binary forms, with or without        */
/* modification, are permitted provided that the following conditions        */
/* are met:                                                                  */
/*                                                                           */
/* Redistributions of source code must retain the above copyright            */
/* notice, this list of conditions and the following disclaimer.             */
/*                                                                           */
/* Redistributions in binary form must reproduce the above copyright         */
/* notice, this list of conditions and the following disclaimer in the       */
/* documentation and/or other materials provided with the distribution.      */
/*                                                                           */
/* Neither the name of NXP nor the names of its                              */
/* contributors may be used to endorse or promote products derived from      */
/* this software without specific prior written permission.                  */
/*                                                                           */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       */
/* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         */
/* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     */
/* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      */
/* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,     */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT          */
/* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,     */
/* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY     */
/* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT       */
/* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE         */
/* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH          */
/* DAMAGE.                                                                   */
/*                                                                           */
/*****************************************************************************/
/**
    @file   phStbVideoRenderer_Errors.h
    @brief  Video Renderer Component

    Error codes which can be returned by the Video Renderer.

    Set your editor for 4 space indentation.
*//*

Rev Date        Author      Comments
--------------------------------------------------------------------------------
  1 20061016    burningh    Original
  2 20061031    burningh    Continuing implementation
  3 20061115    burningh    Adding BAD_STATE error code
  4 20071123    burningh    Improving documentation of top level API
  5 20090120    batelaan    Update file copyright/license text
--------------------------------------------------------------------------------
    For consistency and standardisation retain the Section Separators.
*/

/**************************************************************
* INCLUDE FILES                                               *
***************************************************************/

#include <tmNxCompId.h>

#ifdef __cplusplus
extern "C"
{
#endif

/**************************************************************
* EXPORTED MACROS                                             *
***************************************************************/

/** Temporary workaround for missing CID. */
#define CID_STBVIDEORENDERER 0x40440000

/** Base of Video Renderer error codes. */
#define PH_STBVIDEORENDERER_ERR_BASE  CID_STBVIDEORENDERER

/** Base of unique Video Renderer error codes. */
#define PH_STBVIDEORENDERER_ERR_UNIQUE_BASE  (CID_STBVIDEORENDERER | TM_ERR_COMP_UNIQUE_START)

/* Standard Error Codes */

/** A bad instance handle was passed; either NULL, or the magic number was missing. */
#define PH_STBVIDEORENDERER_ERR_BAD_INSTANCE       (PH_STBVIDEORENDERER_ERR_UNIQUE_BASE | 1U)

/** A bad out parameter was passed; either NULL or otherwise invalid. */
#define PH_STBVIDEORENDERER_ERR_BAD_OUT_PARAM      (PH_STBVIDEORENDERER_ERR_UNIQUE_BASE | 2U)

/** The component was in a bad state. */
#define PH_STBVIDEORENDERER_ERR_BAD_STATE          (PH_STBVIDEORENDERER_ERR_UNIQUE_BASE | 3U)

/** A NULL in parameter was passed. */
#define PH_STBVIDEORENDERER_ERR_NULL_IN_PARAM      (PH_STBVIDEORENDERER_ERR_UNIQUE_BASE | 4U)

/** The requested command was not supported. */
#define PH_STBVIDEORENDERER_ERR_NOT_SUPPORTED      (PH_STBVIDEORENDERER_ERR_UNIQUE_BASE | 5U)

/** The provided rectangle was invalid. */
#define PH_STBVIDEORENDERER_ERR_INVALID_RECT       (PH_STBVIDEORENDERER_ERR_UNIQUE_BASE | 6U)

/** The component is out of memory while performing the requested operation. */
#define PH_STBVIDEORENDERER_ERR_OUT_OF_MEMORY      (PH_STBVIDEORENDERER_ERR_UNIQUE_BASE | 7U)

/** The requested layer was out of range. */
#define PH_STBVIDEORENDERER_ERR_LAYER_OUT_OF_RANGE (PH_STBVIDEORENDERER_ERR_UNIQUE_BASE | 8U)

/** The requested unit was out of range. */
#define PH_STBVIDEORENDERER_ERR_UNIT_OUT_OF_RANGE  (PH_STBVIDEORENDERER_ERR_UNIQUE_BASE | 9U)

/** The provided rectangle was invalid. */
#define PH_STBVIDEORENDERER_ERR_BAD_RECTANGLE      (PH_STBVIDEORENDERER_ERR_UNIQUE_BASE | 10U)

/** The requested level was out of range. */
#define PH_STBVIDEORENDERER_ERR_LEVEL_OUT_OF_RANGE (PH_STBVIDEORENDERER_ERR_UNIQUE_BASE | 11U)

/** The requested path was out of range. */
#define PH_STBVIDEORENDERER_ERR_PATH_OUT_OF_RANGE  (PH_STBVIDEORENDERER_ERR_UNIQUE_BASE | 12U)

/** The requested format was unknown. */
#define PH_STBVIDEORENDERER_ERR_UNKNOWN_FORMAT     (PH_STBVIDEORENDERER_ERR_UNIQUE_BASE | 13U)

/** The requested chain was out of range. */
#define PH_STBVIDEORENDERER_ERR_CHAIN_OUT_OF_RANGE (PH_STBVIDEORENDERER_ERR_UNIQUE_BASE | 14U)

/** An internal error occurred while performing the requested operation. */
#define PH_STBVIDEORENDERER_ERR_INTERNAL           (PH_STBVIDEORENDERER_ERR_UNIQUE_BASE | 15U)


/**************************************************************
* EXPORTED TYPEDEFS                                           *
***************************************************************/

/**************************************************************
* EXPORTED DATA                                               *
***************************************************************/

/**************************************************************
* EXPORTED FUNCTION PROTOTYPES                                *
***************************************************************/

#ifdef __cplusplus
}
#endif


#endif /* Do not add any thing below this line */

